﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class Administracio : PhoneApplicationPage
    {
        private bool administrador;
        private ServeiWeb.ServeiClient servei;

        public Administracio()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'arriba al formulari.
         * Inicia els botons per administradors amb permisos
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            //Si s'arriba des de el menu principal
            if (e.NavigationMode != NavigationMode.Back)
            {
                administrador = Convert.ToBoolean(NavigationContext.QueryString["administrador"]);
                //Nomes els administradors poden realitzar modificacions
                if (administrador)
                {
                    botoAfegirUsuari.IsEnabled = true;
                    botoModificarUsuari.IsEnabled = true;
                    botoHabilitarUsuari.IsEnabled = true;
                    botoDeshabilitarUsuari.IsEnabled = true;
                    botoAfegirCarrec.IsEnabled = true;
                    botoModificarCarrec.IsEnabled = true;
                    botoHabilitarCarrec.IsEnabled = true;
                    botoDeshabilitarCarrec.IsEnabled = true;
                    botoAfegirTipus.IsEnabled = true;
                    botoModificarTipus.IsEnabled = true;
                    botoHabilitarTipus.IsEnabled = true;
                    botoDeshabilitarTipus.IsEnabled = true;
                }
            }
            //Sempre que s'arriba a aquesta pagina s'actualitzen les dades dels llistats
            carregarUsuaris();
            carregarCarrecs();
            carregarTipus();
        }
        #endregion

        #region Carregar usuaris
        /*
         * Metode que realitza una crida asíncrona al servei web per buscar els usuaris existents
         */
        private void carregarUsuaris()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarUsuarisCompleted += serveiLlistarUsuarisCompleted;
            servei.llistarUsuarisAsync();
        }

        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir tots els usuaris existents
         */
        private void serveiLlistarUsuarisCompleted(object sender, ServeiWeb.llistarUsuarisCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
            }
            //Si hi han usuaris, s'afegeixen al llistat
            else if (e.Result != null && e.Result.Count > 0)
            {
                scrollUsuaris.Content = Usuari.llistaGridUsuaris(e.Result);
            }
        }
        #endregion

        #region Carregar carrecs
        /*
         * Metode que realitza una crida asíncrona al servei per buscar els carrecs existents
         */
        private void carregarCarrecs()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarCarrecsCompleted += serveiLlistaCarrecsCompleted;
            servei.llistarCarrecsAsync();
        }

        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir tots els carrecs existents
         */
        private void serveiLlistaCarrecsCompleted(object sender, ServeiWeb.llistarCarrecsCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
            }
            //Si hi han carrecs, s'afegeixen a la llista
            else if (e.Result != null && e.Result.Count > 0)
            {
                scrollCarrecs.Content = Carrec.llistaCarrecs(e.Result);
            }
        }
        #endregion

        #region Afegir adjunt
        /*
         * Event que obre el formulari per afegir nous usuaris
         */
        private void clickAfegirUsuari(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri(string.Format("/Administrar/AdministracioUsuari.xaml?usuariId=0"), UriKind.Relative));
        }
        #endregion

        #region Modificar usuari
        /*
         * Event que permet realitzar modificacions sobre un usuari seleccionat
         */
        private void clickModificarUsuari(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Usuari.usuarisSeleccionats(scrollUsuaris);
            //Si no es seleccionen usuaris no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un usuari seleccionat, es realitza una crida asíncrona al servei per habilitar l'usuari
            else if (llista.Count == 1)
            {
                NavigationService.Navigate(new Uri(string.Format("/Administrar/AdministracioUsuari.xaml?usuariId=" + llista[0]), UriKind.Relative));
            }
            //Nomes es pot modificar un usuari a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_USUARI);
            }
        }
        #endregion

        #region Habilitar usuari
        /*
         * Event que permet habilitar un usuari seleccionat
         */
        private void clickHabilitarUsuari(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Usuari.usuarisSeleccionats(scrollUsuaris);
            //Si no es seleccionen usuaris no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un usuari seleccionat, es crida asincronament el servei per habilitar l'usuari
            else if (llista.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.activarUsuariCompleted += serveiActivarUsuariCompleted;
                servei.activarUsuariAsync(llista[0]);
            }
            //Nomes es pot habilitar un usuari a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_USUARI);
            }
        }

        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per habilitar un usuari
         */
        private void serveiActivarUsuariCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.USUARI_HABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                carregarUsuaris();
            }
        }
        #endregion

        #region Deshabilitar usuari
        /*
         * Event que permet deshabilitar un usuari seleccionat
         */
        private void clickDeshabilitarUsuari(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Usuari.usuarisSeleccionats(scrollUsuaris);
            //Si no es seleccionen usuaris no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un usuari seleccionat, es crida asincronament el servei per deshabilitar l'usuari
            else if (llista.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.desactivarUsuariCompleted += serveiDesactivarUsuariCompleted;
                servei.desactivarUsuariAsync(llista[0]);
            }
            //Nomes es pot deshabilitar un usuari a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_USUARI);
            }
        }

        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per deshabilitar un usuari
         */
        private void serveiDesactivarUsuariCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.USUARI_DESHABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                carregarUsuaris();
            }
        }
        #endregion

        #region Afegir carrecs nou
        /*
         * Event que obre el formulari per afegir un nou carrec
         */
        private void clickAfegirCarrec(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri(string.Format("/Administrar/AdministracioCarrec.xaml?carrecId=0"), UriKind.Relative));
        }
        #endregion

        #region Modificar carrec existent
        /*
         * Event que permet realitzar modificacions sobre un carrec seleccionat
         */
        private void clickModificarCarrec(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Carrec.carrecsSeleccionats(scrollCarrecs);
            //Si no es seleccionen carrecs no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un usuari seleccionat, es crida el formulari per modificar carrecs enviant el seu id per parametre
            else if (llista.Count == 1)
            {
                NavigationService.Navigate(new Uri(string.Format("/Administrar/AdministracioCarrec.xaml?carrecId=" + llista[0]), UriKind.Relative));
            }
            //Nomes es pot modificar un carrec a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_CARREC);
            }
        }
        #endregion

        #region Habilitar carrec
        /*
         * Event que permet habilitar un carrec seleccionat
         */
        private void clickHabilitarCarrec(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Carrec.carrecsSeleccionats(scrollCarrecs);
            //Si no es seleccionen carrecs no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un usuari seleccionat, es crida asincronament el servei per habilitar el carrec
            else if (llista.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.habilitarCarrecCompleted += serveiHabilitarCarrecCompleted;
                servei.habilitarCarrecAsync(llista[0]);
            }
            //Nomes es pot habilitar un carrec a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_CARREC);
            }
        }
        #endregion

        #region Habilitar carrec finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per habilitar un carrec
         */
        private void serveiHabilitarCarrecCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CARREC_HABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                carregarCarrecs();
            }
        }
        #endregion

        #region Deshabilitar carrec
        /*
        * Event que permet deshabilitar un carrec seleccionat
        */
        private void clickDeshabilitarCarrec(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Carrec.carrecsSeleccionats(scrollCarrecs);
            //Si no es seleccionen carrecs no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un usuari seleccionat, es crida asincronament el servei per deshabilitar el carrec
            else if (llista.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.deshabilitarCarrecCompleted += serveiDeshabilitarCarrecCompleted;
                servei.deshabilitarCarrecAsync(llista[0]);
            }
            //Nomes es pot deshabilitar un carrec a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_CARREC);
            }
        }
        #endregion

        #region Deshabilitar carrec finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per deshabilitar un carrec
         */
        private void serveiDeshabilitarCarrecCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CARREC_HABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                carregarCarrecs();
            }
        }
        #endregion

        #region Iniciar llista de tipus
        /*
         * Metode que realitza una crida asíncrona al servei per buscar els tipus de despeses existents
         */
        private void carregarTipus()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarTipusCompleted += serveiLlistarTipusCompleted;
            servei.llistarTipusAsync();
        }
        #endregion

        #region Llista de tipus fionalitzada
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir tots els tipus dedespeses existents
         */
        private void serveiLlistarTipusCompleted(object sender, ServeiWeb.llistarTipusCompletedEventArgs e)
        {
            servei.CloseAsync();
            //Si hi han tipus de despeses,s'afegeixen al llistat
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
            }
            else if (e.Result != null && e.Result.Count > 0)
            {
                scrollTipus.Content = Tipus.llistaGridTipus(e.Result);
            }
        }
        #endregion

        #region Afegir tipus nou
        /*
         * Event que obre el formulari per afegir un nou tipus de despesa
         */
        private void clickAfegirTipus(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri(string.Format("/Administrar/AdministracioTipus.xaml?tipusId=0"), UriKind.Relative));
        }
        #endregion

        #region Modificar tipus existent
        /*
         * Event per modificar un tipus de despesa existent
         */
        private void clickModificarTipus(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Tipus.tipusSeleccionats(scrollTipus);
            //Si no es seleccionen tipus de despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un tipus de despesa seleccionat, s'obre el formulari per modificar-la enviant el id per parametre
            else if (llista.Count == 1)
            {
                NavigationService.Navigate(new Uri(string.Format("/Administrar/AdministracioTipus.xaml?tipusId=" + llista[0]), UriKind.Relative));
            }
            //Nomes es pot modificar un tipus de despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_TIPUS);
            }
        }
        #endregion

        #region Habilitar tipus
        /*
         * Event que permet habilitar un tipus de despesa seleccionat
         */
        private void clickHabilitarTipus(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Tipus.tipusSeleccionats(scrollTipus);
            //Si no es seleccionen tipus de despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un tipus de despesa seleccionat, escrida asincronament el servei per habilitar-lo
            else if (llista.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.habilitarTipusCompleted += serveiHabilitarTipusCompleted;
                servei.habilitarTipusAsync(llista[0]);
            }
            //Nomes es pot habilitar un tipus de despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_TIPUS);
            }
        }
        #endregion

        #region Habilitar tipus finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per habilitar un tipus de despesa
         */
        private void serveiHabilitarTipusCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.TIPUS_HABILITAR_ERROR);
            }
            else
            {
                carregarTipus();
            }
        }
        #endregion

        #region Deshabilitar tipus
        /*
        * Event que permet deshabilitar un tipus de despesa seleccionat
        */
        private void clickDeshabilitarTipus(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Tipus.tipusSeleccionats(scrollTipus);
            //Si no es seleccionen tipus de despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un tipus de despesa seleccionat, escrida asincronament el servei per deshabilitar-lo
            else if (llista.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.deshabilitarTipusCompleted += serveiDeshabilitarTipusCompleted;
                servei.deshabilitarTipusAsync(llista[0]);
            }
            //Nomes es pot deshabilitar un tipus de despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_TIPUS);
            }
        }
        #endregion

        #region Deshabilitar tipus finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per deshabilitar un tipus de despesa
         */
        private void serveiDeshabilitarTipusCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.TIPUS_HABILITAR_ERROR);
            }
            else
            {
                carregarTipus();
            }
        }
        #endregion
    }
}