﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class AdministracioCarrec : PhoneApplicationPage
    {
        private Int32 carrecId, permisosId;
        private ServeiWeb.ServeiClient servei;

        public AdministracioCarrec()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'arriba al formulari.
         * Inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            //Si s'arriba des de el menu principal es carreguen les dades del  càrrec
            if (e.NavigationMode != NavigationMode.Back)
            {
                carrecId = Convert.ToInt32(NavigationContext.QueryString["carrecId"]);
                //Carrec nou
                if (carrecId == 0)
                {
                    botoSeleccioPermisos.Content = "<<Sense permisos>>";
                }
                //Càrrec existent
                else if (carrecId > 0)
                {
                    carregarCarrec();
                }
                carregarLlistaPermisos();
            }
        }
        #endregion

        #region Buscar carrec
        /*
         * Metode que realitza una crida asíncrona per buscar un carrec pel seu id
         */
        private void carregarCarrec()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarCarrecPerIdCompleted += serveiBuscarCarrecPerIdCompleted;
            servei.buscarCarrecPerIdAsync(carrecId);
        }
        #endregion

        #region Buscar carrec finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar un carrec
         */
        private void serveiBuscarCarrecPerIdCompleted(object sender, ServeiWeb.buscarCarrecPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if(e.Error!=null){
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            //S' hi ha carrec, es carreguen els seus permisos
            else if (e.Result != null)
            {
                textNom.Text = e.Result.Nom;
                permisosId = (Int32)e.Result.IdPermisos;
                if (permisosId > 0)
                {
                    carregarPermisos();
                }
                else
                {
                    botoSeleccioPermisos.Content = "<<Sense permisos>>";
                }
            }
        }
        #endregion

        #region Buscar permis
        /*
         * Métode que inicia la crida asíncrona per buscar un permis, donat el seu id
         */
        private void carregarPermisos()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarPermisosPerIdCompleted += serveiBuscarPermisosPerIdCompleted;
            servei.buscarPermisosPerIdAsync(permisosId);
        }
        #endregion

        #region Buscar permis finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar un permis
         */
        private void serveiBuscarPermisosPerIdCompleted(object sender, ServeiWeb.buscarPermisosPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            //Si es troba el permis, s'afegeixen els permisos que otorga
            else if (e.Result != null)
            {
                botoSeleccioPermisos.Content = e.Result.Nom;
                if (e.Result.Administrar)
                {
                    checkAdministrar.IsChecked = true;
                }
                else
                {
                    checkAdministrar.IsChecked = false;
                }
                if (e.Result.Validar)
                {
                    checkValidar.IsChecked = true;
                }
                else
                {
                    checkValidar.IsChecked = false;
                }
                if (e.Result.VeureUsuaris != null && (bool)e.Result.VeureUsuaris)
                {
                    checkVeureUsuaris.IsChecked = true;
                }
                else
                {
                    checkVeureUsuaris.IsChecked = false;
                }
                if (e.Result.VeureDespeses != null && (bool)e.Result.VeureDespeses)
                {
                    checkVeureDespeses.IsChecked = true;
                }
                else
                {
                    checkVeureDespeses.IsChecked = false;
                }
            }
        }
        #endregion

        #region Llista de permisos
        /*
         * Metode que inicia la crida asincrona per buscar els permisos existents
         */
        private void carregarLlistaPermisos()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarPermisosCompleted += serveiLlistarPermisosCompleted;
            servei.llistarPermisosAsync();
        }
        #endregion

        #region Llista de permisos finalitzada
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar els permisos existents
         */
        private void serveiLlistarPermisosCompleted(object sender, ServeiWeb.llistarPermisosCompletedEventArgs e)
        {
            if(e.Error!=null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
            }
            else if (e.Result != null)
            {
                scrollPermisos.Content = Permisos.llistaGridPermisos(e.Result);
            }
        }
        #endregion

        #region Permisos seleccionats
        /*
         * Event per canvir de pantalla per mostrar els carrecs disponibles
         */
        private void clickSeleccionarPermisos(object sender, RoutedEventArgs e)
        {
            itemPanPermisos.Visibility = Visibility.Visible;
            panCarrecs.DefaultItem = itemPanPermisos;
            itemPanCarrec.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Permis seleccionat
        /*
         * Event per acceptar els permisos del carrec
         */
        private void clickAcceptarPermisos(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Permisos.permisosSeleccionats(scrollPermisos);
            //Si no s'han seleccionat permisos no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
                //Si s'ha seleccionat un permis es modifiquen les dades
            else if (llista.Count == 1)
            {
                permisosId = llista[0];
                carregarPermisos();
                itemPanCarrec.Visibility = Visibility.Visible;
                panCarrecs.DefaultItem = itemPanCarrec;
                itemPanPermisos.Visibility = Visibility.Collapsed;
            }
                //Nomes es pot seleccionar un permis a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_PERMIS);
            }
        }
        #endregion

        #region Cancel.lar permisos
        /*
         * Event per retornar a la pagina principal per afegir o modificar un carrec
         */
        private void clickCancelarPermisos(object sender, RoutedEventArgs e)
        {
            itemPanCarrec.Visibility = Visibility.Visible;
            panCarrecs.DefaultItem = panCarrecs.Items[0];
            itemPanPermisos.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Guardar carrec
        /*
         * Event per guardar un carrec nou o modificar un existent.
         * Realitza la crida asincrona per afegir o modificar
         */
        private void clickGuardarCarrec(object sender, RoutedEventArgs e)
        {
            /*
             * Si hi han dades
             */
            if (textNom.Text.Length > 0)
            {
                //Carrec nou
                if (carrecId == 0)
                {
                    servei = ConnexioServei.connexioNova();
                    servei.afegirCarrecCompleted += serveiAfegirCarrecCompleted;
                    servei.afegirCarrecAsync(textNom.Text, permisosId);
                }
                //Modificar carrec existent
                else if (carrecId > 0)
                {
                    servei = ConnexioServei.connexioNova();
                    servei.modificarCarrecExistentCompleted += serveiModificarCarrecExistentCompleted;
                    servei.modificarCarrecExistentAsync(carrecId, textNom.Text, permisosId);
                }
            }
            else
            {
                MessageBox.Show(Missatges.DADES_FALTEN);
            }
        }
        #endregion

        #region Afegir carrec nou finalitzat
        /*
         * Event posterior a afegir un carrec nou.
         * Retorna a la pantalla principal d'administració
         */
        private void serveiAfegirCarrecCompleted(object sender, ServeiWeb.afegirCarrecCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CARREC_AFEGIR_ERROR + Environment.NewLine + Missatges.CARREC_DUPLICAT);
            }
            else
            {
                NavigationService.GoBack();
            }
        }
        #endregion

        #region Modificar carrec existent finalitzat
        /*
         * Event posterior a modificar un carrec.
         * Retorna a la pantalla principal d'administració
         */
        private void serveiModificarCarrecExistentCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CARREC_MODIFICAR_ERROR + Environment.NewLine + Missatges.CARREC_DUPLICAT);
            }
            else
            {
                NavigationService.GoBack();
            }
        }
        #endregion

        #region Carrec sense permisos
        /*
         * Event per indicar que el carrec no disposa de permisos 
         */
        private void clickSensePermisos(object sender, RoutedEventArgs e)
        {
            permisosId = 0;
            botoSeleccioPermisos.Content = "<<Sense permisos>>";
            checkAdministrar.IsChecked = false;
            checkValidar.IsChecked = false;
            checkVeureUsuaris.IsChecked = false;
            checkVeureDespeses.IsChecked = false; ;
        }
        #endregion

        #region Tornar
        /*
         * Event per retornar a la pantalla principal d'administracio
         */
        private void clickCancelar(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();
        }
        #endregion
    }
}