﻿using System;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;

namespace ClientSGD
{
    public partial class AdministracioTipus : PhoneApplicationPage
    {
        private Int32 tipusId;
        private ServeiWeb.ServeiClient servei;
        private string nomTipus;

        public AdministracioTipus()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event cridat en carregar la página que verifica des de on s'arriba al formulari
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                tipusId = Convert.ToInt32(NavigationContext.QueryString["tipusId"]);
                if (tipusId > 0)
                {
                    carregarTipus();
                }
            }
        }
        #endregion

        #region Buscar tipus de despesa
        /*
         * Metode que realitza una crida asíncrona per buscar un tipus de despesa, nodat el seu id
         */
        private void carregarTipus()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarTipusPerIdCompleted += serveiBuscarTipusPerIdCompleted;
            servei.buscarTipusPerIdAsync(tipusId);
        }
        #endregion

        #region Buscar tipus de despesa finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar un tipus de despesa
         */
        private void serveiBuscarTipusPerIdCompleted(object sender, ServeiWeb.buscarTipusPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            //Estableix el tipus actual
            else if (e.Result != null)
            {
                nomTipus = e.Result.Nom;
                textNom.Text = e.Result.Nom;
            }
        }
        #endregion

        #region Guardar tipus de despesa
        /*
         * Event per verificar dades i realitzar la crida asincrona per afegir un nou tipus 
         * o modificar un existent
         */
        private void clickGuardarTipus(object sender, RoutedEventArgs e)
        {
            //Si hi han dades
            if (textNom.Text.Length > 0 && !textNom.Text.Equals(nomTipus))
            {
                servei = ConnexioServei.connexioNova();
                if (tipusId == 0)
                {
                    servei.afegirTipusNouCompleted += serveiAfegirTipusNouCompleted;
                    servei.afegirTipusNouAsync(textNom.Text);
                }
                else if (tipusId > 0)
                {
                    servei.modificarTipusCompleted += serveiModificarTipusCompleted;
                    servei.modificarTipusAsync(tipusId, textNom.Text);
                }
                servei.CloseAsync();
            }
        }
        #endregion

        #region Modificar tipus de despesa finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per modificar un tipus de despesa
         */
        private void serveiModificarTipusCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            NavigationService.GoBack();
        }
        #endregion

        #region Afegir tipus de despesa nou finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per afegir un nou tipus de despesa
         */
        private void serveiAfegirTipusNouCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.TIPUS_AFEGIR_ERROR + Environment.NewLine + Missatges.TIPUS_DUPLICAT);
            }
            else
            {
                NavigationService.GoBack();
            }
        }
        #endregion

        #region Tornar
        /*
         * Event per cancel·lar la gestio del tipus de despesa i tornar a la pagina anterior
         */
        private void clickCancelarTipus(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();
        }
        #endregion
    }
}