﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class AdministracioUsuari : PhoneApplicationPage
    {
        private Int32 usuariId, carrecId, responsableId;
        private ServeiWeb.ServeiClient servei;

        public AdministracioUsuari()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'arriba a la pagina
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                usuariId = Convert.ToInt32(NavigationContext.QueryString["usuariId"]);
                //Si l'usuari existeix es carreguen les seves dades
                if (usuariId > 0)
                {
                    buscarUsuari();
                }
                //Si es un usuari nou
                else
                {
                    carrecId = 0;
                    responsableId = 0;
                }
                //Carregar llista de carrecs disponibles
                carregarLlistaCarrecs();
                //Carregar llista de responsable
                carregarLlistaResponsables();
            }
        }
        #endregion

        #region Buscar usuari
        /*
         * Metode que inicia la crida asincrona per buscar un usuari,donat el seu id
         */
        private void buscarUsuari()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarUsuariPerIdCompleted += serveiBuscarUsuariPerIdCompleted;
            servei.buscarUsuariPerIdAsync(usuariId);
        }
        #endregion
        
        #region Buscar usuari finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir totes les dades d'un usuari
         */
        private void serveiBuscarUsuariPerIdCompleted(object sender, ServeiWeb.buscarUsuariPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                textDni.Text = e.Result.Dni;
                textNom.Text = e.Result.Nom;
                textCognoms.Text = e.Result.Cognoms;
                carrecId = e.Result.IdCarrec;
                responsableId = e.Result.IdResponsable;
                //Es buscan les dades del carrec de l'usuari
                carregarCarrec();
                //Es busca el nom del responsable de l'usuari
                carregarResponsable();
            }
        }
        #endregion

        #region Buscar carrec
        /*
         * Metode que realitza una crida asincrona per carregar el carrec de l'usuari
         */
        private void carregarCarrec()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarCarrecPerIdCompleted += serveiBuscarCarrecPerIdCompleted;
            servei.buscarCarrecPerIdAsync(carrecId);
        }
        #endregion

        #region Buscar carrec
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir el carrec de l'usuari
         */
        private void serveiBuscarCarrecPerIdCompleted(object sender, ServeiWeb.buscarCarrecPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                carrecId = e.Result.Id;
                botoCarrecUsuari.Content = e.Result.Nom;
            }
        }
        #endregion

        #region Buscar responsable
        /*
         * Metode que realitza una crida asincrona per carregar el responsable de l'usuari
         */
        private void carregarResponsable()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarUsuariPerIdCompleted += serveiBuscarNomResponsablePerIdCompleted;
            servei.buscarUsuariPerIdAsync(responsableId);
        }
        #endregion

        #region Buscar nom de responsable finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir el responsable de l'usuari
         */
        private void serveiBuscarNomResponsablePerIdCompleted(object sender, ServeiWeb.buscarUsuariPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                botoResponsableUsuari.Content = e.Result.Nom;
            }
        }
        #endregion

        #region Carrec seleccionat
        /*
         * Event que modifica la pagina de visualitzacio per la de seleccio de carrec
         */
        private void clickSeleccionarCarrec(object sender, RoutedEventArgs e)
        {
            itemPanCarrec.Visibility = Visibility.Visible;
            panUsuaris.DefaultItem = itemPanCarrec;
            itemPanUsuari.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Llista de carrecs habilitats
        /*
         * Metode que realitza una crida asíncrona per carregar els carrecs disponibles
         */
        private void carregarLlistaCarrecs()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarCarrecsActiusCompleted += serveiLlistarCarrecsActiusCompleted;
            servei.llistarCarrecsActiusAsync();
        }
        #endregion

        #region Llista de carrecs actius finalitzada
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir els carrecs disponibles
         */
        private void serveiLlistarCarrecsActiusCompleted(object sender, ServeiWeb.llistarCarrecsActiusCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null && e.Result.Count > 0)
            {
                scrollSeleccioCarrec.Content = Carrec.llistaCarrecsHabilitats(e.Result);
            }
        }
        #endregion

        #region Llista de responsables
        /*
         * Metode que realitza una crida asíncrona per carregar els usuaris habilitats
         */
        private void carregarLlistaResponsables()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistaDeUsuarisActiusCompleted += serveiLlistaDeUsuarisActiusCompleted;
            servei.llistaDeUsuarisActiusAsync();
        }
        #endregion

        #region Responsable seleccionat
        /*
         * Event que modifica la pagina de visualitzacio per la de seleccio de responsable
         */
        private void clickSeleccionarResponsable(object sender, RoutedEventArgs e)
        {
            itemPanResponsable.Visibility = Visibility.Visible;
            panUsuaris.DefaultItem = itemPanResponsable;
            itemPanUsuari.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Llista d'usuaris habilitats
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obtenir els usuaris habilitats
         */
        private void serveiLlistaDeUsuarisActiusCompleted(object sender, ServeiWeb.llistaDeUsuarisActiusCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null && e.Result.Count > 0)
            {
                scrollSeleccioResponsable.Content = Usuari.llistaGridResponsables(e.Result);
            }
        }
        #endregion

        #region Llista de carrecs seleccionats

        /*
         * Event per determinar el carrec seleccionat i canviar la pantalla
         */
        private void clickCarrecSeleccionat(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Carrec.carrecsHabilitatsSeleccionats(scrollSeleccioCarrec);
            //Si no s'han sleccionat carrecs no es fa res
            if (llista.Count == 0)
            {
            }
            //Si s'ha seleccionat un carrec es modifiquen les dades
            else if (llista.Count == 1)
            {
                carrecId = llista[0];
                carregarCarrec();
                itemPanUsuari.Visibility = Visibility.Visible;
                panUsuaris.DefaultItem = itemPanUsuari;
                itemPanCarrec.Visibility = Visibility.Collapsed;

            }
            //Nomes es pot modificar un carrec a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_CARREC);
            }
        }
        #endregion

        #region Llista de responsables seleccionats
        /*
         * Event que revisa el responsable seleccionat
         */
        private void clickResponsableSeleccionat(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Usuari.responsablesSeleccionats(scrollSeleccioResponsable);
            //Si no s'han seleccioant responsables no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccioant un responsable
            else if (llista.Count == 1)
            {
                responsableId = llista[0];
                //Si l'usuari es el propi responsable
                if (llista[0] == 0)
                {
                    botoResponsableUsuari.Content = "<Propi responsable>";
                }
                else
                {
                    carregarResponsable();
                }
                //Canviar la pantalla per mostrar
                itemPanUsuari.Visibility = Visibility.Visible;
                panUsuaris.DefaultItem = itemPanUsuari;
                itemPanResponsable.Visibility = Visibility.Collapsed;
            }
            //Nomes es pot seleccionat un responsable a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_RESPONSABLE);
            }
        }
        #endregion

        #region Guardar usuari
        /*
         * Event per afegir un nou usuari o modificar un existent.
         * Realitza la crida asíncrona per afegir o modificar l'usuari
         */
        private void clickGuardarUsuari(object sender, RoutedEventArgs e)
        {
            string contrasenya1 = textContrasenya.Password, contrasenya2 = textRepetirContrasenya.Password;
            if (textDni.Text.Length > 0 && textNom.Text.Length > 0 && textCognoms.Text.Length > 0 && carrecId > 0 && !botoResponsableUsuari.Content.Equals(string.Empty))
            {
                #region Usuari nou
                //Usuari nou
                if (usuariId == 0)
                {
                    if (contrasenya1.Length > 0 && contrasenya2.Length > 0)
                    {
                        if (contrasenya1.Equals(contrasenya2))
                        {
                            //Afegir i modificar responsable amb nou id
                            if (responsableId == 0)
                            {
                                servei = ConnexioServei.connexioNova();
                                servei.afegirNouUsuariCompleted += serveiAfegirNouUsuariCompleted;
                                servei.afegirNouUsuariAsync(textDni.Text, textNom.Text, textCognoms.Text, carrecId, 1, contrasenya1);
                            }
                            //Afegir usuari amb responsable existent
                            else
                            {
                                servei = ConnexioServei.connexioNova();
                                servei.afegirNouUsuariCompleted += serveiAfegirNouUsuariCompleted;
                                servei.afegirNouUsuariAsync(textDni.Text, textNom.Text, textCognoms.Text, carrecId, responsableId, contrasenya1);
                            }
                        }
                        else
                        {
                            MessageBox.Show(Missatges.CONTRASENYA_COINCIDIR);
                        }
                    }
                    else
                    {
                        MessageBox.Show(Missatges.CONTRASENYA_COINCIDIR);
                    }
                }
                #endregion
                #region Usuari existent
                //Usuari existent
                else
                {
                    if ((contrasenya1.Length > 0 && !contrasenya1.Equals(contrasenya2)) || (contrasenya2.Length > 0 && !contrasenya2.Equals(contrasenya1)))
                    {
                        MessageBox.Show(Missatges.CONTRASENYA_COINCIDIR);
                    }
                    else
                    {
                        //Es un responsable existent
                        if (responsableId == 0)
                        {
                            servei = ConnexioServei.connexioNova();
                            servei.modificarUsuariExistentCompleted += serveiModificarUsuariExistentCompleted;
                            servei.modificarUsuariExistentAsync(usuariId, textDni.Text, textNom.Text, textCognoms.Text, carrecId, usuariId, contrasenya1);
                        }
                        else
                        {
                            //L'usuari es el propi responsable 
                            servei = ConnexioServei.connexioNova();
                            servei.modificarUsuariExistentCompleted += serveiModificarUsuariExistentCompleted;
                            servei.modificarUsuariExistentAsync(usuariId, textDni.Text, textNom.Text, textCognoms.Text, carrecId, responsableId, contrasenya1);
                        }
                    }
                }
                #endregion
            }
                
            else
            {
                MessageBox.Show(Missatges.DADES_FALTEN);
            }
        }
        #endregion

        #region Modificar usuari finalitzat
        /*
         * Event posterior a la crida asincrona per modificar un usuari
         */
        private void serveiModificarUsuariExistentCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.USUARI_MODIFICAR_ERROR + Environment.NewLine + Missatges.USUARI_DUPLICAT);
            }
            else
            {
                NavigationService.GoBack();
            }
        }
        #endregion

        #region Afegir usuari nou finalitzat
        /*
         * Event posterior a la crida asincrona per afegir un usuari nou.
         * Realitza la crida asincrona per ajustar el responsable quan s'afegeix un usuari que es el propi responsable
         */
        private void serveiAfegirNouUsuariCompleted(object sender, ServeiWeb.afegirNouUsuariCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.USUARI_AFEGIR_ERROR + Environment.NewLine + Missatges.USUARI_DUPLICAT);
            }
            else if (e.Result != null)
            {
                if (responsableId == 0)
                {
                    string contrasenya = textContrasenya.Password;
                    servei = ConnexioServei.connexioNova();
                    servei.modificarUsuariExistentCompleted += serveiModificarUsuariExistentCompleted;
                    servei.modificarUsuariExistentAsync(e.Result.Id, e.Result.Dni, e.Result.Nom, e.Result.Cognoms, e.Result.IdCarrec, e.Result.Id, contrasenya);
                }
                else
                {
                    //Es retorna a la pantalla principal d'administracio
                    NavigationService.GoBack();
                }
            }
        }
        #endregion

        #region Tornar
        /*
         * Event per retornar a la pantalla principal d'administracio
         */
        private void clickCancelar(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();
        }
        #endregion

        #region Cancel.lar carrec
        /*
         * Event per canviar la pantalla que es mostra
         */
        private void clickCancelarCarrec(object sender, RoutedEventArgs e)
        {
            itemPanUsuari.Visibility = Visibility.Visible;
            panUsuaris.DefaultItem = itemPanUsuari;
            itemPanCarrec.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Cancel.lar responsable
        /*
         * Event per canviar la pantalla que es mostra
         */
        private void clickCancelarResponsable(object sender, RoutedEventArgs e)
        {
            itemPanUsuari.Visibility = Visibility.Visible;
            panUsuaris.DefaultItem = itemPanUsuari;
            itemPanResponsable.Visibility = Visibility.Collapsed;
        }
        #endregion
    }
}