﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class Calendari : PhoneApplicationPage
    {
        private bool responsable;
        private ServeiWeb.ServeiClient servei;

        public Calendari()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'arriba a la pagina.
         * Inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                responsable = Convert.ToBoolean(NavigationContext.QueryString["responsable"]);
                if (responsable)
                {
                    botoAfegirFestiu.Visibility = Visibility.Visible;
                    botoEliminarFestiu.Visibility = Visibility.Visible;
                }
            }
            carregarCalendari();
        }
        #endregion

        #region Buscar dates festives per al llistat
        /*
         * Metode que inicia la crida asincrona per buscar els dies festius
         */
        private void carregarCalendari()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarFestiusCompleted += serveiLlistarFestiusCompleted;
            servei.llistarFestiusAsync();
        }
        #endregion

        #region Buscar dates festives finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar els dies festius
         */
        private void serveiLlistarFestiusCompleted(object sender, ServeiWeb.llistarFestiusCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            //Si hi han dates es mostren al llistat
            else if (e.Result != null)
            {
                scrollFestius.Content = Festiu.llistaGridFestius(e.Result);
            }
        }
        #endregion

        #region Data seleccionada
        /*
         * Event que estableix la data seleccionada
         */
        private void clickData(object sender, WPControls.SelectionChangedEventArgs e)
        {
            blockData.Text = e.SelectedDate.ToString();
        }
        #endregion

        #region Afegir festiu nou
        /*
         * Event que inicia la crida asincrona per afegir una data nova al calendari
         */
        private void clickAfegirData(object sender, RoutedEventArgs e)
        {
            if (blockData.Text.Length > 0 && textDescripcio.Text.Length > 0)
            {
                servei = ConnexioServei.connexioNova();
                servei.afegirFestiuNouCompleted += serveiAfegirFestiuNouCompleted;
                servei.afegirFestiuNouAsync(Convert.ToDateTime(blockData.Text), textDescripcio.Text);
                blockData.Text = string.Empty;
                textDescripcio.Text = string.Empty;
            }
        }
        #endregion

        #region Afegir festiu nou finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per afegir una data al calendari.
         * Recarrega les dates de la llista i canvia a la pantalla del llistat
         */
        private void serveiAfegirFestiuNouCompleted(object sender, ServeiWeb.afegirFestiuNouCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FESTIU_AFEGIR_ERROR + Environment.NewLine + Missatges.FESTIU_DUPLICAT);
            }
            else if (e.Result != null)
            {
                carregarCalendari();
                panItemLlista.Visibility = Visibility.Visible;
                panCalendari.DefaultItem = panItemLlista; ;
                panItemAfegir.Visibility = Visibility.Collapsed;
            }
        }
        #endregion

        #region Eliminar festiu
        /*
         * Event que inicia la crida asincrona per eliminar una data del calendari
         */
        private void clickEliminarFestiu(object sender, RoutedEventArgs e)
        {
            List<DateTime> llistaDates = Festiu.datesSeleccionades(scrollFestius);
            //Si no es seleccionen festius no es fa res
            if (llistaDates.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona un festiu es pot eliminar
            else if (llistaDates.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.eliminarFestiuCompleted += serveiEliminarFestiuCompleted;
                servei.eliminarFestiuAsync(llistaDates[0]);
            }
            //Nomes es pot eliminar un festiu cada cop
            else if (llistaDates.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FESTIU);
            }
        }
        #endregion

        #region Eliminar festiu finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per eliminar una data festiva
         */
        private void serveiEliminarFestiuCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FESTIU_ELIMINAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                carregarCalendari();
            }
        }
        #endregion

        #region Festiu nou
        /*
         * Event per establir la pantalla a la de afegir dates
         */
        private void clickNouFestiu(object sender, RoutedEventArgs e)
        {
            panItemAfegir.Visibility = Visibility.Visible;
            panCalendari.DefaultItem = panItemAfegir;
            panItemLlista.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Cancel.lar afegir festiu nou
        /*
         * Event per establir la pantalla a la del llistat de dates festives
         */
        private void clickCancelarAfegir(object sender, RoutedEventArgs e)
        {
            panItemLlista.Visibility = Visibility.Visible;
            panCalendari.DefaultItem = panItemLlista; ;
            panItemAfegir.Visibility = Visibility.Collapsed;
        }
        #endregion
    }
}