﻿using System.Windows;
using Microsoft.Phone.Controls;
using System.Net;
using System.Windows.Navigation;
using System.IO.IsolatedStorage;
using ClientSGD.Resources;
using System.Windows.Media;

namespace ClientSGD
{
    public partial class Configuracio : PhoneApplicationPage
    {
        public static bool estatWCF { get; set; }
        private IsolatedStorageSettings config;
        private ServeiWeb.ServeiClient servei;

        public Configuracio()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que revisa des de on s'arriba a la pagina i si hi ha connexio amb internet
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            estatWCF = false;
            iniciar();
        }

        private void iniciar()
        {
            config = IsolatedStorageSettings.ApplicationSettings;
            textIp.Text = IsolatedStorageSettings.ApplicationSettings["IpServei"].ToString();
            textDirectori.Text = IsolatedStorageSettings.ApplicationSettings["PublicacioServei"].ToString();
        }
        #endregion

        #region Guardar configuracio
        /*
         * Event que verifica les dades i actualitza l'arxiu de configuracio de l'aplicacio
         */
        private void clickGuardar(object sender, RoutedEventArgs e)
        {
            if (textIp.Text.Length > 0 && textDirectori.Text.Length > 0)
            {
                if (verificarIp(textIp.Text))
                {
                    estatWCF = false;
                    textIp.IsEnabled = false;
                    textDirectori.IsEnabled = false;
                    botoGuardar.IsEnabled = false;
                    botoCancelar.IsEnabled = false;

                    blockResposta.Text = Missatges.VERIFICAR_SERVEI;
                    blockResposta.Foreground = new SolidColorBrush(Colors.Black);
                    gridResultat.Background = new SolidColorBrush(Colors.Cyan);

                    config["IpServei"] = textIp.Text;
                    config["PublicacioServei"] = textDirectori.Text;
                    config["UrlCompletaServei"] = "http://" + config["IpServei"] + "/" + config["PublicacioServei"];
                    config.Save();

                    servei = ConnexioServei.connexioNova();
                    servei.validarUsuariCompleted += serveiVerificarServeiWeb;
                    servei.validarUsuariAsync("", "");
                }
                else
                {
                    MessageBox.Show(Missatges.IP_ERROR);
                }
            }
            else
            {
                MessageBox.Show(Missatges.DADES_FALTEN);
            }
        }
        #endregion

        #region Verificar connexio amb servei web finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per verificar la connexio amb el servei web
         */
        private void serveiVerificarServeiWeb(object sender, ClientSGD.ServeiWeb.validarUsuariCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                blockResposta.Text = "Sense connexió amb el servei web";
                blockResposta.Foreground = new SolidColorBrush(Colors.Black);
                gridResultat.Background = new SolidColorBrush(Colors.Red);
            }
            else
            {
                blockResposta.Text = "Connexió correcte amb el servei web";
                blockResposta.Foreground = new SolidColorBrush(Colors.Black);
                gridResultat.Background = new SolidColorBrush(Colors.Green);
                estatWCF = true;
                NavigationService.GoBack();
            }
            textIp.IsEnabled = true;
            textDirectori.IsEnabled = true;
            botoGuardar.IsEnabled = true;
            botoCancelar.IsEnabled = true;
        }
        #endregion

        #region Verificar IP
        /*
         * Metode que verifica si la IP introduida te el format adequat
         */
        private static bool verificarIp(string ip)
        {
            IPAddress ipTemp;
            if (IPAddress.TryParse(ip, out ipTemp))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion

        #region Tornar
        /*
         * Metode per cancel·lar la modificacio de l'arxiu de configuracio de l'aplicacio
         */
        private void clickCancelar(object sender, RoutedEventArgs e)
        {
            estatWCF = false;
            NavigationService.GoBack();
        }
        #endregion
    }
}