﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using System.IO;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class GestioDespesa : PhoneApplicationPage
    {
        private Int32 fullId, despesaActualId, adjuntId;
        private ServeiWeb.ServeiClient servei;
        private Int32 tipusId;
        private PhotoResult imatgeAdjunt;
        private static decimal maxImport = 1000000;
        private static Int32 maxAdjunt = 10485760;//10 Megabytes

        public GestioDespesa()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'accedeix.
         * Inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                despesaActualId = Convert.ToInt32(NavigationContext.QueryString["despesaId"]);
                fullId = Convert.ToInt32(NavigationContext.QueryString["fullId"]);
                //Despesa nova
                if (despesaActualId == 0)
                {
                    tipusId = 0;
                    adjuntId = 0;
                    botoData.Content = "Seleccionar";
                    botoTipus.Content = "Seleccionar";
                }
                //Despesa existent
                else if (despesaActualId > 0)
                {
                    carregarDespesa();
                }
                carregarTipusActius();
            }
        }
        #endregion

        #region Buscar despesa
        /*
         * Metode que inicia la crida asincrona per buscar una despesa,donat el seu id
         */
        private void carregarDespesa()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarDespesaCompleted += serveiBuscarDespesaCompleted;
            servei.buscarDespesaAsync(despesaActualId);
        }
        #endregion

        #region Buscar despesa finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per busca una despesa
         */
        private void serveiBuscarDespesaCompleted(object sender, ServeiWeb.buscarDespesaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            //Si es troba la despesa es carreguen les dades
            else if (e.Result != null)
            {
                botoData.Content = (e.Result.Data).ToString("dd/MM/yyyy");
                textImport.Text = e.Result.Import.ToString();
                textComentari.Text = e.Result.Comentari;
                tipusId = e.Result.Tipus;
                botoTipus.Content = e.Result.NomTipus;
                //Si te adjunt es busca el seu nom i id
                if (e.Result.Adjunt)
                {

                    servei = ConnexioServei.connexioNova();
                    servei.buscarNomDeFacturaCompleted += serveiBuscarNomDeFacturaCompleted;
                    servei.buscarNomDeFacturaAsync(despesaActualId);

                    servei = ConnexioServei.connexioNova();
                    servei.buscarIdFacturaCompleted += serveiBuscarIdFacturaCompleted;
                    servei.buscarIdFacturaAsync(despesaActualId);
                }
            }
        }
        #endregion

        #region Buscar nom de factura
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar el nom d'una factura, donat el id de la despesa
         */
        private void serveiBuscarNomDeFacturaCompleted(object sender, ServeiWeb.buscarNomDeFacturaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                textAdjunt.Text = e.Result;
            }
        }
        #endregion

        #region Buscar nom de factura finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar el id d'una factura, donat el id de la despesa
         */
        private void serveiBuscarIdFacturaCompleted(object sender, ServeiWeb.buscarIdFacturaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != 0)
            {
                adjuntId = e.Result;
            }
        }
        #endregion

        #region Buscar tipus de despesa finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar el tipus de despesa, donat el seu id
         */
        private void serveiBuscarTipusPerIdCompleted(object sender, ServeiWeb.buscarTipusPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                botoTipus.Content = e.Result.Nom;
            }
        }
        #endregion

        #region Buscar tipus de despeses actius
        /*
         * Metode inicia la crida asincrona per buscar els tipus de despeses habilitats
         */
        private void carregarTipusActius()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarTipusActiusCompleted += serveiLlistarTipusActiusCompleted;
            servei.llistarTipusActiusAsync();
        }
        #endregion

        #region Buscar tipus de despeses actius finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar els tipus de despeses habilitats
         */
        private void serveiLlistarTipusActiusCompleted(object sender, ServeiWeb.llistarTipusActiusCompletedEventArgs e)
        {
            servei.CloseAsync();
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null && e.Result.Count > 0)
            {
                StackPanel stack = new StackPanel();
                scrollTipus.Content = Tipus.llistaGridTipusActivats(e.Result);
            }
        }
        #endregion

        #region Pantalla de seleccio de data
        /*
         * Event que estableix la pantalla de seleccio de data 
         */
        private void clickData(object sender, RoutedEventArgs e)
        {
            itemPanData.Visibility = Visibility.Visible;
            panoramaDespeses.DefaultItem = itemPanData;
            itemPanDespesa.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Data seleccionada
        /*
         * Event que estableix la data seleccionada i estableix la pantalla de despesa
         */
        private void dataSeleccionada(object sender, WPControls.SelectionChangedEventArgs e)
        {
            botoData.Content = e.SelectedDate.ToString("dd/MM/yyyy");
            itemPanDespesa.Visibility = Visibility.Visible;
            panoramaDespeses.DefaultItem = itemPanDespesa;
            itemPanData.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Tipus de despesa seleccionat
        /*
         * Event que estableix la pantalla de seleccio de tipus de despesa 
         */
        private void clickTipus(object sender, RoutedEventArgs e)
        {
            itemPanTipus.Visibility = Visibility.Visible;
            panoramaDespeses.DefaultItem = itemPanTipus;
            itemPanDespesa.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Acceptar tipus de despesa seleccionat
        /*
         * Event que carrega el tipus seleccionat i estableix la pantalla de seleccio de data 
         */
        private void clickAcceptarTipus(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Tipus.tipusActivatsSeleccionats(scrollTipus);
            //Si no es seleccionen tipus de despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona un tipus de despesa, es carrega el tipus mitjanánt la crida asincrona
            else if (llista.Count == 1)
            {
                tipusId = llista[0];
                servei = ConnexioServei.connexioNova();
                servei.buscarTipusPerIdCompleted += serveiBuscarTipusPerIdCompleted;
                servei.buscarTipusPerIdAsync(tipusId);

                itemPanDespesa.Visibility = Visibility.Visible;
                panoramaDespeses.DefaultItem = itemPanDespesa;
                itemPanTipus.Visibility = Visibility.Collapsed;
            }
            //Nomes es pot seleccionar un tipus de despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_TIPUS);
            }
        }
        #endregion

        #region Cancel.lar seleccion de tipus de despesa
        /*
         * Event que estableix la pantalla de gestio de despesa
         */
        private void clickCancelarTipus(object sender, RoutedEventArgs e)
        {
            itemPanDespesa.Visibility = Visibility.Visible;
            panoramaDespeses.DefaultItem = itemPanDespesa;
            itemPanTipus.Visibility = Visibility.Collapsed;
        }
        #endregion

        #region Guardar despesa
        /*
         * Event que inicia la crida asincrona per afegir o modificar una despesa.
         */
        private void clickGuardarDespesa(object sender, RoutedEventArgs e)
        {
            if (botoData.Content.Equals("Seleccionar") || botoTipus.Content.Equals("Seleccionar") || textImport.Text.Length == 0)
            {
                MessageBox.Show(Missatges.DADES_FALTEN);
            }
            else
            {
                if (!noAdmesosImport(textImport.Text))
                {
                    if (verificarImport(textImport.Text))
                    {
                        decimal import = decimal.Parse(textImport.Text);
                        if (import <= maxImport)
                        {
                            if (import >= 90 && textAdjunt.Text.Length == 0)
                            {
                                MessageBox.Show(Missatges.FACTURA_OBLIGAT);
                            }
                            else
                            {
                                string nomArxiu = string.Empty;
                                byte[] adjunt = new byte[0];
                                #region Despesa nova
                                //Nova despesa
                                if (despesaActualId == 0)
                                {
                                    //Hi ha adjunt
                                    if (imatgeAdjunt != null)
                                    {
                                        MemoryStream lectorAdjunt = new MemoryStream();
                                        imatgeAdjunt.ChosenPhoto.CopyTo(lectorAdjunt);
                                        adjunt = lectorAdjunt.ToArray();
                                        nomArxiu = Path.GetFileName(imatgeAdjunt.OriginalFileName);
                                    }
                                    //No hi ha adjunt
                                    else
                                    {
                                        //No fer res
                                    }
                                    servei = ConnexioServei.connexioNova();
                                    servei.afegirNovaDespesaCompleted += serveiAfegirNovaDespesaCompleted;
                                    servei.afegirNovaDespesaAsync(fullId, Convert.ToDateTime(botoData.Content), import, tipusId, textComentari.Text, nomArxiu, adjunt);
                                }
                                #endregion
                                #region Despesa existent
                                //Modificar despesa existent
                                else
                                {
                                    if (adjuntId == 0)
                                    {
                                        //No hi havia adjunt, ara hi ha adjunt nou
                                        if (imatgeAdjunt != null)
                                        {
                                            MemoryStream lectorAdjunt = new MemoryStream();
                                            imatgeAdjunt.ChosenPhoto.CopyTo(lectorAdjunt);
                                            adjunt = lectorAdjunt.ToArray();
                                            nomArxiu = Path.GetFileName(imatgeAdjunt.OriginalFileName);
                                            servei = ConnexioServei.connexioNova();
                                            servei.modificarDespesaCompleted += serveiModificarDespesaCompleted;
                                            servei.modificarDespesaAsync(despesaActualId, Convert.ToDateTime(botoData.Content), import, tipusId, textComentari.Text, adjuntId, nomArxiu, adjunt);
                                        }
                                        //No hi havia adjunt i segueix sense adjunt
                                        else
                                        {
                                            servei = ConnexioServei.connexioNova();
                                            servei.modificarDespesaCompleted += serveiModificarDespesaCompleted;
                                            servei.modificarDespesaAsync(despesaActualId, Convert.ToDateTime(botoData.Content), import, tipusId, textComentari.Text, adjuntId, nomArxiu, adjunt);
                                        }
                                    }
                                    else
                                    {
                                        //Hi havia adjunt i ara hi ha un adjunt nou
                                        if (imatgeAdjunt != null)
                                        {
                                            MemoryStream lectorAdjunt = new MemoryStream();
                                            imatgeAdjunt.ChosenPhoto.CopyTo(lectorAdjunt);
                                            adjunt = lectorAdjunt.ToArray();
                                            nomArxiu = Path.GetFileName(imatgeAdjunt.OriginalFileName);
                                            servei = ConnexioServei.connexioNova();
                                            servei.modificarDespesaCompleted += serveiModificarDespesaCompleted;
                                            servei.modificarDespesaAsync(despesaActualId, Convert.ToDateTime(botoData.Content), import, tipusId, textComentari.Text, adjuntId, nomArxiu, adjunt);
                                        }
                                        else
                                        {
                                            //Hi havia adjunt i no es modifica
                                            if (!textAdjunt.Text.Equals(string.Empty))
                                            {
                                                servei = ConnexioServei.connexioNova();
                                                servei.modificarDespesaCompleted += serveiModificarDespesaCompleted;
                                                servei.modificarDespesaAsync(despesaActualId, Convert.ToDateTime(botoData.Content), import, tipusId, textComentari.Text, adjuntId, textAdjunt.Text, adjunt);
                                            }
                                            //Hi havia adjunt i ara s'elimina
                                            else
                                            {
                                                servei = ConnexioServei.connexioNova();
                                                servei.modificarDespesaCompleted += serveiModificarDespesaCompleted;
                                                servei.modificarDespesaAsync(despesaActualId, Convert.ToDateTime(botoData.Content), import, tipusId, textComentari.Text, adjuntId, nomArxiu, adjunt);
                                            }
                                        }
                                    }
                                }
                                #endregion
                            }
                        }
                        else
                        {
                            MessageBox.Show(Missatges.IMPORT_MAXIM);
                        }
                    }
                    else
                    {
                        MessageBox.Show(Missatges.IMPORT_ERROR);
                    }
                }
                else
                {
                    MessageBox.Show(Missatges.IMPORT_ERROR);
                }
            }
        }
        #endregion

        #region Modificar despesa finalitzat
        /*
         * Metode posterior a la finalitzacio de la crida asincrona per modificar una despesa
         */
        private void serveiModificarDespesaCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.DESPESA_MODIFICAR_ERROR);
            }
            else
            {
                NavigationService.GoBack();
            }
        }
        #endregion

        #region Afegir despesa nova finalitzat
        /*
         * Metode posterior a la finalitzacio de la crida asincrona per afegir una despesa nova
         */
        private void serveiAfegirNovaDespesaCompleted(object sender, ServeiWeb.afegirNovaDespesaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.DESPESA_AFEGIR_ERROR);
            }
            else
            {
                NavigationService.GoBack();
            }
        }
        #endregion

        #region Verificar import
        /*
         * Metode que verifica que una cadena es un decimal valid
         */
        private bool verificarImport(string import)
        {
            decimal temp;
            if (decimal.TryParse(import, out temp))
            {
                return true;
            }
            return false;
        }
        #endregion

        #region Verificar caracters no permesos d'import
        /*
         * Metode que verifica que l'import no sigui negativu ni tingui punts
         */
        private bool noAdmesosImport(string import)
        {
            if (import.Contains(".") || import.Contains("-"))
            {
                return true;
            }
            return false;
        }
        #endregion

        #region Adjuntar imatges al mobil
        /*
         * Event que inicia la crida asincrona per buscar una imatge al dispositiu
         */
        private void clickAdjuntar(object sender, RoutedEventArgs e)
        {
            PhotoChooserTask buscarAdjunt = new PhotoChooserTask();
            buscarAdjunt.Completed += new EventHandler<PhotoResult>(buscarAdjuntCompleted);
            buscarAdjunt.Show();
        }
        #endregion

        #region Buscar imatges al mobil finalitzat
        /*
         * Event posterior a la cerca d'una imatge
         */
        private void buscarAdjuntCompleted(object sender, PhotoResult e)
        {
            //Si es selecciona imatge,es guarda
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.ADJUNT_ERROR);
            }
            else
            {
                if (e.TaskResult == TaskResult.OK)
                {
                    if (e.ChosenPhoto.Length > maxAdjunt)
                    {
                        MessageBox.Show(Missatges.ADJUNT_MAXIM_ERROR);
                    }
                    else
                    {
                        imatgeAdjunt = e;
                        textAdjunt.Text = imatgeAdjunt.OriginalFileName;
                    }
                }
            }
        }
        #endregion

        #region Eliminar adjunt
        /*
         * Event per eliminar l'adjunt si hi ha
         */
        private void clickEliminarAdjunt(object sender, RoutedEventArgs e)
        {
            imatgeAdjunt = null;
            textAdjunt.Text = string.Empty;
        }
        #endregion

        #region Cancel.lar seleccio de data
        /*
         * Event per establir la pantalla a la de despesa
         */
        private void clickCancelarData(object sender, RoutedEventArgs e)
        {
            itemPanDespesa.Visibility = Visibility.Visible;
            panoramaDespeses.DefaultItem = itemPanDespesa;
            itemPanData.Visibility = Visibility.Collapsed;
        }
        #endregion 

        #region Tornar
        /*
         * Event per tornar a la pantalla d'administracio
         */
        private void clickCancelar(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();
        }
        #endregion
    }
}