﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class GestioDespesesPersonals : PhoneApplicationPage
    {
        private Int32 fullId, despesaId;
        private string nomFactura;
        private ServeiWeb.ServeiClient servei;

        public GestioDespesesPersonals()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'arriba
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            //Si la navegacio es cap enrere 
            if (e.NavigationMode != NavigationMode.Back)
            {
                fullId = Convert.ToInt32(NavigationContext.QueryString["fullId"]);
                bool modificar = Convert.ToBoolean(NavigationContext.QueryString["modificar"]);
                //Nomes es poden modificar despesese de fulls oberts
                if (modificar)
                {
                    botoAfegir.IsEnabled = true;
                    botoModificar.IsEnabled = true;
                    botoEliminar.IsEnabled = true;
                }
            }
            llistarDespeses();
        }
        #endregion

        #region Llista de despeses
        /*
         * Metode que realitza la crida asincrona per buscar les despeses d'un full, donat el id del full
         */
        private void llistarDespeses()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistaDespesesDefullCompleted += serveiBuscarDespesesDeFullCompleted;
            servei.llistaDespesesDefullAsync(fullId);
        }
        #endregion

        #region Llista de despeses finalitzada
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar les despeses d'un full
         */
        private void serveiBuscarDespesesDeFullCompleted(object sender, ServeiWeb.llistaDespesesDefullCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            //Si hi han despeses es carrega la llista
            else if (e.Result != null)
            {
                scrollDespeses.Content = Despesa.llistaGridDespeses(e.Result);
            }
        }
        #endregion

        #region Modificar despesa
        /*
         * Event que determina la despesa seleccionada i obre la pantalla per modifica-la
         */
        private void clickModificarDespesa(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Despesa.despesesSeleccionades(scrollDespeses);
            //Si no es seleccionen despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona una despesa s' obre la pagina enviat el id de la despesa i el full
            else if (llista.Count == 1)
            {
                NavigationService.Navigate(new Uri(string.Format("/GestionarDespeses/GestioDespesa.xaml?despesaId=" + llista[0] + "&fullId=" + fullId), UriKind.Relative));
            }
            //Nomes es pot modificar una despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_DESPESA);
            }
        }
        #endregion

        #region Afegir nova despesa
        /*
         * Event que obre la pagina per afegir una despesa nova a un full,donat el id del full
         */
        private void clickAfegirDespesa(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri(string.Format("/GestionarDespeses/GestioDespesa.xaml?despesaId=0" + "&fullId=" + fullId), UriKind.Relative));
        }
        #endregion

        #region Eliminar despeses seleccionades
        /*
         * Event que revisa les despeses seleccionades i inicia la crida asincrona per eliminar les despeses
         */
        private void clickEliminar(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Despesa.despesesSeleccionades(scrollDespeses);
            //Si no es seleccionen despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona una despesa o mes
            else if (llista.Count > 0)
            {
                List<bool> llistaConteAdjunt = Despesa.despesesAmbAdjunt(scrollDespeses);
                servei = ConnexioServei.connexioNova();
                servei.esborrarDespesesCompleted += serveiEsborrarDespesesCompleted;
                servei.esborrarDespesesAsync(llista, llistaConteAdjunt);
            }
        }
        #endregion

        #region Eliminar despeses seleccionades finalitzat
        /*
         * Event posterior a la finaltizacio de la crida asincrona per eliminar despeses
         */
        private void serveiEsborrarDespesesCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.DESPESA_ELIMINAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                llistarDespeses();
            }
        }
        #endregion

        #region Obrir adjunt
        private void clickObrirAdjunt(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Despesa.despesesSeleccionades(scrollDespeses);
            //Si no es seleccionen despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona una despesa es busca el nom de la factura
            else if (llista.Count == 1)
            {
                despesaId = llista[0];
                servei = ConnexioServei.connexioNova();
                servei.verificarAdjuntCompleted += serveiVerificarAdjuntCompleted;
                servei.verificarAdjuntAsync(llista[0]);
            }
            //Nomes es pot obrir l'adjunt d'una despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FACTURA);
            }
        }
        #endregion

        #region Verificar adjunt finalitzat
        private void serveiVerificarAdjuntCompleted(object sender, ServeiWeb.verificarAdjuntCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if ((bool)e.Result)
            {
                servei = ConnexioServei.connexioNova();
                servei.buscarNomDeFacturaCompleted += serveiBuscarNomDeFacturaCompleted;
                servei.buscarNomDeFacturaAsync(despesaId);
            }
            else
            {
                //No hi ha factura
            }
        }
        #endregion

        #region Buscar nom de factura finalitzat
        private void serveiBuscarNomDeFacturaCompleted(object sender, ServeiWeb.buscarNomDeFacturaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if (!e.Result.Equals(string.Empty))
            {
                if (e.Result.EndsWith(".jpg") || e.Result.EndsWith(".bmp") || e.Result.EndsWith(".gif") || e.Result.EndsWith(".jpeg") || e.Result.EndsWith(".png") || e.Result.EndsWith(".tif")
                    || e.Result.EndsWith(".tiff") || e.Result.EndsWith(".jfif") || e.Result.EndsWith(".jpe") || e.Result.EndsWith(".dib") || e.Result.EndsWith(".pdf"))
                {
                    nomFactura = e.Result; servei.Abort();
                    servei = ConnexioServei.connexioNova();
                    servei.buscarAdjuntDeDespesaCompleted += serveiBuscarAdjuntDeDespesaCompleted;
                    servei.buscarAdjuntDeDespesaAsync(despesaId);
                }
                else
                {
                    MessageBox.Show(Missatges.FACTURA_FORMAT);
                }
            }
        }
        #endregion

        #region Buscar adjunt finalitzat
        private void serveiBuscarAdjuntDeDespesaCompleted(object sender, ServeiWeb.buscarAdjuntDeDespesaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if (e.Result != null)
            {
                Accions.Factura.ObrirAdjunt(nomFactura, e.Result);
            }
        }
        #endregion
    }
}