﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class GestioDespesesTancades : PhoneApplicationPage
    {
        private Int32 fullId, despesaId;
        private string nomFactura;
        private ServeiWeb.ServeiClient servei;

        public GestioDespesesTancades()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on es realitza la navegacio.
         * Inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                fullId = Convert.ToInt32(NavigationContext.QueryString["fullId"]);
            }
            carregarDespeses();
        }
        #endregion

        #region Llista de despeses de full
        /*
         * Metode que inicia la crida asincrona per buscar les despeses d'un full,donat el seu id
         */
        private void carregarDespeses()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistaDespesesDefullCompleted += serveiLlistaDespesesDefullCompleted;
            servei.llistaDespesesDefullAsync(fullId);
        }
        #endregion

        #region Llista de despeses de full finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar les despeses d'un full
         */
        private void serveiLlistaDespesesDefullCompleted(object sender, ServeiWeb.llistaDespesesDefullCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            //Si hi han despeses es mostren al llistat
            else if (e.Result != null && e.Result.Count > 0)
            {
                scrollDespeses.Content = Despesa.llistaGridDespesesTancades(e.Result);
            }
        }
        #endregion

        #region Buscar comentari de despesa
        /*
         * Metode que inicia la crida asincrona per buscar el comentari d'una despesa,donat el seu id
         */
        private void clickObrirComentari(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaDespesesTancades = Despesa.despesesTancadesSeleccionades(scrollDespeses);
            //Si no es seleccionen despeses,no es fa res
            if (llistaDespesesTancades.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona una despesa, s'inicia la crida asincrona per carregar el comentri, donat el id de la despesa
            else if (llistaDespesesTancades.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.buscarComentariDeDespesaCompleted += serveiBuscarComentariDeDespesaCompleted;
                servei.buscarComentariDeDespesaAsync(llistaDespesesTancades[0]);
            }
            //Nomes es pot obrir un comentari a l'hora
            else if (llistaDespesesTancades.Count > 1)
            {
                MessageBox.Show(Missatges.UN_DESPESA);
            }
        }
        #endregion

        #region Buscar comentari de despesa finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona perb uscar el comentari d'un full.
         * Mostra el comentari a una pagina nova
         */
        private void serveiBuscarComentariDeDespesaCompleted(object sender, ServeiWeb.buscarComentariDeDespesaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.COMENTARI_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else if (e.Result != null)
            {
                NavigationService.Navigate(new Uri("/VisioComentari.xaml?comentari=" + e.Result, UriKind.Relative));
            }
        }
        #endregion

        #region Obrir factura
        /*
         * Metode que inicia la crida asincrona per obrir una factura adjunta
         */
        private void clickObrirFactura(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Despesa.despesesTancadesSeleccionades(scrollDespeses);
            //Si no es seleccionen despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona una despesa es busca el nom de la factura
            else if (llista.Count == 1)
            {
                despesaId = llista[0];
                servei = ConnexioServei.connexioNova();
                servei.verificarAdjuntCompleted += serveiVerificarAdjuntCompleted;
                servei.verificarAdjuntAsync(llista[0]);
            }
            //Nomes es pot obrir l'adjunt d'una despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FACTURA);
            }
        }
        #endregion

        #region Verificar adjunt
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per verificar una factura adjunta
         */
        private void serveiVerificarAdjuntCompleted(object sender, ServeiWeb.verificarAdjuntCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if ((bool)e.Result)
            {
                servei = ConnexioServei.connexioNova();
                servei.buscarNomDeFacturaCompleted += serveiBuscarNomDeFacturaCompleted;
                servei.buscarNomDeFacturaAsync(despesaId);
            }
            else
            {
                //No hi ha factura
            }
        }
        #endregion

        #region Buscar nom de factura finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar el nom d'una factura adjunta
         */
        private void serveiBuscarNomDeFacturaCompleted(object sender, ServeiWeb.buscarNomDeFacturaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if (!e.Result.Equals(string.Empty))
            {
                nomFactura = e.Result;
                servei = ConnexioServei.connexioNova();
                servei.buscarAdjuntDeDespesaCompleted += serveiBuscarAdjuntDeDespesaCompleted;
                servei.buscarAdjuntDeDespesaAsync(despesaId);
            }
        }
        #endregion

        #region Buscar factura finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar una factura adjunta
         */
        private void serveiBuscarAdjuntDeDespesaCompleted(object sender, ServeiWeb.buscarAdjuntDeDespesaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if (e.Result != null)
            {
                Accions.Factura.ObrirAdjunt(nomFactura, e.Result);
            }
        }
        #endregion
    }
}