﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class GestioFullsPersonals : PhoneApplicationPage
    {

        private Int32 usuariActual, fullId;
        private ServeiWeb.ServeiClient servei;
        private bool fullNou = false;
        private bool pagamentFull = false;

        public GestioFullsPersonals()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on es realitza la navegacio.
         * Inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                usuariActual = Convert.ToInt32(NavigationContext.QueryString["usuariId"]);
                iniciarFullPendents();
                carregarFullsTancats();
            }
            else
            {
                //Si s'ha retornar desde la pagina de notificacions per obrir full nou i s'ha acceptat
                if (fullNou && Notificacio.estat)
                {
                    obrirFullNou();
                }
                //Si s'ha retornar desde la pagina de notificacions per demanar el pagament de les despeses i s'ha acceptat
                else if (pagamentFull && Notificacio.estat)
                {
                    solicitarPagament();
                }
                else
                {
                    iniciarFullPendents();
                    carregarFullsTancats();
                }
                //Es reinicien les variables 
                fullNou = false;
                pagamentFull = false;
                Notificacio.estat = false;
            }
        }
        #endregion

        #region Buscar full obert
        /*
         * Metode que inicia la càrrega de les dades d'un full de despeses.
         * Inicia la crida a l'event per buscar el full obert actual,donat el id d'un usuari
         */
        private void iniciarFullPendents()
        {
            botoObrirFullNou.Visibility = Visibility.Collapsed;
            botoObrirDespeses.Visibility = Visibility.Collapsed;
            botoSolicitarPagament.Visibility = Visibility.Collapsed;
            servei = ConnexioServei.connexioNova();
            servei.buscarFullObertCompleted += serveiBuscarFullObertCompleted;
            servei.buscarFullObertAsync(usuariActual);
        }
        #endregion

        #region Buscar full obert finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar un full obert
         */
        private void serveiBuscarFullObertCompleted(object sender, ServeiWeb.buscarFullObertCompletedEventArgs e)
        {
            servei.CloseAsync();
            if(e.Error!=null){
                MessageBox.Show(Missatges.FULL_BUSCAR_OBERT_ERROR);
                NavigationService.GoBack();
            }
            //Hi ha un full obert
            else if (e.Result != null)
            {
                botoObrirDespeses.Visibility = Visibility.Visible;
                labelObert.Visibility = Visibility.Visible;
                textObert.Visibility = Visibility.Visible;
                textObert.Text = e.Result.Obert.ToString();
                labelImportTotal.Visibility = Visibility.Visible;
                textImportTotal.Visibility = Visibility.Visible;
                fullId = e.Result.Id;
                textImportTotal.Text = e.Result.Total.ToString();
                //Pendent de sol·licitar pagament
                if (e.Result.Solicitat == null)
                {
                    botoSolicitarPagament.Visibility = Visibility.Visible;
                }
                //Pagament sol·licitat
                else
                {
                    labelSolicitat.Visibility = Visibility.Visible;
                    textSolicitat.Visibility = Visibility.Visible;
                    textSolicitat.Text = e.Result.Solicitat.ToString();
                    labelEstat.Visibility = Visibility.Visible;
                    textEstat.Visibility = Visibility.Visible;
                    labelValidat.Visibility = Visibility.Visible;
                    textValidat.Visibility = Visibility.Visible;
                    //Pendent de validar
                    if (e.Result.Estat == null)
                    {
                        textEstat.Text = "Pendent";
                        textValidat.Text = "Pendent";
                    }
                    //Validat
                    else
                    {
                        textValidat.Text = e.Result.Validat.ToString();
                        if (!(bool)e.Result.Estat)
                        {
                            textEstat.Text = "Denegat";

                        }
                        else
                        {
                            textEstat.Text = "Acceptat";
                            textPagat.Text = "Pendent";
                            labelPagat.Visibility = Visibility.Visible;
                            textPagat.Visibility = Visibility.Visible;
                        }
                    }
                }
            }
            //No hi ha full obert
            else
            {
                botoObrirFullNou.Visibility = Visibility.Visible;
            }
        }
        #endregion

        #region Solicitat full nou
        /*
         * Event que mostra la pantalla per confirmar que es vol obrir un nou full de despeses
         */
        private void clickObrirFullNou(object sender, RoutedEventArgs e)
        {
            fullNou = true;
            NavigationService.Navigate(new Uri("/Notificacio.xaml?notificacio=Només es pot tindre un full obert, segur que vols continuar?", UriKind.Relative));
        }
        #endregion

        #region Obrir full nou
        /*
         * Metode que inicia la crida asincrona per obrir un nou full de despeses
         */
        private void obrirFullNou()
        {
            servei = ConnexioServei.connexioNova();
            servei.obrirFullNouCompleted += serveiObrirFullNouCompleted;
            servei.obrirFullNouAsync(usuariActual);
        }
        #endregion

        #region Obrir full nou finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per obrir un nou full de despeses
         */
        private void serveiObrirFullNouCompleted(object sender, ServeiWeb.obrirFullNouCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FULL_OBRIR_NOU_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                iniciarFullPendents();
            }
        }
        #endregion

        #region Solicitat pagament de full
        /*
         * Metode mostra la pantalla per verificar que es vol sol·licitar el pagament del full de despeses
         */
        private void clickSolicitarPagament(object sender, RoutedEventArgs e)
        {
            pagamentFull = true;
            NavigationService.Navigate(new Uri("/Notificacio.xaml?notificacio=Si es sol·licita el pagament no es podran realitzar més operacions sobre el full.", UriKind.Relative));
        }
        #endregion

        #region Solicitar pagament
        /*
         * Metode que inicia la crida asincrona per sol·licitar el pagament d'un full de despeses,donat el seu id
         */
        private void solicitarPagament()
        {
            servei = ConnexioServei.connexioNova();
            servei.solicitarPagamentFullDespesesCompleted += serveiSolicitarPagamentFullDespesesCompleted;
            servei.solicitarPagamentFullDespesesAsync(fullId);
        }
        #endregion

        #region Solicitar pagament finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per sol·Licitar el pagament del full de despeses
         */
        private void serveiSolicitarPagamentFullDespesesCompleted(object sender, ServeiWeb.solicitarPagamentFullDespesesCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FULL_SOLICITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                iniciarFullPendents();
            }
        }
        #endregion

        #region Obrir despeses de full
        /*
         * Event que obre la pagina per visualitzar les desepses d'un full,enviat el id del full i indicant si es pot modificar o no
         */
        private void clickObrirDespeses(object sender, RoutedEventArgs e)
        {
            string modificar = "false";
            if (textSolicitat.Text.Length == 0)
            {
                modificar = "true";
            }
            NavigationService.Navigate(new Uri(string.Format("/GestionarDespeses/GestioDespesesPersonals.xaml?fullId=" + fullId + "&modificar=" + modificar), UriKind.Relative));
        }
        #endregion

        #region Buscar fulls tancats
        /*
         * Metode que inicia la crida asincrona per buscar els fulls tancats d'un usuari, donat el seu id
         */
        private void carregarFullsTancats()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistaFullsTancatsCompleted += serveiLlistaFullsTancatsCompleted;
            servei.llistaFullsTancatsAsync(usuariActual);
        }
        #endregion

        #region Buscar fulls tancats finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar els fulls tancats
         */
        private void serveiLlistaFullsTancatsCompleted(object sender, ServeiWeb.llistaFullsTancatsCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null && e.Result.Count > 0)
            {
                scrollFullsTancats.Content = FullDespeses.llistaFullsTancats(e.Result);
            }
        }
        #endregion

        #region Obrir comentari
        /*
         * Event que verifica la despesa seleccionada.
         * Inicia la crida asincrona per veure el comentari d'un full de despeses
         */
        private void clickVeureComentari(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaSeleccionats = FullDespeses.fullsTancatsSeleccionats(scrollFullsTancats);
            //Si no es selecciona cap comentari o no hi han fulls tancats no es fa res
            if (llistaSeleccionats.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona una despesa es realitza la crida asincrona
            else if (llistaSeleccionats.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.comentariDeFullCompleted += serveiComentariDeFullCompleted;
                servei.comentariDeFullAsync(llistaSeleccionats[0]);
            }
            //Nomes es pot visualitzar un comentari a l'hora
            else if (llistaSeleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FULL);
            }
        }
        #endregion

        #region Obrir comentari finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar el comentari d'un full de despeses.
         * Mostra el comentari a una pagina diferent, enviant el comentari per parametre
         */
        private void serveiComentariDeFullCompleted(object sender, ServeiWeb.comentariDeFullCompletedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/VisioComentari.xaml?comentari=" + e.Result, UriKind.Relative));
        }
        #endregion

        #region Obrir despedes
        /*
         * Event que revisa el full de desepses seleccionat i obre la pagina per visualitza les despeses associades
         */
        private void clickObrirDespesesTancades(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaSeleccionades = FullDespeses.fullsTancatsSeleccionats(scrollFullsTancats);
            //Si no es seleccionan despeses no es fa res
            if (llistaSeleccionades.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona un full de despeses
            else if (llistaSeleccionades.Count == 1)
            {
                NavigationService.Navigate(new Uri(string.Format("/GestionarDespeses/GestioDespesesTancades.xaml?fullId=" + llistaSeleccionades[0]), UriKind.Relative));
            }
            //Nomeses poden visualitzar les despeses d'un full a l'hora
            else if (llistaSeleccionades.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FULL);
            }
        }
        #endregion
    }
}