﻿using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using System.IO.IsolatedStorage;
using System;

namespace ClientSGD
{
    public partial class LoginMenu : PhoneApplicationPage
    {

        public static string usuariDni { get; set; }
        private int max = 0;
        private ServeiWeb.ServeiClient servei;

        public LoginMenu()
        {
            InitializeComponent();
            usuariDni = string.Empty;
        }

        #region Navegacio
        /*
         * Event que revisa des de on s'arriba a la pagina
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                IsolatedStorageSettings config = IsolatedStorageSettings.ApplicationSettings;
                if (!config.Contains("IpServei"))
                {
                    config.Add("IpServei", "");
                    config.Add("PublicacioServei", "");
                    config.Add("UrlCompletaServei", "");
                    config.Save();
                }
                verificarServeiWeb();
            }
            //Si entra es per que la connexio al servei web fallaba i s'arriba des de la pantalla de configuracio
            else
            {
                
                if (Configuracio.estatWCF)
                {
                    botoGuardar.IsEnabled = true;
                    botoCancelar.IsEnabled = true;
                    textUsuari.IsEnabled = true;
                    textContrasenya.IsEnabled = true;
                }
                else
                {
                    MessageBox.Show(Missatges.CONNEXIO_OBLIGATORIA + Environment.NewLine + Missatges.TANCAR);
                    Application.Current.Terminate();
                }
            }
        }
        #endregion

        #region Verificar connexio amb el servei web
        /*
         * Metode que inicia una crida asincrona per verificar si hi ha connexi oamb el servei web
         */
        private void verificarServeiWeb()
        {
            try
            {
                servei = ConnexioServei.connexioNova();
                servei.validarUsuariCompleted += serveiVerificarServeiWeb;
                servei.validarUsuariAsync(string.Empty,string.Empty);
                blockConnexio.Text = Missatges.VERIFICAR_SERVEI;
            }
            catch
            {
                //Si falla la connexio, s'obre la finestra de configuracio
                NavigationService.Navigate(new Uri("/Configuracio.xaml", UriKind.Relative));
            }
        }
        #endregion

        #region Verificar connexio amb servei web finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per verificar la connexio amb el servei web
         */
        private void serveiVerificarServeiWeb(object sender, ServeiWeb.validarUsuariCompletedEventArgs e)
        {
            servei.CloseAsync();
            blockConnexio.Text = string.Empty;
            if (e.Error != null)
            {
                NavigationService.Navigate(new Uri("/Configuracio.xaml", UriKind.Relative));
            }
            else
            {
                botoGuardar.IsEnabled = true;
                botoCancelar.IsEnabled = true;
                textUsuari.IsEnabled = true;
                textContrasenya.IsEnabled = true;
            }
        }
        #endregion

        #region Verificar usuari i contrasenya
        /*
         * Event que verifica que hi hagi usuari i contrasenya.
         * Inicia la crida asincrona per verificar si es un usuari valid
         */
        private void clickAcceptar(object sender, RoutedEventArgs e)
        {
            if (textUsuari.Text.Length == 0 || textContrasenya.Password.Length == 0)
            {
                MessageBox.Show(Missatges.LOGIN_OBLIGATORI);
            }
            else
            {
                servei = ConnexioServei.connexioNova();
                servei.validarUsuariCompleted += serveiValidarUsuariCompleted;
                servei.validarUsuariAsync(textUsuari.Text, textContrasenya.Password);
            }
        }
        #endregion

        #region Verificar usuari i contrasenya finalitzat
        /*
         * Event posterior a la crida asincrona per verifica si usuari i contrasenya son correctes
         */
        private void serveiValidarUsuariCompleted(object sender, ServeiWeb.validarUsuariCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR);
            }
            //Si l'usuari es correcte es guarda el seu DNI i es torna al menu principal
            else if (e.Result)
            {
                usuariDni = textUsuari.Text;
                NavigationService.GoBack();
            }
            //Si usuari o contrasenya son incorrectes
            else
            {
                MessageBox.Show(Missatges.LOGIN_INCORRECTE + "\n" + Missatges.REPETIR);
                //Si es falla l'autentificacio 3 cops es tanca l'aplicacio
                if (max == 2)
                {
                    Application.Current.Terminate();
                }
                max++;
            }
        }
        #endregion

        #region Tornar
        /*
         * Event per tornar al menu principal sense validar-se
         */
        private void clickCancelar(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();
        }
        #endregion
    }
}