﻿using System;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using System.Globalization;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class MenuPrincipal : PhoneApplicationPage
    {
        private Usuari usuari;
        private ServeiWeb.ServeiClient servei;

        public MenuPrincipal()
        {
            //Es canvia l'idioma per defecte del mobil al Catala
            try
            {
                System.Threading.Thread.CurrentThread.CurrentCulture = new CultureInfo("ca-ES");
            }
            catch
            {
                MessageBox.Show(Missatges.IDIOMA_ERROR);
            }
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que revisa des de on s'arriba a la pagina i si hi ha connexio amb internet.
         * Inicia la validacio d'usuari i els accesos que tindra a l'aplicacio depenent els permisos
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            try
            {
                //Si s'inicia la pagina per primer cop, cal identificar-se a l'aplicació
                if (e.NavigationMode != NavigationMode.Back)
                {
                    try
                    {
                        if (!ConnexioServei.verificarConnexioInternet())
                        {
                            MessageBox.Show(Missatges.INTERNET_ERROR + Environment.NewLine + Missatges.TANCAR);
                            Application.Current.Terminate();
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                        Application.Current.Terminate();
                    }

                    NavigationService.Navigate(new Uri("/LoginMenu.xaml", UriKind.Relative));
                }
                //Si no s'inicia la pagina per primer cop
                else
                {
                    //Si no hi ha un usuari
                    if (usuari == null)
                    {
                        //Es que no s'ha identificat correctament
                        if (LoginMenu.usuariDni.Equals(string.Empty))
                        {
                            Application.Current.Terminate();
                        }
                        //S'ha identificat i cal carregar les dades
                        else
                        {
                            string dni = LoginMenu.usuariDni;
                            servei = ConnexioServei.connexioNova();
                            servei.buscarUsuariPerDniCompleted += serveiBuscarUsuariPerDniCompleted;
                            //S'inicia la crida asincrona per buscar un usuari,donat el seu id
                            servei.buscarUsuariPerDniAsync(dni);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Buscar usuari del sistema
        /*
         * Event posterior a la finalitzacio d'una crida asincrona per buscar un usuari pel seu id
         */
        private void serveiBuscarUsuariPerDniCompleted(object sender, ServeiWeb.buscarUsuariPerDniCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR + Environment.NewLine + Missatges.TANCAR);
                Application.Current.Terminate();
            }
            //Si hi ha usuari es busca el seu carrec amb una crida asincrona
            else if (e.Result != null)
            {
                usuari = new Usuari(e.Result.Id, e.Result.Dni, e.Result.Nom, e.Result.Cognoms, e.Result.IdCarrec, e.Result.IdResponsable);
                servei = ConnexioServei.connexioNova();
                servei.buscarCarrecPerIdCompleted += serveiBuscarCarrecPerIdCompleted;
                servei.buscarCarrecPerIdAsync(usuari.CarrecId);
            }
        }
        #endregion

        #region Buscar usuari del sistema finalitzat
        /*
         * Event posterior a la finalitzacio d'una crida asincrona per buscar el carrec d'un usuari
         */
        private void serveiBuscarCarrecPerIdCompleted(object sender, ServeiWeb.buscarCarrecPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            //Si hi ha carrec i te permisos, cal buscar-los
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR + Environment.NewLine + Missatges.TANCAR);
                Application.Current.Terminate();
            }
            else if (e.Result != null)
            {
                usuari.Carrec = new Carrec(usuari.CarrecId, e.Result.Nom, e.Result.IdPermisos);
                servei = ConnexioServei.connexioNova();
                //Tots els usuaris poden accedir a les despeses personals i al calendari
                botoGestioDespeses.IsEnabled = true;
                botoCalencari.IsEnabled = true;
                if (usuari.Carrec.PermisosId != 0)
                {
                    servei.buscarPermisosPerIdCompleted += serveiBuscarPermisosPerIdCompleted;
                    servei.buscarPermisosPerIdAsync((Int32)usuari.Carrec.PermisosId);
                }
            }
        }
        #endregion

        #region Buscar permisos finalitzat
        /*
         * Event posterior a la finalitzacio d'una crida asincrona per buscar els permisos d'un carrec
         */
        private void serveiBuscarPermisosPerIdCompleted(object sender, ServeiWeb.buscarPermisosPerIdCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.CONSULTA_ERROR + Environment.NewLine + Missatges.TANCAR);
                Application.Current.Terminate();
            }
            //Si hi han permisos
            if (e.Result != null)
            {
                usuari.Carrec.Permisos = new Permisos(e.Result.Id, e.Result.Nom, e.Result.Administrar, e.Result.Validar, e.Result.VeureUsuaris, e.Result.VeureDespeses);
                //Nomes administradors i tecnics poden accedir a la administracio
                if (usuari.Carrec.Permisos.Administrar || (usuari.Carrec.Permisos.VeureUsuaris != null && (bool)usuari.Carrec.Permisos.VeureUsuaris))
                {
                    botoAdministrar.IsEnabled = true;
                }
                //Nomes els responsables poden validar despeses
                if (usuari.Carrec.Permisos.Validar)
                {
                    botoValidarDespeses.IsEnabled = true;
                }
                //Nomes els financers poden pagar despeses
                if (usuari.Carrec.Permisos.VeureDespeses != null && (bool)usuari.Carrec.Permisos.VeureDespeses)
                {
                    botoPagarDespeses.Visibility = Visibility.Visible;
                }
            }
        }
        #endregion

        #region Acces a gestio de despeses personals
        /*
         * Event que inicia la pagina de gestio de despeses personals
         */
        private void clickGestioDespeses(object sender, RoutedEventArgs e)
        {
            try
            {
                NavigationService.Navigate(new Uri("/GestionarDespeses/GestioFullsPersonals.xaml?usuariId=" + usuari.IdUsuari, UriKind.Relative));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Acces a validacio de despeses
        /*
         * Event que inicia la pagina de validacio de fulls pendents
         */
        private void clickValidar(object sender, RoutedEventArgs e)
        {
            try
            {
                NavigationService.Navigate(new Uri("/Validar/ValidarFullsDespeses.xaml?usuariId=" + usuari.IdUsuari, UriKind.Relative));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Acces a l'administracio
        /*
         * Event que inicia la pagina d'administracio
         */
        private void clickAdministrar(object sender, RoutedEventArgs e)
        {
            try
            {
                NavigationService.Navigate(new Uri("/Administrar/Administracio.xaml?administrador=" + usuari.Carrec.Permisos.Administrar, UriKind.Relative));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Acces al calendari
        /*
         * Event que inicia la pagina del calendari
         */
        private void clickCalendari(object sender, RoutedEventArgs e)
        {
            try
            {
                if (usuari.Carrec.PermisosId != 0 && usuari.Carrec.Permisos.Validar)
                {
                    NavigationService.Navigate(new Uri("/Calendari.xaml?responsable=true", UriKind.Relative));
                }
                else
                {
                    NavigationService.Navigate(new Uri("/Calendari.xaml?responsable=false", UriKind.Relative));
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Acces al pagament de despeses
        /*
         * Event que inicia la pagina per pagar despeses
         */
        private void clickPagarDespeses(object sender, RoutedEventArgs e)
        {
            try
            {
                NavigationService.Navigate(new Uri("/PagarDespeses.xaml", UriKind.Relative));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion
    }
}