﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class PagarDespeses : PhoneApplicationPage
    {
        private ServeiWeb.ServeiClient servei;

        public PagarDespeses()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event inicial en carregar la pagina que inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            carregarFullsPendents();
        }
        #endregion

        #region Buscar fulls pendents de pagament
        /*
         * Event que inicia la crida asincrona per buscar els fulls pendents de pagament i validats
         */
        private void carregarFullsPendents()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarFullsPendentsPagarCompleted += serveiLlistarFullsPendentsPagarCompleted;
            servei.llistarFullsPendentsPagarAsync();
        }
        #endregion

        #region Buscar fulls pendents de pagament finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar els fulls pendents de pagament i validats
         */
        private void serveiLlistarFullsPendentsPagarCompleted(object sender, ServeiWeb.llistarFullsPendentsPagarCompletedEventArgs e)
        {
            servei.CloseAsync();
            if(e.Error!=null){
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                scrollDespeses.Content = FullDespeses.llistaFullsPendentsDePagament(e.Result);
            }
        }
        #endregion

        #region Pagar fulls seleccionats
        /*
         * Event que inicia la crida asincrona per pagar els fulls seleccionats
         */
        private void clickPagar(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaFulls = FullDespeses.fullsPendentsDePagamentSeleccionats(scrollDespeses);
            //Si no es seleccionen fulls
            if (llistaFulls.Count == 0)
            {
                //No fer res
            }
            //Si com a minim s'ha seleccionat un full
            else if (llistaFulls.Count > 0)
            {
                servei = ConnexioServei.connexioNova();
                servei.pagarFullsDespesesPendentsCompleted += serveiPagarFullsDespesesPendentsCompleted;
                servei.pagarFullsDespesesPendentsAsync(llistaFulls);
            }
        }
        #endregion

        #region Pagar fulls seleccionats finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per pagar fulls de despeses
         */
        private void serveiPagarFullsDespesesPendentsCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if(e.Error!=null)
            {
                MessageBox.Show(Missatges.FULLS_PAGAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else 
            {
                carregarFullsPendents();
            }
        }
        #endregion
    }
}