﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class ValidarDespeses : PhoneApplicationPage
    {
        private Int32 fullId, facturaId;
        private string nomFactura;
        private List<DateTime> llistaDatesFestives;
        private ServeiWeb.ServeiClient servei;

        public ValidarDespeses()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'arriba a la pagina.
         * Inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                fullId = Convert.ToInt32(NavigationContext.QueryString["fullId"]);
            }
            iniciarCarregarDespeses();
        }
        #endregion

        #region Buscar despeses pendents
        /*
         * Metode que inicia la crida asincrona per buscar les despeses.
         * Primer s'obte un llistat de les dates festives
         */
        private void iniciarCarregarDespeses()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistarDatesFestivesCompleted += serveiLlistarDatesFestivesCompleted;
            servei.llistarDatesFestivesAsync();
        }
        #endregion

        #region Llista de dates festives finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar les dates festives.
         * S'inicia la carrega de les despeses del full
         */
        private void serveiLlistarDatesFestivesCompleted(object sender, ServeiWeb.llistarDatesFestivesCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                llistaDatesFestives = e.Result;
                carregarDespeses();
            }
        }
        #endregion

        #region Llista de despeses
        /*
         * Metode que inicia la crida asincrona per visualitzar les despeses d'un full, donat el id del full
         */
        private void carregarDespeses()
        {
            servei = ConnexioServei.connexioNova();
            servei.llistaDespesesDefullCompleted += serveiBuscarDespesesDeFullCompleted;
            servei.llistaDespesesDefullAsync(fullId);
        }
        #endregion

        #region Llista de despeses finalitzada
        /*
         * Event posterior a la finalitzacio de buscar les despeses d'un full
         */
        private void serveiBuscarDespesesDeFullCompleted(object sender, ServeiWeb.llistaDespesesDefullCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            else if (e.Result != null)
            {
                scrollDespesesPendents.Content = Despesa.llistaGridDespesesPendents(e.Result, llistaDatesFestives);
            }
        }
        #endregion

        #region Validar despeses
        /*
         * Event que inicia la crida asincrona per validar les despeses seleccionades
         */
        private void clickValidarDespeses(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaDespesesPendents = Despesa.despesesPendentsSeleccionades(scrollDespesesPendents);
            //Si no es seleccionen despeses no es fa res
            if (llistaDespesesPendents.Count == 0)
            {
                //No fer res
            }
            //Si com a minim s'ha seleccionat una despesa
            else if (llistaDespesesPendents.Count > 0)
            {
                servei = ConnexioServei.connexioNova();
                servei.acceptarDespesesCompleted += serveiAcceptarDespesesCompleted;
                servei.acceptarDespesesAsync(llistaDespesesPendents);
            }
        }
        #endregion

        #region Validar despeses finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per acceptar una despesa
         */
        private void serveiAcceptarDespesesCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.DESPESA_ACCEPTAR_ERROR);
                NavigationService.GoBack();
            }
            else
            {
                carregarDespeses();
            }
        }
        #endregion

        #region Rebutjar despeses
        /*
         * Event que inicia la crida asincrona per denegar les despeses seleccionades
         */
        private void clickRebutjarDespesa(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaDespesesPendents = Despesa.despesesPendentsSeleccionades(scrollDespesesPendents);
            //Si no es seleccionen despeses no es fa res
            if (llistaDespesesPendents.Count == 0)
            {
                //No fer res
            }
            //Si com a minim s'ha seleccionat una despesa
            else if (llistaDespesesPendents.Count > 0)
            {
                servei = ConnexioServei.connexioNova();
                servei.denegarDespesesCompleted += serveiDenegarDespesesCompleted;
                servei.denegarDespesesAsync(llistaDespesesPendents);
            }
        }
        #endregion

        #region Rebutjar despeses finalitzat
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per denegar una despesa
         */
        private void serveiDenegarDespesesCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.DESPESA_DENEGAR_ERROR);
                NavigationService.GoBack();
            }
            else
            {
                carregarDespeses();
            }
        }
        #endregion

        #region Llista de despeses pendents finalitzada
        /*
         * Event que inicia la crida asincrona per obre el comentari d'una despesa, donat el seu id
         */
        private void clickVeureComentari(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaDespesesPendents = Despesa.despesesPendentsSeleccionades(scrollDespesesPendents);
            //Si no es seleccionen despeses no es fa res
            if (llistaDespesesPendents.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat una despesa
            else if (llistaDespesesPendents.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.buscarComentariDeDespesaCompleted += serveiBuscarComentariDeDespesaCompleted;
                servei.buscarComentariDeDespesaAsync(llistaDespesesPendents[0]);
            }
            //Nomes es pot obrir un comentari a l'hora
            else if (llistaDespesesPendents.Count > 1)
            {
                MessageBox.Show(Missatges.UN_DESPESA);
            }
        }
        #endregion

        #region Buscar comentari
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar el comentari d'una despesa, donat el seu id.
         * El comentari es mostra a una pagina diferent
         */
        private void serveiBuscarComentariDeDespesaCompleted(object sender, ServeiWeb.buscarComentariDeDespesaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.COMENTARI_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else if (e.Result != null)
            {
                NavigationService.Navigate(new Uri("/VisioComentari.xaml?comentari=" + e.Result, UriKind.Relative));
            }
        }
        #endregion

        #region Buscar factura adjunta
        /*
         * Event per obrir la factura d'una despesa,donat el id de la despesa
         */
        private void clickObrirFactura(object sender, RoutedEventArgs e)
        {
            List<Int32> llista = Despesa.despesesPendentsSeleccionades(scrollDespesesPendents);
            //Si no es seleccionen despeses no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona una despesa es busca el nom de la factura
            else if (llista.Count == 1)
            {
                facturaId = llista[0];
                servei = ConnexioServei.connexioNova();
                servei.verificarAdjuntCompleted += serveiVerificarAdjuntCompleted;
                servei.verificarAdjuntAsync(llista[0]);
            }
            //Nomes es pot obrir l'adjunt d'una despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FACTURA);
            }
        }
        #endregion

        #region Verificar adjunt de despesa
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per verificar si una despesa te adjunt
         */
        private void serveiVerificarAdjuntCompleted(object sender, ServeiWeb.verificarAdjuntCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if ((bool)e.Result)
            {
                servei = ConnexioServei.connexioNova();
                servei.buscarNomDeFacturaCompleted += serveiBuscarNomDeFacturaCompleted;
                servei.buscarNomDeFacturaAsync(facturaId);
            }
            else
            {
                //No hi ha factura
            }
        }
        #endregion

        #region Buscar nom de factura
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar el nom d'una factura adjunta
         */
        private void serveiBuscarNomDeFacturaCompleted(object sender, ServeiWeb.buscarNomDeFacturaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if (!e.Result.Equals(string.Empty))
            {
                if (e.Result.EndsWith(".jpg") || e.Result.EndsWith(".bmp") || e.Result.EndsWith(".gif") || e.Result.EndsWith(".jpeg") || e.Result.EndsWith(".png") || e.Result.EndsWith(".tif")
                    || e.Result.EndsWith(".tiff") || e.Result.EndsWith(".jfif") || e.Result.EndsWith(".jpe") || e.Result.EndsWith(".dib") || e.Result.EndsWith(".pdf"))
                {
                    nomFactura = e.Result;
                    servei = ConnexioServei.connexioNova();
                    servei.buscarAdjuntDeDespesaCompleted += serveiBuscarAdjuntDeDespesaCompleted;
                    servei.buscarAdjuntDeDespesaAsync(facturaId);
                }
                else
                {
                    MessageBox.Show(Missatges.FACTURA_FORMAT);
                }
            }
        }
        #endregion

        #region Buscar factura adjunta
        /*
         * Funcio posterior a la finalitzacio de la crida asincrona per buscar una factura adjunta
         */
        private void serveiBuscarAdjuntDeDespesaCompleted(object sender, ServeiWeb.buscarAdjuntDeDespesaCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FACTURA_ERROR);
            }
            else if (e.Result != null)
            {
                Accions.Factura.ObrirAdjunt(nomFactura, e.Result);
            }
        }
        #endregion
    }
}