﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using ClientSGD.Resources;
using ClientSGD.Accions;

namespace ClientSGD
{
    public partial class ValidarFullsDespeses : PhoneApplicationPage
    {
        private Int32 responsableId;
        private ServeiWeb.ServeiClient servei;

        public ValidarFullsDespeses()
        {
            InitializeComponent();
        }

        #region Navegacio
        /*
         * Event que verifica des de on s'arriba a la pagina.
         * Inicia la carrega de dades
         */
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            if (e.NavigationMode != NavigationMode.Back)
            {
                responsableId = Convert.ToInt32(NavigationContext.QueryString["usuariId"]);
            }
            carregarFullsPendents();
            carregarTreballadors();
        }
        #endregion

        #region Buscar fulls pendents de validar
        /*
         * Metode que inicia la crida asincrona per buscar les despeses d'un full pendent de validar
         */
        private void carregarFullsPendents()
        {
            servei = ConnexioServei.connexioNova();
            servei.buscarFullsPendentsDeResponsableCompleted += serveiFullDespesesPendentsCompleted;
            servei.buscarFullsPendentsDeResponsableAsync(responsableId);
        }
        #endregion

        #region Buscar fulls pendents de validacio finalitzat
        /*
         * Event posterior a la finalitzacio de la crida sincrona per buscar les despeses d'un full pendent de validar
         */
        private void serveiFullDespesesPendentsCompleted(object sender, ServeiWeb.buscarFullsPendentsDeResponsableCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                NavigationService.GoBack();
            }
            //Si hi han despeses es mostren al llistat
            else if (e.Result != null)
            {
                scrollFullsPendents.Content = FullDespeses.llistaFullsPendents(e.Result);
            }
        }
        #endregion

        #region Obrir despeses de full
        /*
         * Event que revisa el full seleccionat i obre la pagina per validar despeses
         */
        private void clickObrirDespeses(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaPendents = FullDespeses.fullsPendentsSeleccionats(scrollFullsPendents);
            //Si no es seleccionen fulls no es fa res
            if (llistaPendents.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona un full s'obre la pagina per veure les despeses, enviat per parametre el id del full
            else if (llistaPendents.Count == 1)
            {
                NavigationService.Navigate(new Uri(string.Format("/Validar/ValidarDespeses.xaml?fullId=" + llistaPendents[0]), UriKind.Relative));
            }
            //Nomes es poden obrir les despeses d'un full a l'hora
            else if (llistaPendents.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FULL);
            }
        }
        #endregion

        #region Validar pagament de full
        /*
         * Event que inicia la crida asincrona per validar un full de despeses pendent
         */
        private void clickValidarFull(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaPendents = FullDespeses.fullsPendentsSeleccionats(scrollFullsPendents);
            //Si no es seleccionen fulls no es fa res
            if (llistaPendents.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona un full s'inicia la crida asincrona
            else if (llistaPendents.Count == 1)
            {
                servei = ConnexioServei.connexioNova();
                servei.acceptarPagamentFullCompleted += serveiAcceptarPagamentFullCompleted;
                servei.acceptarPagamentFullAsync(llistaPendents[0]);
            }
            //Nomes es pot validar un full a l'hora
            else if (llistaPendents.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FULL);
            }
        }
        #endregion

        #region Validar pagament de full finalitzat
        /*
         * Event posterior a la finalitzacio de la crida sincrona per acceptar el pagament d'un full
         */
        private void serveiAcceptarPagamentFullCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.FULL_VALIDAR_ERROR + Environment.NewLine + Missatges.REPETIR);
            }
            else
            {
                carregarFullsPendents();
            }
        }
        #endregion

        #region Rebutjar pagament de full
        /*
         * Event donat un full seleccionat per rebutjar, obre la pagina per afegir el comentari
         */
        private void clickRebutjarFull(object sender, RoutedEventArgs e)
        {
            List<Int32> llistaPendents = FullDespeses.fullsPendentsSeleccionats(scrollFullsPendents);
            //Si no es seleccionen fulls no es fa res
            if (llistaPendents.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona un full s'obre la pagina per afegir el comentari i s'envia per parametre el id del full
            else if (llistaPendents.Count == 1)
            {
                NavigationService.Navigate(new Uri(string.Format("/Validar/ComentariDenegarFull.xaml?fullId=" + llistaPendents[0]), UriKind.Relative));
            }
            //Nomes es pot rebutjar un full a l'hora
            else if (llistaPendents.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FULL);
            }
        }
        #endregion

        #region Rebutjar pagament de full finalitzat
        /*
         * Metode que inicia la crida asincrona per buscar els treballadors d'un responsable
         */
        private void carregarTreballadors()
        {
            servei = ConnexioServei.connexioNova();
            servei.usuarisDeResponsableCompleted += serveiUsuarisDeResponsableCompleted;
            servei.usuarisDeResponsableAsync(responsableId);
        }
        #endregion

        #region Buscar treballadors de responsable finalitzat
        /*
         * Event posterior a la finalitzacio de la crida sincrona per buscar els treballadors d'un responsable
         */
        private void serveiUsuarisDeResponsableCompleted(object sender, ServeiWeb.usuarisDeResponsableCompletedEventArgs e)
        {
            servei.CloseAsync();
            if (e.Error != null)
            {
                MessageBox.Show(Missatges.LLISTAT_ERROR);
                //Si falla, deixar el llistat sense responsables
            }
            else if (e.Result != null)
            {
                scrollTreballadors.Content = Usuari.llistaGridTreballadors(e.Result);
            }
        }
        #endregion
    }
}