--CREACIO BBDD
CREATE DATABASE GestioDespesesPersonals;
GO

--CREACIO USUARI DE BBDD
CREATE LOGIN SGDAdmin WITH PASSWORD='NetSGD', DEFAULT_DATABASE=[GestioDespesesPersonals]
GO

--PERMISOS DE USUARI DE BBDD
EXEC master..sp_addsrvrolemember @loginame = 'SGDAdmin', @rolename = 'sysadmin'
GO


--CREACIO DE TAULES
USE GestioDespesesPersonals;
GO

--**************PERMISOS**************
CREATE TABLE Permisos(
	id int identity(1,1) CONSTRAINT PK_permisos_id PRIMARY KEY,
	nom nvarchar(25) CONSTRAINT NN_permisos_tipus NOT NULL,
	administrar bit  CONSTRAINT NN_permisos_administrador NOT NULL,
	validar bit  CONSTRAINT NN_permisos_validador NOT NULL,
	veure_usuaris bit,
	veure_despeses bit
);
GO

--************CARRECS************
CREATE TABLE Carrecs(
	id int identity(1,1) CONSTRAINT PK_carrec_id PRIMARY KEY,
	nom nvarchar(25) CONSTRAINT NN_carrec_nom NOT NULL
			 CONSTRAINT UN_carrec_nom UNIQUE,
	permisos_id int CONSTRAINT RF_carrec_permisos_id REFERENCES Permisos(id),
	habilitat bit CONSTRAINT NN_carrec_habilitat NOT NULL,
actualitzacio datetime CONSTRAINT NN_carrec_actualitzacio NOT NULL
);
GO

--************USUARIS************
CREATE TABLE Usuaris(
	id int identity(1,1) CONSTRAINT UN_usuari_id UNIQUE
     CONSTRAINT NN_usuari_id NOT NULL,
	dni nvarchar(20) CONSTRAINT PK_usuari_dni PRIMARY KEY,
	nom nvarchar(25) CONSTRAINT NN_usuari_nom NOT NULL,
	cognoms nvarchar(100) CONSTRAINT NN_usuari_cognoms NOT NULL,
	carrec_id int CONSTRAINT RF_usuari_carrec REFERENCES Carrecs(id)
                      CONSTRAINT NN_usuari_carrec NOT NULL,
	responsable_id int CONSTRAINT RF_usuari_responsable_id REFERENCES Usuaris(id)
  CONSTRAINT NN_usuari_responsable NOT NULL,
contrasenya nvarchar(16) CONSTRAINT NN_usuari_contrasenya NOT NULL,
habilitat bit CONSTRAINT NN_usuari_habilitat NOT NULL,
actualitzacio datetime CONSTRAINT NN_usuari_actualitzacio NOT NULL
);
GO

--*******************FULL DESPESES********************
CREATE TABLE FullsDespeses(
	id int identity(1,1) CONSTRAINT PK_fulldespesa_id PRIMARY KEY,
	usuari_id int CONSTRAINT RF_fulls_usuari_id REFERENCES Usuaris(id)
     CONSTRAINT NN_fulls_usuari_id NOT NULL,
	obert datetime CONSTRAINT NN_fulls_obert_data NOT NULL,
	total decimal(9,2) CONSTRAINT NN_fulls_total NOT NULL,
estat bit,
	solicitat datetime,
	validat datetime,
	pagat datetime,
	comentari nvarchar(250),
actualitzacio datetime CONSTRAINT NN_full_actualitzacio NOT NULL
);
GO

--*************TIPUS DESPESA*************
CREATE TABLE Tipus(
	id int identity(1,1) CONSTRAINT PK_tipus_id PRIMARY KEY,
	nom nvarchar(25) CONSTRAINT NN_tipus_nom NOT NULL
CONSTRAINT UN_tipus_nom UNIQUE,
	habilitat bit CONSTRAINT NN_tipus_habilitat NOT NULL,
actualitzacio datetime CONSTRAINT NN_tipus_actualitzacio NOT NULL
);
GO

--*************DESPESES*************
CREATE TABLE Despeses(
	id int identity(1,1) CONSTRAINT PK_despesa_id PRIMARY KEY,
	full_despesa int CONSTRAINT RF_despesa_full_id REFERENCES FullsDespeses(id)
CONSTRAINT NN_despesa_full_id NOT NULL,
	data datetime CONSTRAINT NN_despesa_data NOT NULL,
	import decimal(9,2) CONSTRAINT NN_despesa_import NOT NULL 
   CONSTRAINT CH_despesa_import CHECK (import>0),
	tipus_id int CONSTRAINT RF_despesa_tipus_id REFERENCES Tipus(id)
CONSTRAINT NN_despesa_tipus_id NOT NULL,
	adjunt bit CONSTRAINT NN_despesa_adjunt_id NOT NULL,
acceptada bit,
	comentari nvarchar(250),
actualitzacio datetime CONSTRAINT NN_despesa_actualitzacio NOT NULL
);
GO

--*************FACTURES*************
CREATE TABLE Factures(
	id int identity(1,1) CONSTRAINT PK_factura_id PRIMARY KEY,
	despesa_id int CONSTRAINT RF_factura_despesa_id REFERENCES Despeses(id)
		       CONSTRAINT NN_factures_despesa_id NOT NULL
CONSTRAINT UN_factures_despesa_id UNIQUE,
	nom_arxiu nvarchar(300) CONSTRAINT NN_facture_nom_arxiu NOT NULL,
	adjunt varbinary(max) CONSTRAINT NN_factures_adjunt NOT NULL,
	actualitzacio datetime CONSTRAINT NN_factures_actualitzacio NOT NULL
);
GO

--***************CALENDARI***************
CREATE TABLE Festius(
	data datetime CONSTRAINT PK_festius_data PRIMARY KEY,
descripcio nvarchar(50) CONSTRAINT NN_festius_descripcio NOT NULL,
actualitzacio datetime CONSTRAINT NN_festius_actualitzacio NOT NULL
);
GO



--CREACIO TRIGGERS
--Actualitzar import total a fulls de despeses quan afegir despesa
CREATE TRIGGER trig_afegir_despesa
ON Despeses
AFTER INSERT
AS
BEGIN
	DECLARE @id int,@import decimal(9,2),@total decimal(9,2)
	SET @id = (SELECT full_despesa FROM inserted)
	SET @import = (SELECT import FROM inserted)
	SET @total = (SELECT total FROM FullsDespeses where @id=id)
	UPDATE FullsDespeses SET total=(@import+@total) where @id=id
END
GO

--Actualitzar import total a fulls de despeses quan esborrar despesa
CREATE TRIGGER trig_esborrar_despesa
ON Despeses
AFTER DELETE
AS
BEGIN
	DECLARE @import decimal(9,2),@total decimal(9,2),@id int
	SET @import = (SELECT import FROM deleted)
	SET @id = (SELECT full_despesa FROM deleted)
	SET @total = (SELECT total FROM fullsdespeses where @id=id)
	UPDATE fullsdespeses SET total=(@total-@import) where @id=id
END
GO

--Actualitzar import total a fulls de despeses quan es modifica una despesa
CREATE TRIGGER trig_modificar_despesa
ON Despeses
AFTER UPDATE
AS
BEGIN
	DECLARE @fullId int,@import decimal(9,2),@total decimal(9,2)
	SET @import = (SELECT import FROM deleted)
	SET @total = (SELECT import FROM inserted)
	SET @import = @total - @import
	SET @fullId = (SELECT full_despesa FROM deleted)
	SET @total = (SELECT total FROM FullsDespeses where id=@fullId)
	UPDATE fullsdespeses SET total=(@total+@import) where @fullId=id
END
GO

--Actualitzar import total a fulls de despeses quan es nega una despesa o quan es tornar a acceptar
CREATE TRIGGER denegarOvalidarDespesa
ON Despeses 
AFTER UPDATE
AS
BEGIN
	DECLARE @import decimal(9,2), @fullId int, @total decimal(9,2)
	SET @import =(SELECT import FROM Despeses where (SELECT id from inserted)=id)
	SET @fullId =(SELECT full_despesa FROM Despeses where (SELECT id from inserted)=id)
	SET @total =(SELECT total FROM FullsDespeses where id=@fullId)
	--Estava denegada i passa a acceptada
	if (SELECT Acceptada from Deleted)=0 and (SELECT Acceptada from Inserted)=1
	BEGIN
		UPDATE FullsDespeses SET total=(@total+@import) where id=@fullId
	END
	--Estava acceptada i passa a denegada
	else if(SELECT Acceptada from Deleted)=1 and (SELECT Acceptada from Inserted)=0
	BEGIN
		UPDATE FullsDespeses SET total=(@total-@import) where id=@fullId
	END
	--Estava acceptada (null) i passa a denegada
	else if(SELECT Acceptada from Deleted) is NULL and (SELECT Acceptada from Inserted)=0
	BEGIN
		UPDATE FullsDespeses SET total=(@total-@import) where id=@fullId
	END
END
GO



--CREACIO STORED PROCEDURES
--USUARIS
--Verificacio usuari i contrasenya
CREATE PROCEDURE autentificarUsuari @usuari nvarchar(20),@contrasenya nvarchar(16)
AS
SELECT COUNT(*) as 'Valid' 
FROM Usuaris 
WHERE dni=@usuari AND contrasenya=@contrasenya AND habilitat=1
GO

--Verificar acces usuari
CREATE PROCEDURE verificarUsuariHabilitat @usuariId int
AS
	SELECT COUNT(*) 
FROM Usuaris 
WHERE id=@usuariId and habilitat=1
GO

--Habilitar acces usuari
CREATE PROCEDURE habilitarUsuari @usuariId int,@actualitzacio DateTime
AS
BEGIN
	UPDATE Usuaris 
SET habilitat=1,actualitzacio=@actualitzacio 
WHERE id=@usuariId
END
GO

--Deshabilitar acces usuari
CREATE PROCEDURE deshabilitarUsuari @usuariId int,@actualitzacio DateTime
AS
BEGIN
	UPDATE Usuaris 
SET habilitat=0,actualitzacio=@actualitzacio 
WHERE id=@usuariId
END
GO

--Afegir nou usuari
CREATE PROCEDURE afegirUsuariNou @dni nvarchar(20),@nom nvarchar(25),@cognoms nvarchar(100),@carrecId int,@responsableId int,@contrasenya nvarchar(16),@actualitzacio DateTime
AS
BEGIN
	INSERT INTO Usuaris (dni, nom, cognoms, carrec_id, responsable_id, contrasenya, habilitat, actualitzacio) 
	VALUES (@dni, @nom, @cognoms, @carrecId, @responsableId, @contrasenya, 1, @actualitzacio)
	SELECT CONVERT(int,@@IDENTITY)
END
GO

--Modificar usuari
CREATE PROCEDURE modificarUsuari @id int, @dni nvarchar(20), @nom nvarchar(25), @cognoms nvarchar(200), @carrecId int,@responsableId int, @actualitzacio DateTime
AS
BEGIN
	UPDATE Usuaris 
SET dni=@dni, nom=@nom, cognoms=@cognoms, carrec_id=@carrecId, responsable_id=@responsableId, actualitzacio=@actualitzacio 
WHERE id=@id
END
GO

--Modificar contrasenya dusuari
CREATE PROCEDURE modificarContrasenya @id int,@contrasenya nvarchar(16),@actualitzacio DateTime
AS
BEGIN
	UPDATE Usuaris 
SET contrasenya=@contrasenya,actualitzacio=@actualitzacio 
WHERE id=@id
END
GO

--Llista dusuaris
CREATE PROCEDURE llistarUsuaris
AS
	SELECT Us.id, Us.dni, Us.nom, Us.cognoms,
(SELECT dni FROM Usuaris WHERE id=Us.responsable_id) as dniResponsable, Us.habilitat,Ca.nom as nomCarrec,Pe.nom as nomPermisos,
case 
	when Ca.permisos_id is null then 0 else Ca.permisos_id end as nomPermisos2
	FROM Usuaris as Us INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id
	LEFT JOIN Permisos as Pe ON Ca.permisos_id=Pe.id
GO

--Llista dusuaris
CREATE PROCEDURE llistarUsuarisDisponibles
AS
	SELECT Us.id, Us.dni, Us.nom, Us.cognoms,
(SELECT dni FROM Usuaris where id=Us.responsable_id) as dniResponsable,Us.habilitat,Ca.nom as nomCarrec,Pe.nom as nomPermisos,
case 
	when Ca.permisos_id is null then 0 else Ca.permisos_id end as nomPermisos2
	FROM Usuaris as Us INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id
	LEFT JOIN Permisos as Pe ON Ca.permisos_id=Pe.id
GO

--Buscar usuaris de responsable
CREATE PROCEDURE llistaUsuarisDeResponsable @responsableId int
AS
	SELECT Us.dni, Us.nom, Us.cognoms,Ca.nom as nomCarrec
	FROM Usuaris as Us INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id
	LEFT JOIN Permisos as Pe ON Ca.permisos_id=Pe.id
	WHERE Us.responsable_id=@responsableId
GO

--Verificar si existeix un usuari per DNI
CREATE PROCEDURE existeixUsuari @dni nvarchar(20)
AS
SELECT COUNT(*) 
FROM Usuaris 
WHERE dni=@dni
GO

--Buscar usuari per DNI
CREATE PROCEDURE buscarUsuariPerDNI @dni nvarchar(20)
AS
SELECT * 
FROM Usuaris 
WHERE @dni=dni
GO

--Buscar usuari que DNI conte
CREATE PROCEDURE buscarPartDNIUsuari @paraula nvarchar(20)
AS
SELECT Us.id,Us.dni,Us.nom,Us.cognoms,Us.habilitat,
(SELECT dni FROM Usuaris WHERE id=Us.responsable_id) as dniResponsable, Ca.nom as nomCarrec 
FROM Usuaris as Us INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id

WHERE dni LIKE '%'+@paraula+'%'
GO

--Buscar usuari per ID
CREATE PROCEDURE buscarUsuariPerID @id nvarchar(20)
AS
SELECT * 
FROM Usuaris 
WHERE @id=id
GO

--Buscar noms d usuari que continguin una paraula
CREATE PROCEDURE nomQueConte @paraula nvarchar(25)
AS
SELECT Us.id,Us.dni,Us.nom,Us.cognoms,Us.habilitat,
(SELECT dni FROM Usuaris where id=Us.responsable_id) as dniResponsable,
Ca.nom as nomCarrec 
FROM Usuaris as Us INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id
WHERE Us.nom LIKE '%'+@paraula+'%'
GO

--Buscar cognoms d usuari que continguin una paraula
CREATE PROCEDURE cognomsQueConten @paraula nvarchar(100)
AS
SELECT Us.id,Us.dni,Us.nom,Us.cognoms,Us.habilitat,
(SELECT dni FROM Usuaris where id=Us.responsable_id) as dniResponsable,
Ca.nom as nomCarrec 
FROM Usuaris as Us INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id
WHERE Us.cognoms LIKE '%'+@paraula+'%'
GO

--Buscar noms i cognoms d usuari que continguin paraules
CREATE PROCEDURE nomsIcognomsQueContenen @paraulaNom nvarchar(25),@paraulaCognoms nvarchar(100)
AS
SELECT Us.id,Us.dni,Us.nom,Us.cognoms,Us.habilitat,
(SELECT dni FROM Usuaris where id=Us.responsable_id) as dniResponsable,
Ca.nom as nomCarrec 
FROM Usuaris as Us INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id
WHERE Us.nom LIKE '%'+@paraulaNom +'%' AND Us.cognoms LIKE '%'+@paraulaCognoms +'%'
GO

--PERMISOS
--Buscar un permis pel seu nom
CREATE PROCEDURE buscarPermisPerNom @nom nvarchar(25)
AS
SELECT * 
FROM Permisos 
WHERE nom=@nom
GO

--Llista de permisos
CREATE PROCEDURE llistaDePermisos
AS
SELECT * FROM Permisos
GO

--Buscar permisos pel seu id
CREATE PROCEDURE buscarPermisosPerId @permisosId int
AS
SELECT * 
FROM Permisos 
WHERE id=@permisosId
GO

--CARRECS
--Llista de carrecs 
CREATE PROCEDURE llistaDeCarrecs
AS
SELECT Ca.id,Ca.nom,Ca.habilitat,Pe.nom as nomPermisos,
case 
	when Ca.permisos_id is null then 0 else Ca.permisos_id end as idPermis
FROM Carrecs as Ca LEFT JOIN Permisos as Pe ON Ca.permisos_id=Pe.id
GO

--Llista de carrecs habilitats
CREATE PROCEDURE llistaDeCarrecsHabilitats
AS
SELECT Ca.id,Ca.nom,Ca.habilitat,Pe.nom as nomPermisos,
case 
	when Ca.permisos_id is null then 0 else Ca.permisos_id end as idPermis
FROM Carrecs as Ca LEFT JOIN Permisos as Pe ON Ca.permisos_id=Pe.id
WHERE Ca.habilitat=1
GO

--Afegir un nou carrec
CREATE PROCEDURE afegirCarrecNou @nom nvarchar(25),@permisosId int,@habilitat bit,@actualitzacio DateTime
AS
BEGIN
	INSERT INTO Carrecs (nom,permisos_id,habilitat,actualitzacio) VALUES (@nom,@permisosId,@habilitat,@actualitzacio)
	SELECT convert(int,@@Identity)
END
GO

--Modificar carrec existent
CREATE PROCEDURE modificarCarrecExistent @id int,@nom nvarchar(25),@permisosId int,@actualitzacio DateTime
AS
BEGIN
	UPDATE Carrecs 
SET nom=@nom,permisos_id=@permisosId,actualitzacio=@actualitzacio 
WHERE id=@id
END
GO

--Verificar si un carrec esta habilitat
CREATE PROCEDURE verificarCarrecHabilitat @id int
AS
	SELECT COUNT(*) 
FROM Carrecs 
WHERE id=@id and habilitat=1
GO

--Habilitar un carrec
CREATE PROCEDURE habilitarCarrec @id int, @actualitzacio DateTime
AS
	UPDATE Carrecs 
SET habilitat=1, actualitzacio=@actualitzacio 
WHERE id=@id
GO

--Deshabilitar un carrec
CREATE PROCEDURE deshabilitarCarrec @id int, @actualitzacio DateTime
AS
	UPDATE Carrecs 
SET habilitat=0, actualitzacio=@actualitzacio 
WHERE id=@id
GO


--Buscar un carrec pel seu nom
CREATE PROCEDURE buscarCarrecPerNom @nom nvarchar(25)
AS
SELECT * 
FROM Carrecs 
WHERE nom=@nom
GO

--Buscar un carrec pel seu id
CREATE PROCEDURE buscarCarrecPerId @carrecId int
AS
SELECT * 
FROM Carrecs 
WHERE id=@carrecId
GO


--FULLS DESPESES
--Llista de fulls pendents de pagar
CREATE PROCEDURE llistarFullsPendentsPagament
AS
	SELECT Fu.id,Us.dni,Us.nom,Us.cognoms,Ca.nom as nomCarrec,Fu.total,Fu.solicitat 
FROM FullsDespeses as Fu INNER JOIN Usuaris as Us ON Fu.usuari_id=Us.id INNER JOIN Carrecs as Ca ON Us.carrec_id=Ca.id
	WHERE Fu.estat=1 and Fu.pagat is null
GO 


--Pagar despeses de full
CREATE PROCEDURE pagarFullDespeses @id int,@pagat DateTime, @actualitzacio DateTime
AS
BEGIN
	UPDATE FullsDespeses 
SET pagat=@pagat,actualitzacio=@actualitzacio 
WHERE id=@id
END
GO

--Nou full de despeses
CREATE PROCEDURE crearFullDespeses @usuariId int,@obert DateTime,@actualitzacio DateTime
AS 
BEGIN
    	INSERT INTO FullsDespeses (usuari_id, obert, total, solicitat, validat, pagat, comentari, actualitzacio)
VALUES (@usuariId,@obert,0,null,null,null,'',@actualitzacio)
	SELECT convert(int,@@Identity)
END
GO

--Buscar full obert dun usuari
CREATE PROCEDURE buscarFullObertDeUsuari @usuariId int
AS
    	SELECT * 
FROM FullsDespeses 
WHERE usuari_id=@usuariId and pagat is null and (estat=1 or estat is null)
GO

--Buscar fulls tancats dun usuari
CREATE PROCEDURE buscarFullsTancats @usuariId int
AS
	SELECT * 
FROM FullsDespeses 
WHERE @usuariId=usuari_id and (pagat is not null or estat=0)
GO

--Buscar fulls pendents dun responsable
CREATE PROCEDURE buscarFullsPententsDeResponsable @responsableId int
AS
	SELECT Us.dni,Us.nom,Us.cognoms,Ca.nom as nomCarrec, Fu.id, Fu.usuari_id, Fu.obert, Fu.total, Fu.estat, Fu.solicitat, Fu.validat, Fu.pagat, Fu.comentari
	FROM Usuaris as Us INNER JOIN FullsDespeses as Fu ON Us.id=Fu.usuari_id
	INNER JOIN Carrecs as Ca ON Ca.id=Us.carrec_id
	WHERE Us.responsable_id=@responsableId and Fu.validat is null and Fu.solicitat is not null
GO

--Sollicitar pagament de full obert
CREATE PROCEDURE solicitarPagamentFullObert @fullId int,@solicitat DateTime,@actualitzacio Datetime
AS
BEGIN
	UPDATE FullsDespeses 
SET solicitat=@solicitat,actualitzacio=@actualitzacio 
WHERE id=@fullId
END
GO

--Rebutjar pagament full de despeses
CREATE PROCEDURE rebutjarFullDespeses @idFull int,@validat DateTime,@comentari nvarchar(250),@actualitzacio DateTime
AS
BEGIN
UPDATE FullsDespeses 
SET estat=0,validat=@validat, actualitzacio=@actualitzacio, comentari=@comentari 
WHERE id=@idFull
END
GO

--Acceptar pagament full de despeses
CREATE PROCEDURE acceptarFullDespeses @idFull int,@validat DateTime,@actualitzacio DateTime
AS
BEGIN
UPDATE FullsDespeses 
SET estat=1,validat=@validat, actualitzacio=@actualitzacio 
WHERE id=@idFull
END
GO


--Buscar fulls tancats dun treballador
CREATE PROCEDURE fullsTancatsDeUsuari @usuariId int
AS
	SELECT * 
FROM FullsDespeses 
WHERE usuari_id=@usuariId and (pagat is not null or estat=0)
GO

--Buscar comentari de full de despeses
CREATE PROCEDURE buscarComentariDeFull @FullId int
AS
	SELECT comentari 
FROM FullsDespeses 
WHERE id=@FullId
GO


--DESPESES
--Mirar si una despesa te adjunt
CREATE PROCEDURE despesaConteAdjunt @id int
AS
	SELECT COUNT(*) 
FROM Despeses 
WHERE id=@id and adjunt=1
GO

--Buscar les despeses dun full 
CREATE PROCEDURE buscarDespesesDeFull @fullId int
AS
	SELECT De.id,De.data,De.import,De.acceptada,De.adjunt,De.comentari,Ti.nom as nomTipus 
FROM Despeses as De INNER JOIN Tipus as Ti ON De.tipus_id=Ti.id
	WHERE full_despesa=@fullId
GO

--Eliminar despesa
CREATE PROCEDURE eliminarDespesa @id int
AS
	DELETE FROM Despeses 
WHERE @id=id
GO
--Denegar despesa
CREATE PROCEDURE denegarDespesa @idDespesa int,@actualitzacio DateTime
AS
BEGIN
UPDATE Despeses 
SET acceptada=0, actualitzacio=@actualitzacio 
WHERE id=@idDespesa
END 
GO

--Acceptar despesa
CREATE PROCEDURE acceptarDespesa @idDespesa int,@actualitzacio DateTime
AS
BEGIN
UPDATE Despeses 
SET acceptada=1, actualitzacio=@actualitzacio 
WHERE id=@idDespesa
END 
GO

--Crear despesa nova
CREATE PROCEDURE crearDespesaNova @fullId int,@data DateTime,@import decimal(9,2),@tipusId int,@adjunt bit,@comentari nvarchar(250),@actualitzacio DateTime
AS
BEGIN
	INSERT INTO Despeses (full_despesa, data,import, tipus_id, adjunt, acceptada, comentari, actualitzacio)
	VALUES (@fullId,@data,@import,@tipusId,@adjunt,null,@comentari,@actualitzacio)
SELECT convert(int,@@Identity)
END
GO

--Buscar despesa per ID
CREATE PROCEDURE buscarDespesa @idDespesa int
AS
	SELECT De.data,De.import,Ti.id as idTipus,Ti.nom as nomTipus,De.adjunt,De.comentari 
FROM Despeses as De INNER JOIN Tipus as Ti ON De.tipus_id=Ti.id
	WHERE De.id=@idDespesa
GO

--Modificar despesa
CREATE PROCEDURE modificarDespesa @id int,@data DateTime,@import decimal(9,2),@tipusId int,@adjunt bit,@comentari nvarchar(250),@actualitzacio Datetime
AS
BEGIN
UPDATE Despeses 
SET data=@data, import=@import, tipus_id=@tipusId, adjunt=@adjunt, comentari=@comentari, actualitzacio=@actualitzacio 
WHERE id=@id
END
GO

--Buscar comentari de despesa
CREATE PROCEDURE buscarComentariDeDespesa @despesaId int
AS
	SELECT comentari 
FROM Despeses 
WHERE id=@despesaId
GO


--FESTIUS
--Llista de festius
CREATE PROCEDURE llistaDeFestius
AS
SELECT * 
FROM Festius
GO

--Afegir dia festiu
CREATE PROCEDURE afegirFestiuNou @data Date, @descripcio nvarchar(50), @actualitzacio DateTime
AS
BEGIN
INSERT INTO Festius (data,descripcio,actualitzacio)
	VALUES (@data,@descripcio,@actualitzacio)
END
GO

--Eliminar dia festiu
CREATE PROCEDURE eliminarFestiu @data Date
AS
BEGIN
DELETE FROM Festius 
WHERE data=@data
END
GO

--Vefificar si una data s festiva
CREATE PROCEDURE dataFestiva @data datetime
AS
	SELECT COUNT(*) 
FROM Festius 
WHERE data=@data 
GO

--Llista de datas festives
CREATE PROCEDURE datesFestives
AS
SELECT data 
FROM Festius 
GO


--FACTURES
--Afegir nou adjunt
CREATE PROCEDURE afegirNouAdjunt @despesaId int,@nomArxiu nvarchar(300),@adjunt varbinary(max),@actualitzacio DateTime
AS
INSERT INTO Factures (despesa_id,nom_arxiu,adjunt,actualitzacio)
	VALUES (@despesaId,@nomArxiu,@adjunt,@actualitzacio)
GO

--Eliminar adjunt
CREATE PROCEDURE eliminarAdjunt @despesaId int
AS
BEGIN
DELETE Factures where despesa_id=@despesaId
END
GO

--Modificar adjunt
CREATE PROCEDURE modificarAdjunt @idFactura int,@nomArxiu nvarchar(300),@adjunt varbinary(max),@actualitzacio DateTime
AS
BEGIN
	UPDATE Factures 
SET nom_arxiu=@nomArxiu,adjunt=@adjunt,actualitzacio=@actualitzacio 
WHERE id=@idFactura
END
GO

--Id de factura amb id de despesa
CREATE PROCEDURE buscarIdDeFacturaPerIdDespesa @despesaId int
AS
SELECT id 
FROM Factures 
WHERE despesa_id=@despesaId
GO

--Nom de factura amb id de despesa
CREATE PROCEDURE buscarNomAdjunt @despesaId int
AS
SELECT nom_arxiu 
FROM Factures 
WHERE despesa_id=@despesaId
GO

--Recuperar adjunt
CREATE PROCEDURE recuperarAdjunt @despesaId int
AS
SELECT adjunt 
FROM Factures 
WHERE despesa_id=@despesaId
GO


--TIPUS
--Buscar tipus pel seu id
CREATE PROCEDURE buscarTipusPerId @tipusId int
AS
	SELECT * 
FROM Tipus 
WHERE id=@tipusId
GO
--Afegir tipus 
CREATE PROCEDURE afegirTipus @nom nvarchar(25),@actualitzacio DateTime
AS
BEGIN
	INSERT INTO Tipus (nom,habilitat,actualitzacio) 
VALUES (@nom,1,@actualitzacio)
	SELECT convert(int,@@Identity)
END
GO
--Modificar tipus 
CREATE PROCEDURE modificarTipus @id int,@nom nvarchar(25),@actualitzacio DateTime
AS
BEGIN
	UPDATE Tipus 	
SET nom=@nom,actualitzacio=@actualitzacio 
WHERE @id=id
END
GO	

--Si el tipus esta habilitat
CREATE PROCEDURE tipusHabilitat @id int
AS
	SELECT Count(*) 
FROM Tipus 
WHERE id=@id and habilitat=1
GO

--Habilitar tipus
CREATE PROCEDURE habilitarTipus @id int,@actualitzacio DateTime
AS
BEGIN
	UPDATE Tipus SET habilitat=1,actualitzacio=@actualitzacio where id=@id
END
GO

--Deshabilitar tipus
CREATE PROCEDURE deshabilitarTipus @id int,@actualitzacio DateTime
AS
BEGIN
	UPDATE Tipus 
SET habilitat=0,actualitzacio=@actualitzacio 
WHERE id=@id
END
GO

--Buscar tipus per nom 
CREATE PROCEDURE buscarTipusPerNom @nom nvarchar(25)
AS
	SELECT * 
FROM Tipus 
WHERE @nom=nom
GO

--Llista de tipus despeses
CREATE PROCEDURE llistaDeTipusDeDespeses
AS
SELECT * FROM Tipus
GO

--Llista de tipus despeses habilitades
CREATE PROCEDURE llistaDeTipusHabilitats
AS
SELECT * 
FROM Tipus 
WHERE habilitat=1
GO




