﻿using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Accions
{
    class Carrec
    {
        private Int32 id;
        private int? permisos_id;
        private String nom;
        private Permisos permisos;

        #region Constructor
        /*
         * Metode constructor
         */
        public Carrec(Int32 id, String nom, int? permisos_id)
        {
            this.id = id;
            this.nom = nom;
            this.permisos_id = permisos_id;
        }
        #endregion

        #region Metodes accessoris
        public Int32 CarrecId
        {
            get { return id; }
        }

        public String CarrecNom
        {
            get { return nom; }
        }

        public int? PermisosId
        {
            get { return permisos_id; }
        }

        public Permisos Permisos
        {
            get { return permisos; }
            set { permisos = value; }
        }
        #endregion

        #region Buscar carrecs
        #region Buscar per id
        /*
         * Metode per buscar un carrec donat el seu id
         */
        public static Carrec buscarCarrecPerId(Int32 id)
        {

            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Carrec carrecServei = servei.buscarCarrecPerId(id);
                    if (carrecServei != null)
                    {
                        Carrec carrec = new Carrec(carrecServei.Id, carrecServei.Nom, carrecServei.IdPermisos as Int32?);
                        return carrec;
                    }
                    return null;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Buscar per nom
        /*
         * Metode per buscar un carrec,donat el seu nom
         */
        public static ServeiWeb.Carrec carrecPernom(string nom)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.buscarCarrecPerNom(nom);
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion
        #endregion

        #region Afegir carrec nou
        /*
         * Metode que permet afegir un nou carrec,donat el seu nom i el id dels permisos assignats
         */
        public static ServeiWeb.Carrec afegirCarrec(string nom, int? permisos)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.afegirCarrec(nom, permisos);
                }
                catch
                {
                    MessageBox.Show(Missatges.CARREC_AFEGIR_ERROR + Environment.NewLine + Missatges.CARREC_DUPLICAT);
                    return null;
                }
            }
        }
        #endregion

        #region Consultar estat de carrec
        /*
         * Metode que retorna si un determinat carrec esta habilitat o no,donat el seu id
         */
        public static bool verificarCarrec(Int32 carrecId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.verificarEstatCarrec(carrecId);
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return false;
                }
            }
        }
        #endregion

        #region Habilitar un carrec
        /*
         * Metode per habilitar un carrec,donat el seu id
         */
        public static bool habilitarCarrec(Int32 carrecId)
        {
            try
            {
                using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
                {
                    servei.habilitarCarrec(carrecId);
                    return true;
                }
            }
            catch
            {
                MessageBox.Show(Missatges.CARREC_HABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                return false;
            }
        }
        #endregion

        #region Deshabilitar un carrec
        /*
         * Metode per deshabilitar un carrec,donat el seu id
         */
        public static bool deshabilitarCarrec(Int32 carrecId)
        {
            try
            {
                using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
                {
                    servei.deshabilitarCarrec(carrecId);
                    return true;
                }
            }
            catch
            {
                MessageBox.Show(Missatges.CARREC_DESHABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                return false;
            }
        }
        #endregion

        #region Modificar un carrec
        /*
         * Metode per modificar un carrec,donat el seu id,el seu nom i el id dels permisos assignats
         */
        public static bool modificarCarrec(Int32 id, string nom, int? permisos)
        {

            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.modificarCarrecExistent(id, nom, permisos);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.CARREC_MODIFICAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Llista de carrecs habilitats
        /*
         * Metode que retorna un llistat dels carrecs habilitats
         */
        public static List<ServeiWeb.Carrec> llistaCarrecsDisponibles()
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.llistarCarrecsActius();
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Llista de carrecs existents
        /*
         * Metode que retorna un llistat amb els carrecs existents
         */
        public static List<ServeiWeb.Carrec> llistaCarrecs()
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.llistarCarrecs();
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion
    }
}
