﻿using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Accions
{
    class Despesa
    {
        #region Llista de despeses d'un full
        /*
         * Metode per retornar el llistat de despeses d'un full, donat el id del full
         */
        public static List<ServeiWeb.Despesa> despesesDeFull(Int32 fullId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.Despesa> llistaServei = servei.llistaDespesesDefull(fullId);
                    return llistaServei;
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Eliminar despeses
        /*
         * Metode per eliminar despeses, donada una llista del seus ids i una llista per indicar si dispossen d'adjunt
         */
        public static bool eliminarDespeses(List<Int32> llista, List<bool> despesesAdjunt)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.esborrarDespeses(llista, despesesAdjunt);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.DESPESA_ELIMINAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Afegir despesa nova
        /*
         * Metode per afegir una nova despesa, donades les dades de la despesa
         */
        public static ServeiWeb.Despesa afegirDespesaNova(Int32 fullId, DateTime data, decimal import, Int32 tipusId, string comentari, string nomArxiu, byte[] adjunt)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Despesa despesaNova = servei.afegirNovaDespesa(fullId, data, import, tipusId, comentari, nomArxiu, adjunt);
                    return despesaNova;
                }
                catch
                {
                    MessageBox.Show(Missatges.DESPESA_AFEGIR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return null;
                }
            }
        }
        #endregion

        #region Denegar pagament de despeses d'un full
        /*
         * Metode per denegar el pagament de despeses, donada una llista dels ids de les despeses
         */
        public static bool denegarDespeses(List<Int32> despeses)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.denegarDespeses(despeses);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.DESPESA_DENEGAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Acceptar pagament de despesesd'un full
        /*
         * Metode per accepter el pagament de despeses, donada una llista dels ids de les despeses
         */
        public static bool acceptarDespeses(List<Int32> despeses)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.acceptarDespeses(despeses);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.DESPESA_ACCEPTAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Buscar una despesa
        /*
         * Metode per buscar una despesa,donat el seu id
         */
        public static ServeiWeb.Despesa buscarDespesa(Int32 despesaId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Despesa despesa = servei.buscarDespesa(despesaId);
                    return despesa;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Modificar una despesa
        /*
         * Metode per modificar una despesa, donat el seu id i les noves dades
         */
        public static bool modificarDespesa(Int32 despesaId, DateTime data, decimal import, Int32 tipusId, string comentari, Int32 facturaId, string nomFactura, byte[] adjunt)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.modificarDespesa(despesaId, data, import, tipusId, comentari, facturaId, nomFactura, adjunt);
                    return true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(Missatges.DESPESA_MODIFICAR_ERROR + Environment.NewLine + Missatges.REPETIR + ex.Message);
                    return false;
                }
            }
        }
        #endregion

        #region Consultar si despesa te adjunt
        /*
         * Metode que verifica si una despesa te adjunt,donat el id de la despesa
         */
        public static bool verificarAdjunt(Int32 despesaId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    bool teAdjunt = servei.verificarAdjunt(despesaId);
                    return teAdjunt;
                }
                catch
                {
                    MessageBox.Show(Missatges.DESPESA_MODIFICAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Buscar comentari de despesa
        /*
         * Metode per buscar el comentari d'una despesa, donat el seu id
         */
        public static string comentarisDespesa(Int32 idDespesa)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.buscarComentariDeDespesa(idDespesa);
                }
                catch
                {
                    MessageBox.Show(Missatges.DESPESA_MODIFICAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return string.Empty;
                }
            }
        }
        #endregion
    }
}
