﻿using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Accions
{
    class Festiu
    {
        #region Llista de festius
        /*
         * Metode que retorna el llistat de festius
         */
        public static List<ServeiWeb.Festiu> llistaDeFestius()
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.Festiu> llista = servei.llistarFestius();
                    return llista;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Afegir nou festiu
        /*
         * Metode per afegir un nou festiu, donada la seva data i la seva descripcio
         */
        public static ServeiWeb.Festiu afegirFestiu(DateTime data, string descripcio)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Festiu festiuNou = servei.afegirFestiuNou(data, descripcio);
                    return festiuNou;
                }
                catch
                {
                    MessageBox.Show(Missatges.FESTIU_AFEGIR_ERROR + Environment.NewLine + Missatges.FESTIU_DUPLICAT);
                    return null;
                }
            }
        }
        #endregion

        #region Eliminar festiu
        /*
         * Metode per eliminar un festiu, donada la seva data
         */
        public static bool eliminarFestiu(DateTime data)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.eliminarFestiu(data);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.FESTIU_ELIMINAR_ERROR + Environment.NewLine + Missatges.FESTIU_DUPLICAT);
                    return false;
                }
            }
        }
        #endregion

        #region Consultar si una data es festiva
        /*
         * Metode per verificar si una data es festiva
         */
        public static bool verificarFestiu(DateTime data)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.verificarDiaFestiu(data);
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return false;
                }
            }
        }
        #endregion
    }
}
