﻿using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Accions
{
    class FullDespeses
    {
        #region Buscar full obert
        /*
         * Metode que retorna el full obert d'un usuari,donat el seu id
         */
        public static ServeiWeb.FullDespeses buscarFullObert(Int32 usuariId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.FullDespeses fullObertServei = servei.buscarFullObert(usuariId);
                    if (fullObertServei != null)
                    {
                        return fullObertServei;
                    }
                    return null;
                }
                catch
                {
                    MessageBox.Show(Missatges.FULL_OBERT_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Obrir full nou
        /*
         * Metode per obrir un full nou,donat el id de l'usuari.
         * Retorna les dades del nou full
         */
        public static ServeiWeb.FullDespeses obrirFullNou(Int32 usuariId)
        {
            try
            {
                using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
                {
                    ServeiWeb.FullDespeses fullNou = servei.obrirFullNou(usuariId);
                    return fullNou;
                }
            }
            catch
            {
                MessageBox.Show(Missatges.FULL_OBRIR_NOU_ERROR + Environment.NewLine + Missatges.REPETIR);
                return null;
            }
        }
        #endregion

        #region Llista de fulls pendents de pagament
        /*
         * Metode que retorna el llistat de despeses pendents de pagament
         */
        public static List<ServeiWeb.FullDespeses> llistaFullsPendentsPagament()
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.FullDespeses> llistaFulls = servei.llistarFullsPendentsPagar();
                    return llistaFulls;
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Pagar fulls pendents
        /*
         * Metode per pagar un llistat de fulls de despeses, donat els seus ids.
         * Retorna si s'han pagat correctament
         */
        public static bool pagarFullsPendents(List<Int32> llistaIdFulls)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.pagarFullsDespesesPendents(llistaIdFulls);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.FULL_PAGAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Rebutjar pagament de full de despeses
        /*
         * Metode per rebutjar el pagament d'un full de despeses,donat el id del full i un comentari
         * Retorna si s'ha rebutjat correctament
         */
        public static bool rebutjarFullDespeses(Int32 idFUll, string comentari)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.rebutjarPagamentFull(idFUll, comentari);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.FULL_REBUTJAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Acceptar pagament de full de despeses
        /*
         * Metode per acceptar el pagament d'un full de despeses,donat el seu id.
         * Retorna si s'ha acceptat correctament
         */
        public static bool acceptarFullDespeses(Int32 idFull)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.acceptarPagamentFull(idFull);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.FULL_ACCEPTAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Sol·licitar pagament de full de despeses
        /*
         * Metode per sol·licitar el pagament d'un full de despeses,donat el seu id.
         * Retorna la data de la sol·licitud
         */
        public static DateTime? solicitarPagamentFull(Int32 fullId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.solicitarPagamentFullDespeses(fullId);
                }
                catch
                {
                    MessageBox.Show(Missatges.FULL_SOLICITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return null;
                }
            }
        }
        #endregion

        #region Llista fulls pendents de validar de responsable
        /*
         * Metode que retorna els llistats pendents de validar d'un responsable,donat el id del responsable
         */
        public static List<ServeiWeb.FullDespeses> buscarFullsPendents(Int32 responsableId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.FullDespeses> llista = servei.buscarFullsPendentsDeResponsable(responsableId);
                    return llista;
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }

        }
        #endregion

        #region Llista de fulls tancats d'usuari
        /*
         * Metode que retorna el llistat de fulls tancats d'un usuari,donat el id de l'usuari
         */
        public static List<ServeiWeb.FullDespeses> llistarFullsTancats(Int32 usuariId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.FullDespeses> llistaTancats = servei.llistaFullsTancats(usuariId);
                    return llistaTancats;
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Buscar comentari de full de despeses
        /*
         * Metode que retorna el comentari d'un full de despeses,donat el seu id
         */
        public static string comentariFull(Int32 fullId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.comentariDeFull(fullId);
                }
                catch
                {
                    MessageBox.Show(Missatges.FULL_COMENTARI_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return null;
                }
            }
        }
        #endregion
    }
}
