﻿using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Accions
{
    class Tipus
    {
        #region Buscar tipus per Id
        /*
         * Metode per buscar un tipus de despesa,tonat el seu id
         */
        public static ServeiWeb.Tipus buscarTipusPerId(Int32 tipusId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Tipus tipus = servei.buscarTipusPerId(tipusId);
                    return tipus;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Buscar tipus per nom
        /*
         * Metode per buscar un tipus de despesa,donat el seu nom
         */
        public static ServeiWeb.Tipus buscarTipusPerNom(string nom)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Tipus tipus = servei.buscarTipusPerNom(nom);
                    return tipus;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Afegir tipus nou
        /*
         * Metode per afegir un tipus de despesa nou,donat el seu nom
         */
        public static ServeiWeb.Tipus afegirTipus(string nom)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.afegirTipusNou(nom);
                }
                catch
                {
                    MessageBox.Show(Missatges.TIPUS_AFEGIR_ERROR + Environment.NewLine + Missatges.TIPUS_DUPLICAT);
                    return null;
                }
            }
        }
        #endregion

        #region Modificar tipus existent
        /*
         * Metode per modificar un tipus de despesa existent,donat el seu id i nom
         */
        public static bool modificarTipus(Int32 id, string nom)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.modificarTipus(id, nom);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.TIPUS_MODIFICAR_ERROR + Environment.NewLine + Missatges.TIPUS_DUPLICAT);
                    return false;
                }
            }
        }
        #endregion

        #region Verificar si un tipus esta habilitat o no
        /*
         * Metode per verificar si un tipus de despesa esta habilitat,donat el seu id
         */
        public static bool verificarTipus(Int32 id)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.tipusHabilitat(id);
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return false;
                }
            }
        }
        #endregion

        #region Habilitar tipus
        /*
         * Metode per habilitar un tipus de despesa,donat el seu id
         */
        public static bool habilitarTipus(Int32 id)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.habilitarTipus(id);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.TIPUS_HABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Deshabilitar tipus
        /*
         * Metode per deshabilitar un tipus de despesa,donat el seu id
         */
        public static bool deshabilitarTipus(Int32 id)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.deshabilitarTipus(id);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.TIPUS_DESHABILITAR_ERROR + Environment.NewLine + Missatges.REPETIR);
                    return false;
                }
            }
        }
        #endregion

        #region Llista de tipus disponibles
        /*
         * Metode que retorna el llistat de tipus de despeses habilitades
         */
        public static List<ServeiWeb.Tipus> llistaTipusActius()
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.Tipus> llista = servei.llistarTipusActius();
                    return llista;
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Llista de tipus existents
        /*
         * Metode que retorna el llistat de tipus de despeses existents
         */
        public static List<ServeiWeb.Tipus> llistaTipus()
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.Tipus> llista = servei.llistarTipus();
                    return llista;
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion
    }
}
