﻿using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Accions
{
    class Usuari
    {
        private Int32 id, carrec_id, responsable_id;
        private string dni, nom, cognoms;
        private Carrec carrec;
        private DateTime actualitzacio;

        #region Constructor
        /*
         * Metode constructor
         */
        public Usuari(Int32 id, string dni, string nom, string cognoms, Int32 carrec_id, Int32 responsable_id, DateTime actualitzacio)
        {
            this.id = id;
            this.dni = dni;
            this.nom = nom;
            this.carrec_id = carrec_id;
            this.cognoms = cognoms;
            this.responsable_id = responsable_id;
            this.actualitzacio = actualitzacio;
        }
        #endregion

        #region Metodes accessoris
        public Int32 IdUsuari
        {
            get { return id; }
        }

        public string DniUsuari
        {
            get { return dni; }
        }

        public string NomUsuari
        {
            get { return nom; }
        }

        public string CognomsUsuari
        {
            get { return cognoms; }
        }

        public Int32 CarrecId
        {
            get { return carrec_id; }
        }

        public Int32 ResponsableId
        {
            get { return responsable_id; }
        }

        public Carrec Carrec
        {
            get { return carrec; }
            set { this.carrec = value; }
        }
        #endregion

        #region Iniciar login de l'usuari
        /*
         * Metode que inicia la pantalla de login i retorna el DNI de l'usuari si es valida correctament
         */
        public static string validarUsuari()
        {
            using (LoginMenu logarse = new LoginMenu())
            {
                try
                {
                    if (logarse.ShowDialog() == DialogResult.Cancel)
                    {
                        return string.Empty;
                    }
                    return logarse.usuari;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return string.Empty;
                }
            }
        }
        #endregion

        #region Llistat d'usuaris d'un responsable
        /*
         * Metode que retorna el llistat de treballadors d'un responsable
         */
        public static List<ServeiWeb.Usuari> usuarisDeResponsable(Int32 responsableId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.Usuari> llistaUsuaris = servei.usuarisDeResponsable(responsableId);
                    return llistaUsuaris;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Buscar usuari per DNI
        /*
         * Metode per buscar un usuari pel seu id
         */
        public static Usuari buscarUsuariPerDni(string dni)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Usuari usuariServei = servei.buscarUsuariPerDni(dni);
                    if (usuariServei == null)
                    {
                        return null;
                    }
                    else
                    {
                        Usuari usuari = new Usuari(usuariServei.Id, usuariServei.Dni, usuariServei.Nom, usuariServei.Cognoms, usuariServei.IdCarrec, usuariServei.IdResponsable, usuariServei.Actualitzacio);
                        return usuari;
                    }
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Verificar usuari i contrasenya
        /*
         * Metode que retorna si l'usuari i contrasenya son correctes
         */
        public static bool esValid(string usuari, string contrasenya)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    if (servei.validarUsuari(usuari, contrasenya))
                    {
                        return true;
                    }
                    return false;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return false;
                }
            }

        }
        #endregion

        #region Llistat d'usuaris que el DNI conte una paraula
        /*
         * Metode per buscar els usuaris que el seu DNI contingui certs caracters
         */
        public static List<ServeiWeb.Usuari> buscarPartDniUsuari(string paraula)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.Usuari> usuari = servei.dniAmb(paraula);
                    return usuari;
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_TORNAR);
                    return null;
                }
            }
        }
        #endregion

        #region Habilitar usuari
        /*
         * Metode per habilitar un usuari,donat el seu id.
         * Retorna si s'ha realitzat correctament
         */
        public static bool habilitarUsuari(Int32 usuariId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.activarUsuari(usuariId);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.USUARI_HABILITAR_ERROR);
                    return false;
                }
            }
        }
        #endregion

        #region Deshabilitar usuari
        /*
         * Metode per deshabilitar un usuari,donat el seu id.
         * Retorna si s'ha realitzat correctament
         */
        public static bool deshabilitarUsuari(Int32 usuariId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.desactivarUsuari(usuariId);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.USUARI_DESHABILITAR_ERROR);
                    return false;
                }
            }
        }
        #endregion

        #region Verificar disponibilitat d'usuari
        /*
         * Metode que retorna si un usuari esta habilitat o no
         */
        public static bool usuariHabilitat(Int32 usuariId)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.usuariHabilitat(usuariId);
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return false;
                }
            }
        }
        #endregion

        #region Afegir nou usuari
        /*
         * Metode per afegir un nou usuari, donades les seves dades.
         * Retorna el nou usuari
         */
        public static ServeiWeb.Usuari afegirUsuari(string dni, string nom, string cognoms, Int32 carrecId, Int32 responsableId, string contrasenya)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Usuari usuari = servei.afegirNouUsuari(dni, nom, cognoms, carrecId, responsableId, contrasenya);
                    return usuari;
                }
                catch
                {
                    MessageBox.Show(Missatges.USUARI_AFEGIR_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Modificar usuari existent
        /*
         * Metode per modificar un usuari,donat el seu id i les noves dades.
         * Retorna si s'ha modificat correctament
         */
        public static bool modificarUsuari(Int32 id, string dni, string nom, string cognoms, Int32 carrecId, Int32 responsableId, string contrasenya)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    servei.modificarUsuariExistent(id, dni, nom, cognoms, carrecId, responsableId, contrasenya);
                    return true;
                }
                catch
                {
                    MessageBox.Show(Missatges.USUARI_MODIFICAR_ERROR);
                }
                return false;
            }
        }
        #endregion

        #region Verificar si algun usuari te un determinat DNI
        /*
         * Metode per mirar si existeix un usuari,donat un DNI qualsevol.
         * Retorna si existeix algun usuari amb el DNI
         */
        public static bool existeixUsuariDNI(string dni)
        {

            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.existeixUsuariPerDNI(dni);
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return false;
                }
            }

        }
        #endregion

        #region Buscar usuari per id
        /*
         * Buscar un usuari,donat el seu id
         */
        public static Usuari buscarUsuariPerId(Int32 id)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    ServeiWeb.Usuari usuariServei = servei.buscarUsuariPerId(id);
                    if (usuariServei == null)
                    {
                        return null;
                    }
                    else
                    {
                        Usuari usuari = new Usuari(usuariServei.Id, usuariServei.Dni, usuariServei.Nom, usuariServei.Cognoms, usuariServei.IdCarrec, usuariServei.IdResponsable, usuariServei.Actualitzacio);
                        return usuari;
                    }
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Llista d'usuaris existents
        /*
         * Metode que retorna el llistat d'usuaris existents
         */
        public static List<ServeiWeb.Usuari> llistaUsuaris()
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    List<ServeiWeb.Usuari> llista = servei.llistarUsuaris();
                    return llista;
                }
                catch
                {
                    MessageBox.Show(Missatges.LLISTAT_ERROR);
                    return null;
                }
            }
        }
        #endregion

        #region Llistat d'usuaris que el nom conte una paraula
        /*
         * Metode per buscar els usuaris que els noms contenen certs caracters
         */
        public static List<ServeiWeb.Usuari> nomConte(string paraula)
        {
            try
            {
                using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
                {
                    return servei.nomsUsuariAmb(paraula);
                }
            }
            catch
            {
                MessageBox.Show(Missatges.CONSULTA_TORNAR);
                return null;
            }
        }
        #endregion

        #region Llistat d'usuaris que els cognoms contenen una paraula
        /*
         * Metode per buscar els usuaris que els cognoms contenen certs caracters
         */
        public static List<ServeiWeb.Usuari> cognomsContenen(string paraula)
        {
            try
            {
                using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
                {
                    return servei.cognomsUsuariAmb(paraula);
                }
            }
            catch
            {
                MessageBox.Show(Missatges.CONSULTA_TORNAR);
                return null;
            }
        }
        #endregion

        #region Llistat d'usuaris que el nom o cognoms contenen una paraula
        /*
         * Metode per buscar els usuaris que els seus noms i cognoms contenen certs caracters
         */
        public static List<ServeiWeb.Usuari> nomIcognomsContenen(string nom, string cognoms)
        {
            using (ServeiWeb.ServeiClient servei = ConnexioServei.nouServei())
            {
                try
                {
                    return servei.nomIcognomsAmb(nom, cognoms);
                }
                catch
                {
                    MessageBox.Show(Missatges.CONSULTA_TORNAR);
                    return null;
                }
            }
        }
        #endregion
    }
}
