﻿using ClientSGDEscriptori.Accions;
using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Administrar
{
    public partial class Administracio : Form
    {
        private bool administrador;

        public Administracio(bool administrador)
        {
            InitializeComponent();
            this.administrador = administrador;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void administracioCarregat(object sender, EventArgs e)
        {
            //Si te permisos d'administrador, pot fer modificacions
            if (administrador)
            {
                botoAfegirUsuari.Enabled = true;
                botoModificarUsuari.Enabled = true;
                botoHabilitarUsuari.Enabled = true;
                botoDeshabilitarUsuari.Enabled = true;
                botoDeshabilitarCarrec.Enabled = true;
                botoModificarCarrec.Enabled = true;
                botoHabilitarCarrec.Enabled = true;
                botoAfegirCarrec.Enabled = true;
                botoAfegirTipus.Enabled = true; ;
                botoModificarTipus.Enabled = true; ;
                botoDeshabilitarTipus.Enabled = true; ;
                botoHabilitarTipus.Enabled = true;
            }
            llistarUsuaris();
        }
        #endregion

        #region Llista usuaris
        /*
         * Metode per carregar els usaris existents a la BBDD
         */
        private void llistarUsuaris()
        {
            gridUsuaris.Rows.Clear();
            //Obtenim la llista d'usuaris
            List<ServeiWeb.Usuari> llista = Usuari.llistaUsuaris();
            //Si la llista esta plena
            if (llista != null && llista.Count > 0)
            {
                string[] linia;
                //Llegim els usuaris i els afegim al grid
                for (int cont = 0; cont < llista.Count; cont++)
                {

                    string habilitat = "No";
                    if ((bool)llista[cont].Habilitat)
                    {
                        habilitat = "Si";
                    }
                    linia = new string[] {llista[cont].Id.ToString(),null,llista[cont].Dni, llista[cont].Nom, llista[cont].Cognoms, llista[cont].NomCarrec, llista[cont].DniResponsable, habilitat };
                    gridUsuaris.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Mostrar llistat usuaris
        /*
         * Event per carregarla llista d'usuaris que oculta el panell per buscar
         */
        private void clickBotoLlista(object sender, EventArgs e)
        {
            gridUsuaris.Visible = true;
            llistarUsuaris();
        }
        #endregion

        #region Buscar usuari
        /*
         * Event per realitzar la cerca d'usuaris
         */
        private void clickIniciarCerca(object sender, EventArgs e)
        {
            //Es neteja el grid
            gridUsuaris.Rows.Clear();
            #region Buscar per DNI
            //Si esta marcat, buscar per contingut al DNI
            if (radiDNI.Checked)
            {
                //Si hi ha text introduit
                if (textDNI.Text.Length > 0)
                {
                    List<ServeiWeb.Usuari> llistaUsuaris = Usuari.buscarPartDniUsuari(textDNI.Text);
                    //Si hi ha usuaris usuaris, s' afegeixen al llistat
                    if (llistaUsuaris != null && llistaUsuaris.Count > 0)
                    {
                        for (int cont = 0; cont < llistaUsuaris.Count; cont++)
                        {
                            string esHabilitat = "No";
                            if (llistaUsuaris[cont].Habilitat)
                            {
                                esHabilitat = "Si";
                            }
                            string[] linia = new string[] { llistaUsuaris[cont].Id.ToString(),null, llistaUsuaris[cont].Dni, llistaUsuaris[cont].Nom, llistaUsuaris[cont].Cognoms, llistaUsuaris[cont].NomCarrec, llistaUsuaris[cont].DniResponsable, esHabilitat };
                            gridUsuaris.Rows.Add(linia);
                        }
                    }
                }
            }
            #endregion
            #region Buscar per nom i/o cognoms
            //Si esta marcat buscar per nom i/o cognoms
            else if (radiNom.Checked)
            {
                List<ServeiWeb.Usuari> llista = new List<ServeiWeb.Usuari>();
                //Si la cerca es per nom i cognoms
                if (textNom.Text.Length > 0 && textCognoms.Text.Length > 0)
                {
                    llista = Usuari.nomIcognomsContenen(textNom.Text, textCognoms.Text);
                }
                //Si la cerca es per nom
                else if (textNom.Text.Length > 0 && textCognoms.Text.Length == 0)
                {
                    llista = Usuari.nomConte(textNom.Text);
                }
                //Si la cerca es per cognoms
                else if (textNom.Text.Length == 0 && textCognoms.Text.Length > 0)
                {
                    llista = Usuari.cognomsContenen(textCognoms.Text);
                }
                //Si s'han trobat usuaris s' afegeixen al grid
                if (llista.Count > 0)
                {
                    for (int cont = 0; cont < llista.Count; cont++)
                    {
                        string esHabilitat = "No";
                        if (llista[cont].Habilitat)
                        {
                            esHabilitat = "Si";
                        }
                        string[] linia = new string[] { llista[cont].Id.ToString(), null, llista[cont].Dni, llista[cont].Nom, llista[cont].Cognoms, llista[cont].NomCarrec.ToString(), llista[cont].DniResponsable.ToString(), esHabilitat };
                        gridUsuaris.Rows.Add(linia);
                    }
                }

            }
            #endregion
            gridUsuaris.Visible = true;
        }
        #endregion

        #region Seleccio del tipus de cerca
        /*
         * Event per activar les zones de texts adients, depenent si es cerca per dni o per nom i/o cognoms
         */
        private void radiSeleccionat(object sender, EventArgs e)
        {
            if (radiDNI.Checked)
            {
                textCognoms.Visible = false;
                textNom.Visible = false;
                textDNI.Visible = true;
            }
            if (radiNom.Checked)
            {
                textDNI.Visible = false;
                textCognoms.Visible = true;
                textNom.Visible = true;
            }
        }
        #endregion

        #region Llista carrecs
        /*
         * Metode per carregar el grid dels carrecs existents
         */
        private void llistarCarrecs()
        {
            //Es busquen els carrecs existents
            List<ServeiWeb.Carrec> llistaCarrecs = Carrec.llistaCarrecs();
            //Es neteja el grid de carrecs
            gridCarrecs.Rows.Clear();
            //Si es troben carrecs, s'afegeixen al grid
            if (llistaCarrecs != null && llistaCarrecs.Count > 0)
            {
                for (int cont = 0; cont < llistaCarrecs.Count; cont++)
                {
                    string[] linia;
                    string habilitat = "No";
                    //Si l' usuari esta activat, s' indica
                    if ((bool)llistaCarrecs[cont].Habilitat)
                    {
                        habilitat = "Si";
                    }
                    string permis = "<<Sense permisos>>";
                    if (llistaCarrecs[cont].IdPermisos > 0)
                    {
                        permis = llistaCarrecs[cont].NomPermisos;
                    }

                    linia = new string[] { llistaCarrecs[cont].Id.ToString(), null, llistaCarrecs[cont].Nom, permis, habilitat };

                    gridCarrecs.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Llista permisos
        /*
         * Metode per carregarel grid de permisos existents 
         */
        private void llistarPermisos()
        {
            //Es busquen els permisos
            List<ServeiWeb.Permisos> llistaPermisos = Permisos.llistaPermisos();
            //Es neteja el grid de permisos
            gridPermisos.Rows.Clear();
            //Si es troben permisos s' afegeixen al grid
            if (llistaPermisos != null && llistaPermisos.Count > 0)
            {
                for (int cont = 0; cont < llistaPermisos.Count; cont++)
                {
                    string admin = "No", resp = "No", tec = "No", fin = "No";
                    if (llistaPermisos[cont].Administrar)
                    {
                        admin = "Si";
                    }
                    if (llistaPermisos[cont].Validar)
                    {
                        resp = "Si";
                    }
                    if (llistaPermisos[cont].VeureUsuaris != null && (bool)llistaPermisos[cont].VeureUsuaris)
                    {
                        tec = "Si";
                    }
                    if (llistaPermisos[cont].VeureDespeses != null && (bool)llistaPermisos[cont].VeureDespeses)
                    {
                        fin = "Si";
                    }
                    string[] linia = new string[] { llistaPermisos[cont].Nom, admin, resp, tec, fin };
                    gridPermisos.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Llista tipus
        /*
         * Metode per carregar el grid dels tipus de despeses existents
         */
        private void llistarTipus()
        {
            //Es busquen els tipus de despeses existents
            List<ServeiWeb.Tipus> llistaTipus = Tipus.llistaTipus();
            //Es neteja el grid de tipus de despeses
            gridTipus.Rows.Clear();
            //Si es troben permisos s'afegeixen al grid
            if (llistaTipus != null && llistaTipus.Count > 0)
            {
                for (int cont = 0; cont < llistaTipus.Count; cont++)
                {
                    string[] linia;
                    if ((bool)llistaTipus[cont].Habilitat)
                    {
                        linia = new string[] { llistaTipus[cont].Id.ToString(), null, llistaTipus[cont].Nom, "Si" };
                    }
                    else
                    {
                        linia = new string[] { llistaTipus[cont].Id.ToString(), null, llistaTipus[cont].Nom, "No" };
                    }
                    gridTipus.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Pestanya seleccionada
        /*
         * Event que recarregar els llistats, depenent de la navegacio per les pestanyes
         */
        private void clickPestanya(object sender, TabControlCancelEventArgs e)
        {
            if (e.TabPageIndex == 0)
            {
                llistarUsuaris();
            }
            else if (e.TabPageIndex == 1)
            {
                llistarCarrecs();
                llistarPermisos();
            }
            else if (e.TabPageIndex == 2)
            {
                llistarTipus();
            }
        }
        #endregion

        #region Habilitar usuari
        /*
         * Event que habilita un usuari si esta deshabilitat 
         */
        private void clickHabilitarUsuari(object sender, EventArgs e)
        {
            //Busquem els usuaris amb el check actiu
            List<Int32> seleccionats = usuarisSelecionats();
            //Si no s' han seleccionat usuaris no es fa res
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un usuari i esta deshabilitat,es procedeix a habilitar-lo
            else if (seleccionats.Count == 1)
            {
                if (!Usuari.usuariHabilitat(seleccionats[0]))
                {
                    if (Usuari.habilitarUsuari(seleccionats[0]))
                    {
                        llistarUsuaris();
                    }
                }
            }
            //No es pot habilitar mes d'un usuari a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_USUARI);
            }
        }
        #endregion

        #region Deshabilitar usuari
        /*
         * Event que deshabilita un usuari si esta habilitat 
         */
        private void clickDeshabilitarUsuari(object sender, EventArgs e)
        {
            //Busquem els usuaris amb el check actiu
            List<Int32> seleccionats = usuarisSelecionats();
            //Si no s' han seleccionat usuaris no es fa res
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un usuari i esta habilitat,es procedeix a deshabilitar-lo
            else if (seleccionats.Count == 1)
            {
                if (Usuari.usuariHabilitat(seleccionats[0]))
                {
                    if (Usuari.deshabilitarUsuari(seleccionats[0]))
                    {
                        llistarUsuaris();
                    }
                }
            }
            //No es pot deshabilitar mes d'un usuari a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_USUARI);
            }
        }
        #endregion

        #region Modificar usuari
        /*
         * Event per mpodificar un usuari existent
         */
        private void clickModificarUsuari(object sender, EventArgs e)
        {
            //Busquem els usuaris amb el check actiu
            List<Int32> seleccionats = usuarisSelecionats();
            //Si no s' han seleccionat usuaris no es fa res
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un usuari,es procedeix a obrir el formulari adient
            else if (seleccionats.Count == 1)
            {
                this.Visible = false;
                using (AdministracioUsuari administrarUsuari = new AdministracioUsuari(seleccionats[0]))
                {
                    //Si dialogResult es OK, vol dir que s'ha modificat l'usuari correctament i cal actualitzar la llista d'usuaris
                    if (administrarUsuari.ShowDialog() == DialogResult.OK)
                    {
                        llistarUsuaris();
                    }
                }
                this.Visible = true;
            }
            //No es pot administrar mes d'un usuari a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_USUARI);
            }
        }
        #endregion

        #region Usuari nou
        /*
         * Event per afegir un nou usuari
         */
        private void clickAfegirUsuari(object sender, EventArgs e)
        {
            this.Visible = false;
            using (AdministracioUsuari administrarUsuari = new AdministracioUsuari(0))
            {
                //Si dialogResult es OK, vol dir que s'ha afegit l'usuari correctament
                if (administrarUsuari.ShowDialog() == DialogResult.OK)
                {
                    //Accedim a l'usuari estatic i l' afegim a la llista d'usuaris
                    ServeiWeb.Usuari usuariNou = administrarUsuari.usuariAfegit;
                    string[] linia = new string[] { usuariNou.Id.ToString(), null, usuariNou.Dni, usuariNou.Nom, usuariNou.Cognoms, Carrec.buscarCarrecPerId(usuariNou.IdCarrec).CarrecNom, Usuari.buscarUsuariPerId(usuariNou.IdResponsable).DniUsuari, "Si" };
                    gridUsuaris.Rows.Add(linia);
                }
            }
            this.Visible = true;
        }
        #endregion

        #region Seleccio usuaris
        /*
         * Metode que proporciona els ids dels usuaris amb el checkbox clicat 
         */
        private List<Int32> usuarisSelecionats()
        {
            List<Int32> files = new List<Int32>();
            for (int cont = 0; cont < gridUsuaris.Rows.Count; cont++)
            {
                //Si esta seleccionat s' afegeix a la llista
                if ((bool)gridUsuaris.Rows[cont].Cells["columnaSeleccionar"].FormattedValue)
                {
                    Int32 fila = Convert.ToInt32(gridUsuaris.Rows[cont].Cells[0].Value);
                    files.Add(fila);
                }
            }
            //Es retorna la llista amb els ids seleccionats
            return files;
        }
        #endregion

        #region Modificar carrec
        /*
         * Event per modificar un usuari existent. 
         */
        private void clickModificarCarrec(object sender, EventArgs e)
        {
            List<Int32> seleccionats = carrecsSeleccionats();
            //Si no s' han seleccionat usuaris no es fa res
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si hi ha un usuari seleccionat, obrim el formulari per la seva administracio enviant el seu id
            else if (seleccionats.Count == 1)
            {
                this.Visible = false;
                using (AdministracioCarrec administrarCarrec = new AdministracioCarrec(seleccionats[0]))
                {
                    //Si la modificació s'ha realitzat correctament, cal actualitzar el llistat d'usuaris
                    if (administrarCarrec.ShowDialog() == DialogResult.OK)
                    {
                        llistarCarrecs();
                    }
                }
                this.Visible = true;
            }
            //Nomes es permet modificar un usuari a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_CARREC);
            }
        }
        #endregion

        #region Afegir carrec
        /*
         * Event per afegir un nou carrec 
         */
        private void clickAfegirCarrec(object sender, EventArgs e)
        {
            using (AdministracioCarrec administrarCarrec = new AdministracioCarrec(0))
            {
                if (administrarCarrec.ShowDialog() == DialogResult.OK)
                {
                    string[] linia;
                    string habilitat = "No";
                    //Si l' usuari esta activat, s' indica
                    if ((bool)administrarCarrec.carrecAfegit.Habilitat)
                    {
                        habilitat = "Si";
                    }

                    //Si no te permisos afegim el carrec al grid
                    if (administrarCarrec.carrecAfegit.IdPermisos == 0)
                    {
                        linia = new string[] { administrarCarrec.carrecAfegit.Id.ToString(), null, administrarCarrec.carrecAfegit.Nom, "<<Sense permisos>>", habilitat };
                    }
                    //Si te permisos, es busca el seu nom per id
                    else
                    {
                        linia = new string[] { administrarCarrec.carrecAfegit.Id.ToString(), null, administrarCarrec.carrecAfegit.Nom, Permisos.buscarPermisosPerId((int)administrarCarrec.carrecAfegit.IdPermisos).Nom, habilitat };
                    }
                    gridCarrecs.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Habilitar carrec
        /*
         * Event per habilitar un carrec 
         */
        private void clickHabilitarCarrec(object sender, EventArgs e)
        {
            List<Int32> seleccionats = carrecsSeleccionats();
            //Si no s' han seleccionat carrecs no es fa res
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            else if (seleccionats.Count == 1)
            {
                //Si s'ha seleccionat un carrec i esta deshabilitat,s'habilita
                if (!Carrec.verificarCarrec(seleccionats[0]))
                {
                    if (Carrec.habilitarCarrec(seleccionats[0]))
                    {
                        llistarCarrecs();
                    }
                }
            }
            //No es pot habilitar més d'un carrec a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_CARREC);
            }
        }
        #endregion

        #region Deshabilitar carrec
        /*
         * Event per deshabilitar un carrec 
         */
        private void clickDeshabilitarCarrec(object sender, EventArgs e)
        {
            List<Int32> seleccionats = carrecsSeleccionats();
            //Si no s' han seleccionat carrecs no es fa res
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un carrec i esta habilitat,es deshabilita
            else if (seleccionats.Count == 1)
            {
                if (Carrec.verificarCarrec(seleccionats[0]))
                {
                    if (Carrec.deshabilitarCarrec(seleccionats[0]))
                    {
                        llistarCarrecs();
                    }
                }
            }
            //No es pot deshabilitar un carrec a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_CARREC);
            }
        }
        #endregion

        #region Carrecs seleccionats
        /*
         * Metode per obtenir els ids dels carrecs seleccionats 
         */
        private List<Int32> carrecsSeleccionats()
        {
            List<Int32> files = new List<Int32>();
            for (int cont = 0; cont < gridCarrecs.Rows.Count; cont++)
            {
                if ((bool)gridCarrecs.Rows[cont].Cells["columnaSeleccionats"].FormattedValue)
                {
                    Int32 fila = Convert.ToInt32(gridCarrecs.Rows[cont].Cells[0].Value);
                    files.Add(fila);
                }
            }
            return files;
        }
        #endregion

        #region Afegir tipus
        /*
         * Event per afegir nous tipus de despeses 
         */
        private void clickAfegirTipus(object sender, EventArgs e)
        {
            this.Visible = false;
            //Carregar formulari per un nou tipus,indicant id de tipus 0 
            using (AdministracioTipus administracioTipus = new AdministracioTipus(0))
            {
                //Si s' ha afegit correctament, s´afegeix a la llista de tipus
                if (administracioTipus.ShowDialog() == DialogResult.OK)
                {
                    if (administracioTipus.tipusAfegit != null)
                    {
                        ServeiWeb.Tipus tipusNou = administracioTipus.tipusAfegit;
                        string[] linia = new string[] { tipusNou.Id.ToString(), null, tipusNou.Nom, "Si" };
                        gridTipus.Rows.Add(linia);
                    }
                }
            }
            this.Visible = true;
        }
        #endregion

        #region Modificar tipus
        /*
         * Event per modificar un tipus existent
         */
        private void clickModificarTipus(object sender, EventArgs e)
        {
            List<Int32> seleccionats = tipusSelecionats();
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un tipus, es carrega el formulari per modificar enviant l' id de la despesa per parametre
            else if (seleccionats.Count == 1)
            {
                using (AdministracioTipus administracioTipus = new AdministracioTipus(seleccionats[0]))
                {
                    this.Visible = false;
                    administracioTipus.ShowDialog();
                    //S'actualitza la llista de tipus
                    llistarTipus();
                    this.Visible = true;
                }
            }
            //Només es pot modificar un tipus a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_TIPUS);
            }
        }
        #endregion

        #region Habilitar tipus
        /*
         * Event per habilitar tipus de despeses 
         */
        private void clickHabilitarTipus(object sender, EventArgs e)
        {
            List<Int32> seleccionats = tipusSelecionats();
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un tipus i esta deshabilitat,s' habilita
            else if (seleccionats.Count == 1)
            {
                if (!Tipus.verificarTipus(seleccionats[0]))
                {
                    if (Tipus.habilitarTipus(seleccionats[0]))
                    {
                        llistarTipus();
                    }
                }
            }
            //Nomes es pot habilitar un tipus a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_TIPUS);
            }
        }
        #endregion

        #region Deshabilitar tipus
        /*
         * Event per deshabilitar tipus de despeses 
         */
        private void clickDeshabilitarTipus(object sender, EventArgs e)
        {
            List<Int32> seleccionats = tipusSelecionats();
            if (seleccionats.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un tipus i esta habilitat,es deshabilita
            else if (seleccionats.Count == 1)
            {
                if (Tipus.verificarTipus(seleccionats[0]))
                {
                    if (Tipus.deshabilitarTipus(seleccionats[0]))
                    {
                        llistarTipus();
                    }
                }
            }
            //Nomes es pot deshabilitar un tipus a l'hora
            else if (seleccionats.Count > 1)
            {
                MessageBox.Show(Missatges.UN_TIPUS);
            }
        }
        #endregion

        #region Tipus seleccionats
        /*
         * Metode que retorna una llista amb els id dels tipus de despeses seleccionats
         */
        private List<Int32> tipusSelecionats()
        {
            List<Int32> files = new List<Int32>();
            for (int cont = 0; cont < gridTipus.Rows.Count; cont++)
            {
                if ((bool)gridTipus.Rows[cont].Cells["columnaSeleccionat"].FormattedValue)
                {
                    Int32 fila = Convert.ToInt32(gridTipus.Rows[cont].Cells[0].Value);
                    files.Add(fila);
                }
            }
            return files;
        }
        #endregion

        #region Tornar
        /*
         * Event per tornar al menú principal
         */
        private void clickTornar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }
        #endregion
    }
}
