﻿using ClientSGDEscriptori.Accions;
using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.Administrar
{
    public partial class AdministracioUsuari : Form
    {
        private Int32 usuariId;
        public ServeiWeb.Usuari usuariAfegit { get; set; }

        public AdministracioUsuari(Int32 usuariId)
        {
            InitializeComponent();
            this.usuariId = usuariId;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void AdministrarUsuariCarregat(object sender, EventArgs e)
        {
            List<ServeiWeb.Carrec> llistaCarrecs = Carrec.llistaCarrecsDisponibles();
            if (llistaCarrecs != null && llistaCarrecs.Count > 0)
            {
                for (int cont = 0; cont < llistaCarrecs.Count; cont++)
                {
                    comboCarrec.Items.Add(llistaCarrecs[cont].Nom);
                }
            }
            //Si l' usuari existeix es carreguen les seves dades
            if (usuariId != 0)
            {
                Usuari usuari = Usuari.buscarUsuariPerId(usuariId);
                if (usuari != null)
                {
                    textDNI.Text = usuari.DniUsuari;
                    textNom.Text = usuari.NomUsuari;
                    textCognoms.Text = usuari.CognomsUsuari;
                    comboCarrec.Text = Carrec.buscarCarrecPerId(usuari.CarrecId).CarrecNom;
                    textResponsable.Text = Usuari.buscarUsuariPerId(usuari.ResponsableId).DniUsuari;
                }
            }

        }
        #endregion

        #region Guardar Usuari
        /*
         * Event que revisa les dades de l' usuari i proecdeix a modificar-lo o a crear un de nou
         */
        private void clickGuardar(object sender, EventArgs e)
        {
            //Si tots els camps son correctes
            if (textDNI.Text.Length > 0 && textNom.Text.Length > 0 && textCognoms.Text.Length > 0 && textResponsable.Text.Length > 0 && comboCarrec.Text.Length > 0)
            {
                //Si el responsable existeix o es el propi usuari
                if (textDNI.Text.Equals(textResponsable.Text) || Usuari.existeixUsuariDNI(textResponsable.Text))
                {
                    #region Usuari nou
                    //Crear nou usuari
                    if (usuariId == 0)
                    {
                        //Si les contrasenyes estan omplertes i son iguals
                        if (textContrasenya1.Text.Length > 0 && textContrasenya2.Text.Length > 0 && textContrasenya1.Text.Equals(textContrasenya2.Text))
                        {
                            ServeiWeb.Usuari usuariNou;
                            //Si el responsable de l'usuari es el propi usuari
                            if (textDNI.Text.Equals(textResponsable.Text))
                            {
                                usuariNou = Usuari.afegirUsuari(textDNI.Text, textNom.Text, textCognoms.Text, Carrec.carrecPernom(comboCarrec.Text).Id, 1, textContrasenya1.Text);
                                //Es modifica el responsable i s'estableix el nou id de l'usuari afegit
                                if (usuariNou != null)
                                {
                                    usuariNou.IdResponsable = usuariNou.Id;
                                    Usuari.modificarUsuari(usuariNou.Id, usuariNou.Dni, usuariNou.Nom, usuariNou.Cognoms, usuariNou.IdCarrec, usuariNou.Id, "");
                                }
                            }
                            //Si es responsable es un usuari diferent 
                            else
                            {
                                usuariNou = Usuari.afegirUsuari(textDNI.Text, textNom.Text, textCognoms.Text, Carrec.carrecPernom(comboCarrec.Text).Id, Usuari.buscarUsuariPerDni(textResponsable.Text).IdUsuari, textContrasenya1.Text);
                            }
                            if (usuariNou != null)
                            {
                                usuariAfegit = usuariNou;
                                DialogResult = DialogResult.OK;
                            }
                        }
                        else
                        {
                            MessageBox.Show(Missatges.CONTRASENYA_COINCIDIR);
                        }
                    }
                    #endregion

                    #region Modificar usuari
                    //Modificar usuari existent
                    else if (usuariId > 0)
                    {
                        //Si es modifica la contrasenya, pero no coincideix la verificacio
                        if ((textContrasenya1.Text.Length > 0 && !textContrasenya1.Text.Equals(textContrasenya2.Text)) || (textContrasenya2.Text.Length > 0 && !textContrasenya2.Text.Equals(textContrasenya1.Text)))
                        {
                            MessageBox.Show(Missatges.CONTRASENYA_COINCIDIR);
                        }
                        else
                        {
                            //Si l'usuari es el seu propi responsable
                            if (textDNI.Text.Equals(textResponsable.Text))
                            {
                                if (Usuari.modificarUsuari(usuariId, textDNI.Text, textNom.Text, textCognoms.Text, Carrec.carrecPernom(comboCarrec.Text).Id, 1, textContrasenya1.Text))
                                {
                                    if (Usuari.modificarUsuari(usuariId, textDNI.Text, textNom.Text, textCognoms.Text, Carrec.carrecPernom(comboCarrec.Text).Id, Usuari.buscarUsuariPerDni(textResponsable.Text).IdUsuari, textContrasenya1.Text))
                                    {
                                        DialogResult = DialogResult.OK;
                                    }
                                }
                            }
                            //Si l'usuari no es el propi responsable
                            else
                            {
                                if (Usuari.modificarUsuari(usuariId, textDNI.Text, textNom.Text, textCognoms.Text, Carrec.carrecPernom(comboCarrec.Text).Id, Usuari.buscarUsuariPerDni(textResponsable.Text).IdUsuari, textContrasenya1.Text))
                                {
                                    DialogResult = DialogResult.OK;
                                }
                            }
                        }
                    }
                    #endregion
                }
                //No existeix el responsable
                else
                {
                    MessageBox.Show(Missatges.RESPONSABLE_INCORRECTE);
                }
            }
            else
            {
                MessageBox.Show(Missatges.DADES_INCOMPLETES);
            }
        }
        #endregion

        #region Tornar
        /*
         * Event per tornar a la pantalla principal d'administració
         */
        private void clickCancelar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }
        #endregion
    }
}
