﻿using ClientSGDEscriptori.Accions;
using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori
{
    public partial class Calendari : Form
    {
        private bool responsable;

        public Calendari(bool responsable)
        {
            InitializeComponent();
            this.responsable = responsable;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void calendariCarregat(object sender, EventArgs e)
        {
            llistaFestius();
            if (responsable)
            {
                botoEliminar.Visible = true;
                panellAfegir.Visible = true;
            }
        }
        #endregion

        #region Llista de festius
        /*
         * Metode que carrega els dies festius existents
         */
        private void llistaFestius()
        {
            gridFestius.Rows.Clear();
            List<ServeiWeb.Festiu> llista = Festiu.llistaDeFestius();
            if (llista != null && llista.Count > 0)
            {
                for (int cont = 0; cont < llista.Count; cont++)
                {
                    string[] linia = new string[] {null, llista[cont].Data.ToString("dd/MM/yyyy"), llista[cont].Descripcio };
                    gridFestius.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Data seleccionada
        /*
         * Event del calendari que donada una data clicada, s'escriu al camp corresponent 
         */
        private void clickData(object sender, DateRangeEventArgs e)
        {
            textData.Text = e.End.Date.ToString("dd/MM/yyyy");
        }
        #endregion

        #region Afegir data
        /*
         * Event per afegir una nova data festiva i la seva descripcio
         */
        private void clickAfegir(object sender, EventArgs e)
        {
            if (textData.Text.Length > 0 && textDescripció.Text.Length > 0)
            {
                //S' afegeix el nou festiu i s'afegeix a la llista de dates
                ServeiWeb.Festiu festiuNou = Festiu.afegirFestiu(Convert.ToDateTime(textData.Text), textDescripció.Text);
                if (festiuNou != null)
                {
                    string[] linia = new string[] {null, festiuNou.Data.ToString("dd/MM/yyyy"), festiuNou.Descripcio };
                    gridFestius.Rows.Add(linia);
                    textData.Text = string.Empty;
                    textDescripció.Text = string.Empty;
                }
            }
            else
            {
                MessageBox.Show(Missatges.DADES_INCOMPLETES);
            }
        }
        #endregion

        #region Eliminar data
        /*
         * Event per eliminar una data festiva existent 
         */
        private void clickEliminar(object sender, EventArgs e)
        {
            List<DateTime> files = festiusSeleccionats();
            //Si es selecciona una data es procedeix a eliminar-la
            if (files.Count == 1)
            {
                if (Festiu.eliminarFestiu(files[0]))
                {
                    llistaFestius();
                }
            }
            //Nomes es pot eliminar una data a l'hora
            else if (files.Count > 1)
            {
                MessageBox.Show(Missatges.UN_FESTIU);
            }
        }
        #endregion

        #region Festius seleccionats
        /*
         * Metode que retorna una llista amb els ids de les dates seleccionades al grid de dates
         */
        private List<DateTime> festiusSeleccionats()
        {
            List<DateTime> files = new List<DateTime>();
            for (int cont = 0; cont < gridFestius.Rows.Count; cont++)
            {
                if ((bool)gridFestius.Rows[cont].Cells["columnaSeleccionats"].FormattedValue)
                {
                    DateTime fila = Convert.ToDateTime(gridFestius.Rows[cont].Cells["columnaData"].Value);
                    files.Add(fila);
                }
            }
            return files;
        }
        #endregion

        #region Tornar
        /*
         * Event per tornar al menú principal
         */
        private void clickTornar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }
        #endregion
    }
}
