﻿using System;
using System.Configuration;
using System.Net;
using System.ServiceModel;

namespace ClientSGDEscriptori
{
    class ConnexioServei
    {
        #region Iniciar connexio amb servei web
        /*
         * Metode que inicia i retorna una nova connexio amb el servei web, agafant la URL de l'arxiu de configuracio
         */
        public static ServeiWeb.ServeiClient nouServei()
        {
            ServeiWeb.ServeiClient servei = new ServeiWeb.ServeiClient();
            string urlCompleta = ConfigurationManager.AppSettings["UrlCompletaServei"];
            servei.Endpoint.Address = new EndpointAddress(new Uri(urlCompleta));
            return servei;
        }
        #endregion

        #region Verificar connexio amb internet
        /*
         * Metode que verifica si hi ha connexio amb internet i retorna el resultat
         */
        public static bool verificarConnexioInternet()
        {
            if (System.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable() == true)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion

        #region Verificar connexio amb servei web
        /*
         * Metode que verifica si hi ha connexio amb el servei web i retorna el resultat
         */
        public static bool verificarConnexioServeiWeb()
        {
            try
            {
                string urlCompleta = ConfigurationManager.AppSettings["UrlCompletaServei"];
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(urlCompleta);
                request.Timeout = 5000;
                request.Credentials = CredentialCache.DefaultNetworkCredentials;
                using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                {
                    if (response.StatusCode == HttpStatusCode.OK)
                    {
                        return true;
                    }
                }
                return false;
            }
            catch
            {
                return false;
            }
        }
        #endregion

        #region Modificar dades de App.config
        /*
         * Metode per modificar les dades de connexio establertes a l'arxiu de configuracio.
         */
        public static void canviarDadesServei(string ip,string url)
        {
            Configuration configuration = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
            configuration.AppSettings.Settings.Remove("IPServei");
            configuration.AppSettings.Settings.Add("IPServei", ip);
            configuration.AppSettings.Settings.Remove("PublicacioServei");
            configuration.AppSettings.Settings.Add("PublicacioServei", url);
            configuration.AppSettings.Settings.Remove("UrlCompletaServei");
            configuration.AppSettings.Settings.Add("UrlCompletaServei", "http://" + ip + "/" + url);
            configuration.Save(ConfigurationSaveMode.Modified);
            ConfigurationManager.RefreshSection("appSettings");
            ConfigurationManager.RefreshSection("connectionStrings");
            ConfigurationManager.RefreshSection("PublicacioServei");
        }
        #endregion

        #region Verificar format IP
        /*
         * Metode que retorna si una ip te un format adequat
         */
        public static bool verificarIp(string ip)
        {
            IPAddress ipTemp;
            if (IPAddress.TryParse(ip, out ipTemp))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion
    }
}
