﻿using ClientSGDEscriptori.Accions;
using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

namespace ClientSGDEscriptori.GestioDespeses
{
    public partial class GestioDespesa : Form
    {
        private static decimal maxImport = 1000000;//Import maxim permes
        private static int maxAdjunt = 10485760;//10 Megabytes
        private Int32 idDespesa, fullId, facturaId;

        public GestioDespesa(Int32 idDespesa, Int32 fullId)
        {
            InitializeComponent();
            this.idDespesa = idDespesa;
            this.fullId = fullId;
            facturaId = 0;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void gestioDespesaCarregat(object sender, EventArgs e)
        {
            llistaDespeses();
        }
        #endregion

        #region Llistat despeses
        /*
         * Metode que carrega una despesa si existeix i carrega les seves dades, 
         * i carrega els tipus de despesa existents a la llista de tipus
         */
        private void llistaDespeses()
        {
            List<ServeiWeb.Tipus> llista = Tipus.llistaTipusActius();
            if (llista.Count > 0)
            {
                for (int cont = 0; cont < llista.Count; cont++)
                {
                    comboTipus.Items.Add(llista[cont].Nom);
                }
            }
            //Despesa existent
            if (idDespesa != 0)
            {
                ServeiWeb.Despesa despesa = Despesa.buscarDespesa(idDespesa);
                textData.Text = despesa.Data.ToString("dd/MM/yyy");
                textImport.Text = despesa.Import.ToString();
                comboTipus.Text = despesa.NomTipus;
                textComentari.Text = despesa.Comentari;
                //Si la despesa te adjunt, es carrega el nom de l'adjunt
                if (despesa.Adjunt)
                {
                    textAdjunt.Text = Factura.buscarNomFactura(idDespesa);
                    facturaId = Factura.buscarIdFactura(idDespesa);
                    botoEliminar.Visible = true;
                }
            }
        }
#endregion

        #region Guardar despesa
        /*
         * Event per modificar un usuari o crear un de nou
         */
        private void clickGuardar(object sender, EventArgs e)
        {
            //Si la despesa te data i tipus de despesa
            if (textData.Text.Length > 0 && textImport.Text.Length > 0 && !comboTipus.Text.Equals(string.Empty))
            {
                //Es verifica que l'import tingui el format adient
                if (verificarImport(textImport.Text))
                {
                    decimal import = decimal.Parse(textImport.Text);
                    if (import <= (maxImport))
                    {
                        if (textAdjunt.Text.Equals(string.Empty) && import >= 90)
                        {
                            MessageBox.Show(Missatges.FACTURA_OBLIGAT);
                        }
                        else
                        {
                            bool resultat = false;
                            byte[] adjunt = new byte[0];
                            string nomArxiu = string.Empty; ;
                            Int32? tipusId = Tipus.buscarTipusPerNom(comboTipus.Text).Id;
                            #region Despesa nova
                            //Nova despesa
                            if (idDespesa == 0)
                            {
                                //Hi ha adjunt
                                if (textAdjunt.Text.Length > 0)
                                {
                                    adjunt = File.ReadAllBytes(textAdjunt.Text);
                                    nomArxiu = Path.GetFileName(textAdjunt.Text);
                                }
                                //No hi ha adjunt
                                else
                                {
                                    //No fer res
                                }

                                if (tipusId != null)
                                {
                                    ServeiWeb.Despesa despesaNova = Despesa.afegirDespesaNova(fullId, Convert.ToDateTime(textData.Text), import, (Int32)tipusId, textComentari.Text, nomArxiu, adjunt);
                                    if (despesaNova != null)
                                    {
                                        DialogResult = DialogResult.OK;
                                    }
                                }
                            }
                            #endregion
                            #region Modificar despesa
                            //Modificar despesa existent
                            else
                            {
                                //Hi havia adjunt
                                if (facturaId != 0)
                                {
                                    //Havia i s'elimina l'adjunt
                                    if (textAdjunt.Text.Length == 0)
                                    {
                                        resultat = Despesa.modificarDespesa(idDespesa, Convert.ToDateTime(textData.Text), import, (Int32)tipusId, textComentari.Text, facturaId, nomArxiu, adjunt);
                                    }
                                    //Havia i es modifica l'adjunt
                                    else if (textAdjunt.Text.Length > 0 && textAdjunt.Text.Contains("\\"))
                                    {
                                        adjunt = File.ReadAllBytes(textAdjunt.Text);
                                        nomArxiu = Path.GetFileName(textAdjunt.Text);
                                        resultat = Despesa.modificarDespesa(idDespesa, Convert.ToDateTime(textData.Text), import, (Int32)tipusId, textComentari.Text, facturaId, nomArxiu, adjunt);
                                    }
                                    //Havia i no es modifica l'adjunt
                                    else if (textAdjunt.Text.Length > 0 && !textAdjunt.Text.Contains("\\"))
                                    {
                                        nomArxiu = textAdjunt.Text;
                                        resultat = Despesa.modificarDespesa(idDespesa, Convert.ToDateTime(textData.Text), import, (Int32)tipusId, textComentari.Text, facturaId, nomArxiu, adjunt);
                                    }
                                }
                                //No havia adjunt
                                else
                                {
                                    //No havia i s'afegeix adjunt
                                    if (textAdjunt.Text.Length > 0)
                                    {
                                        adjunt = File.ReadAllBytes(textAdjunt.Text);
                                        nomArxiu = Path.GetFileName(textAdjunt.Text);
                                        resultat = Despesa.modificarDespesa(idDespesa, Convert.ToDateTime(textData.Text), import, (Int32)tipusId, textComentari.Text, facturaId, nomArxiu, adjunt);
                                    }
                                    //No havia i no s'afegeix adjunt
                                    else
                                    {
                                        resultat = Despesa.modificarDespesa(idDespesa, Convert.ToDateTime(textData.Text), import, (Int32)tipusId, textComentari.Text, facturaId, nomArxiu, adjunt);
                                    }
                                }
                                //Despres de crear o modificar la despesa, es torna al formulari anterior
                                if (resultat)
                                {
                                    DialogResult = DialogResult.OK;
                                }
                            }
                            #endregion
                        }
                    }
                    else
                    {
                        MessageBox.Show(Missatges.IMPORT_MAXIM);
                    }
                }
                else
                {
                    MessageBox.Show(Missatges.IMPORT_INCORRECTE);
                }
            }
            else
            {
                MessageBox.Show(Missatges.DADES_INCOMPLETES);
            }
        }
        #endregion

        #region Verificar import
        /*
         * Metode que canvia l'import de string a decimal, nomes si el canvi es posible
         */
        private bool verificarImport(string import)
        {
            decimal valor;
            if (decimal.TryParse(import, out valor))
            {
                return true;
            }
            return false;
        }
        #endregion

        #region Caracters no permesos
        /*
         * Event que bloqueja l'insercio de caracters no valids a l'import
         */
        private void caractersImport(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == '.' || e.KeyChar == '-')
            {
                e.Handled = true;
            }
        }
        #endregion

        #region Data seleccionada
        /*
         * Event que escriu la data seleccionada de la despesa
         */
        private void dataSeleccionada(object sender, DateRangeEventArgs e)
        {
            textData.Text = e.End.ToString("dd/MM/yyyy");
        }
        #endregion

        #region Buscar adjunt
        /*
         * Event que inicia el buscador d'arxius per seleccionar l'adjunt de la factura
         */
        private void clickBuscarAdjunt(object sender, EventArgs e)
        {
            DialogResult resultat = buscadorArxius.ShowDialog();
            if (resultat == DialogResult.OK)
            {
                if (File.ReadAllBytes(buscadorArxius.FileName).Length <= maxAdjunt)
                {
                    textAdjunt.Text = buscadorArxius.FileName;
                    botoEliminar.Visible = true;
                }
                else
                {
                    MessageBox.Show(Missatges.MAXIM_ADJUNT);
                }
            }
        }
        #endregion

        #region Eliminar adjunt
        /*
         * Event que elimina les dades d'un adjunt existent al formulari
         */
        private void clickEliminarFactura(object sender, EventArgs e)
        {
            textAdjunt.Text = string.Empty;
            botoEliminar.Visible = false;
        }
        #endregion

        #region Tornar
        /*
         * Event per tornar a la pantalla del llistat de despeses
         */
        private void clickCancelar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }
        #endregion
    }
}
