﻿using ClientSGDEscriptori.Accions;
using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;

namespace ClientSGDEscriptori.GestioDespeses
{
    public partial class GestioDespesesPersonals : Form
    {
        private Int32 fullId;
        private bool modificable = false;

        public GestioDespesesPersonals(Int32 fullId, bool modificable)
        {
            InitializeComponent();
            this.fullId = fullId;
            this.modificable = modificable;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void gestioDespesesPersonalsCarregat(object sender, EventArgs e)
        {
            if (modificable)
            {
                botoAfegir.Enabled = true;
                botoModificar.Enabled = true;
                botoEliminar.Enabled = true;
            }
            llistarDespeses();
        }
        #endregion

        #region Llista despeses
        /*
         * Metode per carregar les despeses del full
         */
        private void llistarDespeses()
        {

            gridDespeses.Rows.Clear();
            List<ServeiWeb.Despesa> llista = Despesa.despesesDeFull(fullId);
            if (llista != null && llista.Count == 0)
            {
                botoEliminar.Enabled = false;
                botoModificar.Enabled = false;
            }
            else
            {
                //Si no esta pendent de validacio, es poden modificar les despeses
                if (modificable)
                {
                    botoEliminar.Enabled = true;
                    botoModificar.Enabled = true;
                }
                for (int cont = 0; cont < llista.Count; cont++)
                {
                    //string per indicar si hi ha adjunt o no
                    string adjuntText;
                    if (llista[cont].Adjunt)
                    {
                        adjuntText = "Obrir";
                    }
                    else
                    {
                        adjuntText = "Sense";
                    }
                    //string per indicar si hi han comentaris o no
                    string comentari = llista[cont].Comentari.ToString();
                    if (comentari.Length == 0)
                    {
                        comentari = "Sense";
                    }
                    else
                    {
                        comentari = "Veure";
                    }
                    string[] linia = new string[] { llista[cont].Id.ToString(), null, llista[cont].Data.ToString("dd/MM/yyyy"), llista[cont].Import.ToString(), llista[cont].NomTipus, comentari, adjuntText };
                    gridDespeses.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Despesa nova
        /*
         * Event per crear una nova despesa
         */
        private void clickDespesaNova(object sender, EventArgs e)
        {
            this.Visible = false;
            using (GestioDespesa gestioDespesa = new GestioDespesa(0, fullId))
            {
                //Si la despesa s'afegeix correctament,s'actualitza la llista de despeses
                if (gestioDespesa.ShowDialog() == DialogResult.OK)
                {
                    //S'actualitza la llista sencera per evitar massa accions
                    llistarDespeses();
                }
            }
            this.Visible = true;
        }
        #endregion

        #region Eliminar despeses
        /*
         *  Event per eliminar les despeses del full
         */
        private void clickEliminarMarcades(object sender, EventArgs e)
        {
            List<Int32> llista = despesesSelecionades();
            if (llista.Count == 0)
            {
                //No fer res
            }
            else
            {
                List<bool> llistaConteAdjunt = new List<bool>();
                for (int cont = 0; cont < llista.Count; cont++)
                {
                    llistaConteAdjunt.Add(Despesa.verificarAdjunt(llista[cont]));
                }
                if (Despesa.eliminarDespeses(llista, llistaConteAdjunt))
                {
                    llistarDespeses();
                }

            }
        }
        #endregion

        #region Modificar despesa
        /*
         * Event per modificar una despesa existent
         */
        private void clickModificarDespesa(object sender, EventArgs e)
        {
            List<Int32> llista = despesesSelecionades();
            //Si nomes es selecciona una despesa no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si hi ha una despesa seleccionada
            else if (llista.Count == 1)
            {
                this.Visible = false;
                //S'obre el formulari adient i s'envia per parametre el id de la despesa existent i el id full al que pertany
                using (GestioDespesa gestioDespesa = new GestioDespesa(llista[0], fullId))
                {
                    if (gestioDespesa.ShowDialog() == DialogResult.OK)
                    {
                        //Si es modifica correctament, s'actualitza la llista de despeses
                        llistarDespeses();
                    }
                }
                this.Visible = true;
            }
            //Nomes es pot modificar una despesa a l'hora
            else if (llista.Count > 1)
            {
                MessageBox.Show(Missatges.UN_DESPESA);
            }
        }
        #endregion

        #region Despeses seleccionades
        /*
         * Metode que retorna un llistat de les despeses seleccionades 
         */
        private List<Int32> despesesSelecionades()
        {
            List<Int32> files = new List<Int32>();
            for (int cont = 0; cont < gridDespeses.Rows.Count; cont++)
            {
                if ((bool)gridDespeses.Rows[cont].Cells["columnaCheck"].FormattedValue)
                {
                    Int32 fila = Convert.ToInt32(gridDespeses.Rows[cont].Cells[0].Value);
                    files.Add(fila);
                }
            }
            return files;
        }
        #endregion

        #region Cel·la seleccionada
        /*
         * Event que verifica la fila i columna seleccionades al grid de despeses 
         */
        private void clickCela(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                //Si es la columna de comentaris i hi ha comentari, s'obre el formulari per mostrar comentaris
                if (e.ColumnIndex == 5 && gridDespeses.Rows[e.RowIndex].Cells["columnaComentaris"].Value.Equals("Veure"))
                {
                    using (VisioComentari comentari = new VisioComentari("Comentari:", Despesa.comentarisDespesa(Convert.ToInt32(gridDespeses.Rows[e.RowIndex].Cells["columnaId"].Value))))
                    {
                        comentari.ShowDialog();
                    }
                }
                //Si es la columna d'adjunts i hi ha adjunt, es descarrega l'adjunt i s'obre
                if (e.ColumnIndex == 6 && gridDespeses.Rows[e.RowIndex].Cells["columnaAdjunt"].Value.Equals("Obrir"))
                {
                    //Es guarda l'adjunt a un array de bytes
                    byte[] adjunt = Factura.buscarFacturaAdjunta(Convert.ToInt32(gridDespeses.Rows[e.RowIndex].Cells["columnaId"].Value));
                    if (adjunt != null)
                    {
                        string nomFactura = Factura.buscarNomFactura(Convert.ToInt32(gridDespeses.Rows[e.RowIndex].Cells["columnaId"].Value));
                        //Es crea l'adjunt a la ruta dels arxius temporals
                        File.WriteAllBytes(Path.GetTempPath() + "\\" + nomFactura, adjunt);
                        //S'inicia un nou proces per visualitzar l'adjunt
                        Process.Start(Path.GetTempPath() + "\\" + nomFactura);
                    }
                }
            }
        }
        #endregion

        #region Tornar
        /*
         * Event per tornar a la pantalla principal de gestió de despeses personals
         */
        private void clickTornar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }
        #endregion
    }
}
