﻿using ClientSGDEscriptori.Accions;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.GestioDespeses
{
    public partial class GestioFullsPersonals : Form
    {
        private Int32 usuariId;
        private Int32 fullId;

        public GestioFullsPersonals(Int32 usuariId)
        {
            InitializeComponent();
            this.usuariId = usuariId;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void GestioFullsPersonalsCarregat(object sender, EventArgs e)
        {
            carregarFullObert();
        }
        #endregion

        #region Dades de full obert
        /*
         * Metode per carregar el full obert de l'usuari
         */
        private void carregarFullObert()
        {
            ServeiWeb.FullDespeses fullObert = FullDespeses.buscarFullObert(usuariId);
            if (fullObert != null)
            {
                fullId = fullObert.Id;
                botoObrirDespeses.Visible = true;
                labelOberta.Visible = true;
                textOberta.Visible = true;
                textOberta.Text = fullObert.Obert.ToString();
                labelTotal.Visible = true;
                textTotal.Visible = true;
                textTotal.Text = fullObert.Total.ToString();

                //Full sense demanar pagament
                if (fullObert.Solicitat == null)
                {
                    botoSolicitar.Visible = true;
                }
                //Full amb pagament sol·licitat
                #region Pagament sol·licitat
                else
                {
                    labelSolicitat.Visible = true;
                    textSolicitat.Visible = true;
                    textSolicitat.Text = fullObert.Solicitat.ToString();
                    labelEstat.Visible = true;
                    textEstat.Visible = true;
                    labelValidada.Visible = true;
                    textValidada.Visible = true;

                    //Pendent de validacio
                    if (fullObert.Estat == null)
                    {
                        textEstat.Text = "Pendent";
                        textValidada.Text = "Pendent";
                    }
                    //Validat
                    else
                    {
                        textValidada.Text = fullObert.Validat.ToString();
                        //Pagament denegat
                        if (!(bool)fullObert.Estat)
                        {
                            textEstat.Text = "Denegat";
                        }
                        //Pendent pagament
                        else
                        {
                            textEstat.Text = "Acceptat";
                            labelPagada.Visible = true;
                            textPagada.Visible = true;
                            textPagada.Text = "Pendent";
                        }
                    }
                }
                #endregion
            }
            else
            {
                //Si no hi ha cap full amb proces pendent, es pot obrir un full nou
                botoFullNou.Visible = true;
            }
        }
        #endregion

        #region Obrir despeses
        /*
         * Metode per obrir les despeses d'un full obret
         */
        private void clickObrirDespeses(object sender, EventArgs e)
        {
            //Variable que indica si es poden realitzar mnodificacions a les despeses del full o no
            bool modificar = false;
            if (textSolicitat.Text.Length == 0)
            {
                modificar = true;
            }
            //S'obre el formulari de gestio de despeses, indicant el full i si es poden realitzar modificacions
            using (GestioDespesesPersonals gestioDespeses = new GestioDespesesPersonals(fullId, modificar))
            {
                this.Visible = false;
                gestioDespeses.ShowDialog();
                carregarFullObert();
                this.Visible = true;
            }
        }
        #endregion

        #region Iniciar sol·licitar pagament
        /*
         * Metode per sol·licitar el pagament d'un full de despeses obert
         */
        private void clickSolicitar(object sender, EventArgs e)
        {
            //Si l'import es de 0
            if (Convert.ToDecimal(textTotal.Text) == 0)
            {
                using (Notificacio notificacio = new Notificacio("L'import és 0!!!" + Environment.NewLine + "Segur que vols continuar?"))
                {
                    //Si s'accepta es procedeix a sol·licitar la seva validacio
                    if (notificacio.ShowDialog() == DialogResult.OK)
                    {
                        //Es notifica que l'import serà de 0 i es permet la sol·licitud 
                        //Permet al responsable rebutjar el full per tancar-lo i que l'usuari pugui obrir un nou si cal
                        solicitarPagament();
                    }
                }
            }
            else
            {
                solicitarPagament();
            }
        }
        #endregion

        #region Completar sol·licitar pagament
        /*
         * Metode per actualitzar les dades del full quan es sol·licita el pagament
         */
        private void solicitarPagament()
        {
            using (Notificacio notificacio = new Notificacio("Si es sol·licita el pagament no es podran realitzar més operacions sobre el full."))
            {
                //Si s'accepta es procedeix a sol·licitar la seva validacio
                if (notificacio.ShowDialog() == DialogResult.OK)
                {
                    DateTime? dataSolicitat = FullDespeses.solicitarPagamentFull(fullId);
                    if (dataSolicitat != null)
                    {
                        botoSolicitar.Visible = false;
                        labelSolicitat.Visible = true;
                        textSolicitat.Visible = true;
                        textSolicitat.Text = dataSolicitat.ToString();
                        labelEstat.Visible = true;
                        textEstat.Visible = true;
                        textEstat.Text = "Pendent";
                    }
                }
            }
        }
        #endregion

        #region Obrir full nou
        /*
         * Metode per obrir un full de despeses nou i actualitzar les dades del formulari 
         */
        private void clickFullNou(object sender, EventArgs e)
        {
            using (Notificacio notificacio = new Notificacio("Només es pot tindre un full obert, segur que vols continuar?"))
            {
                //Si s'accepta es procedeix a obrir el nou full
                if (notificacio.ShowDialog() == DialogResult.OK)
                {
                    ServeiWeb.FullDespeses full = FullDespeses.obrirFullNou(usuariId);
                    if (full != null)
                    {
                        fullId = full.Id;
                        botoObrirDespeses.Visible = true;
                        labelOberta.Visible = true;
                        textOberta.Visible = true;
                        textOberta.Text = full.Obert.ToString();
                        labelTotal.Visible = true;
                        textTotal.Visible = true;
                        textTotal.Text = full.Total.ToString();
                        botoFullNou.Visible = false;
                        botoSolicitar.Visible = true;
                    }
                }
            }
        }
        #endregion

        #region Llista fulls tancats
        /*
         * Metode per llistar els fulls tancats de l'usuari
         */
        private void carregarLlistaFullsTancats()
        {
            List<ServeiWeb.FullDespeses> llistaTancats = FullDespeses.llistarFullsTancats(usuariId);
            if (llistaTancats != null && llistaTancats.Count > 0)
            {
                gridFullsTancats.Rows.Clear();
                for (int cont = 0; cont < llistaTancats.Count; cont++)
                {
                    string comentari = llistaTancats[cont].Comentari.ToString();
                    if (comentari.Length == 0)
                    {
                        comentari = "Sense";
                    }
                    else
                    {
                        comentari = "Veure";
                    }
                    string estat;
                    if ((bool)llistaTancats[cont].Estat)
                    {
                        estat = "Acceptat";
                    }
                    else
                    {
                        estat = "Rebutjat";
                    }
                    string[] linia = new string[] { llistaTancats[cont].Id.ToString(), llistaTancats[cont].Obert.ToString(), llistaTancats[cont].Total.ToString(), llistaTancats[cont].Solicitat.ToString(), estat, llistaTancats[cont].Validat.ToString(), llistaTancats[cont].Pagat.ToString(), comentari, "Obrir" };
                    gridFullsTancats.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Canviar pestanya
        /*
         * Metode que actualitza els llistats depenent de la pestanya seleccionada
         */
        private void canviPestanya(object sender, TabControlCancelEventArgs e)
        {
            if (e.TabPageIndex == 0)
            {
                carregarFullObert();
            }
            else if (e.TabPageIndex == 1)
            {
                carregarLlistaFullsTancats();
            }
        }
        #endregion

        #region Cel·la seleccionada
        /*
         * Metode per visualitzar els comentaris d'un full tancat,si n'hi han o obrir les despeses d'un full tancat
         */
        private void clickCela(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                //Columna del comentari
                if (e.ColumnIndex == 7 && gridFullsTancats.Rows[e.RowIndex].Cells["columnaComentari"].Value.Equals("Veure"))
                {
                    using (VisioComentari comentari = new VisioComentari("Comentari:", FullDespeses.comentariFull(Convert.ToInt32(gridFullsTancats.Rows[e.RowIndex].Cells["columnaId"].FormattedValue))))
                    {
                        comentari.ShowDialog();
                    }
                }
                //Columna de despeses
                if (e.ColumnIndex == 8)
                {
                    //VIsualitzar les despeses, enviant el id del full com a parametre
                    using (GestioDespesesTancades gestioDespeses = new GestioDespesesTancades(Convert.ToInt32(gridFullsTancats.Rows[e.RowIndex].Cells["columnaId"].FormattedValue)))
                    {
                        this.Visible = false;
                        gestioDespeses.ShowDialog();
                        carregarFullObert();
                        this.Visible = true;
                    }
                }
            }
        }
        #endregion

        #region Tornar
        private void clickTornar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }
        #endregion
    }
}
