﻿using ClientSGDEscriptori.Accions;
using ClientSGDEscriptori.GestioDespeses;
using ClientSGDEscriptori.Administrar;
using ClientSGDEscriptori.ValidacioDespeses;
using System;
using System.Windows.Forms;
using ClientSGDEscriptori.Resources;

namespace ClientSGDEscriptori
{
    public partial class MenuPrincipal : Form
    {
        private Usuari usuari;

        public MenuPrincipal()
        {
            InitializeComponent();
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void menuCarregat(object sender, EventArgs e)
        {
            if (!ConnexioServei.verificarConnexioInternet())
            {
                MessageBox.Show(Missatges.INTERNET_ERROR + Environment.NewLine + Missatges.TANCAR);
                this.Close();
            }
            else
            {
                if (!ConnexioServei.verificarConnexioServeiWeb())
                {
                    MessageBox.Show(Missatges.SERVEI_ERROR);
                }
                else
                {
                    iniciarUsuariPrincipal();
                }
            }
        }
        #endregion

        #region Carregar usuari logat
        /*
         * Metode que inicia la validacio i les dades de l'usuari que vol accedir al sistema
         */
        private void iniciarUsuariPrincipal()
        {
            try
            {
                string resultat = Usuari.validarUsuari();
                if (resultat.Equals(string.Empty))
                {
                    this.Close();
                }
                usuari = Usuari.buscarUsuariPerDni(resultat);
                iniciarCarrec();
                iniciarPermisos();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Carregar carrec de l'usuari logat
        /*
         * Metode per buscar el carrec de l'usuari del sistema
         */
        private void iniciarCarrec()
        {
            usuari.Carrec = Carrec.buscarCarrecPerId(usuari.CarrecId);
        }
        #endregion

        #region Carregar permisos de l'usuari logat
        /*
         * Metode per buscar els permisos de l'usuari i preparar el menu principal depenent dels permisos
         */
        private void iniciarPermisos()
        {
            botoAdministrar.Enabled = false;
            botoValidarDespeses.Enabled = false;

            //Si l'usuari te permisos
            if (usuari.Carrec.PermisosId != 0)
            {
                usuari.Carrec.Permisos = Permisos.buscarPermisosPerId((Int32)usuari.Carrec.PermisosId);
                //Si pot validar
                if (usuari.Carrec.Permisos.Validar)
                {
                    botoValidarDespeses.Enabled = true;
                }
                //Si es administrador o pot veure usuaris
                if (usuari.Carrec.Permisos.Administrar || (usuari.Carrec.Permisos.VeureUsuaris != null && (bool)usuari.Carrec.Permisos.VeureUsuaris))
                {
                    botoAdministrar.Enabled = true;
                }
                //Si l'usuari pot realitzar pagament de despeses
                if (usuari.Carrec.Permisos.VeureDespeses != null && (bool)usuari.Carrec.Permisos.VeureDespeses)
                {
                    botoPagaments.Visible = true;
                }
            }
            else
            {
                botoAdministrar.Enabled = false;
                botoValidarDespeses.Enabled = false;
            }
            botoGestioDespeses.Enabled = true;
            botoCalendari.Enabled = true;
        }
        #endregion

        #region Gestio de despeses personals
        /*
         * Event per accedir a la gestio de despeses personals
         */
        private void clickGestioDespeses(object sender, EventArgs e)
        {
            this.Visible = false;
            using (GestioFullsPersonals fullPersonal = new GestioFullsPersonals(usuari.IdUsuari))
            {
                fullPersonal.ShowDialog();
                assegurarConnexioServei();
            }
            this.Visible = true;
        }
        #endregion

        #region Validar despeses
        /*
         * Event per accedir a la validacio de despeses
         */
        private void clickValidarDespeses(object sender, EventArgs e)
        {
            this.Visible = false;
            using (ValidarFullsDespeses validarFulls = new ValidarFullsDespeses(usuari.IdUsuari))
            {
                validarFulls.ShowDialog();
                assegurarConnexioServei();
            }
            this.Visible = true;
        }
        #endregion

        #region Administrar
        /*
         * Event per accedir a l'administracio
         */
        private void clickAdministracio(object sender, EventArgs e)
        {
            try
            {
                this.Visible = false;
                using (Administracio administrar = new Administracio(usuari.Carrec.Permisos.Administrar))
                {
                    administrar.ShowDialog();
                    assegurarConnexioServei();
                }
                this.Visible = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Calendari
        /*
         * Event per accedir al calendari
         */
        private void clickCalendari(object sender, EventArgs e)
        {
            this.Visible = false;
            bool permisos = false;
            if (usuari.Carrec.PermisosId != 0 && usuari.Carrec.Permisos.Validar)
            {
                permisos = true;
            }
            using (Calendari calendari = new Calendari(permisos))
            {
                calendari.ShowDialog();
                assegurarConnexioServei();
            }
            this.Visible = true;
        }
        #endregion

        #region Configuracio
        /*
         * Event per accedir a la configuracio
         */
        private void clickConfiguracio(object sender, EventArgs e)
        {
            try
            {
                this.Visible = false;
                using (ConfiguracioServei configuracio = new ConfiguracioServei())
                {
                    configuracio.ShowDialog();
                    assegurarConnexioServei();
                }
                this.Visible = true;


            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        #endregion

        #region Pagar despeses
        /*
         * Event per accedir al pagament de despeses
         */
        private void clickPagarDespeses(object sender, EventArgs e)
        {
            this.Visible = false;
            using (PagarDespeses pagar = new PagarDespeses())
            {
                pagar.ShowDialog();
                assegurarConnexioServei();
            }
            this.Visible = true;
        }
        #endregion

        #region Desactivar menu principal
        /*
         * Metode per desactivar els accesos del menu principal
         */
        private void desactivarMenu()
        {
            botoAdministrar.Enabled = false;
            botoGestioDespeses.Enabled = false;
            botoCalendari.Enabled = false;
            botoValidarDespeses.Enabled = false;
            botoPagaments.Visible = false;
        }
        #endregion

        #region Revisar connexio a servei web
        /*
         * Metode per verificar la connexio amb el servei web
         */
        private void assegurarConnexioServei()
        {
            if (!ConnexioServei.verificarConnexioServeiWeb())
            {
                desactivarMenu();
                MessageBox.Show(Missatges.SERVEI_ERROR);
            }
            else
            {
                if (usuari == null)
                {
                    iniciarUsuariPrincipal();
                }
                iniciarPermisos();
            }
        }
        #endregion

        #region Sortir de l'aplicacio
        /*
         * Event per sortir de l'aplicacio
         */
        private void clickSortir(object sender, EventArgs e)
        {
            this.Close();
        }
        #endregion
    }
}
