﻿using ClientSGDEscriptori.Accions;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace ClientSGDEscriptori.ValidacioDespeses
{
    public partial class ValidarDespeses : Form
    {
        private Int32 fullId;

        public ValidarDespeses(Int32 fullId)
        {
            InitializeComponent();
            this.fullId = fullId;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void validarDespesesCarregat(object sender, EventArgs e)
        {
            carregarDespeses();
        }
        #endregion

        #region Llista de despeses
        /*
         * Metode per accedir a les despeses d'un full 
         */
        private void carregarDespeses()
        {
            List<ServeiWeb.Despesa> llista = Despesa.despesesDeFull(fullId);
            if (llista != null && llista.Count > 0)
            {
                gridDespeses.Rows.Clear();
                for (int cont = 0; cont < llista.Count; cont++)
                {
                    //S'indica si hi han comentaris
                    string comentari = "Veure";
                    if (llista[cont].Comentari.Length == 0)
                    {
                        comentari = "Sense";
                    }
                    //S'indica si hi ha adjunt
                    string hiHaAdjunt = "Obrir";
                    if (!llista[cont].Adjunt)
                    {
                        hiHaAdjunt = "Sense";
                    }
                    string acceptat = "Acceptada";
                    if (llista[cont].Acceptada != null && (bool)!llista[cont].Acceptada)
                    {
                        acceptat = "Denegada";
                    }
                    string[] linia = new string[] { llista[cont].Id.ToString(), null, llista[cont].Data.ToString("dd/MM/yyyy"), llista[cont].Import.ToString(), llista[cont].NomTipus, hiHaAdjunt, comentari, acceptat };
                    gridDespeses.Rows.Add(linia);
                    if (Festiu.verificarFestiu(llista[cont].Data))
                    {
                        gridDespeses.Rows[cont].DefaultCellStyle.BackColor = Color.Aquamarine;
                    }
                }
            }
        }
        #endregion

        #region Denegar despeses
        /*
         * Event per denegar el pagament de les despeses seleccionades
         */
        private void clickBotoDenegar(object sender, EventArgs e)
        {
            List<Int32> llista = despesesSelecionades();
            if (llista != null && llista.Count > 0)
            {
                if (Despesa.denegarDespeses(llista))
                {
                    carregarDespeses();
                }
            }
        }
        #endregion

        #region Validar despeses
        /*
         * Event per autoritzar el pagament d'una despesa.
         */
        private void clickBotoValidar(object sender, EventArgs e)
        {
            List<Int32> llista = despesesSelecionades();
            if (llista != null && llista.Count > 0)
            {
                if (Despesa.acceptarDespeses(llista))
                {
                    carregarDespeses();
                }
            }
        }
        #endregion

        #region Seleccionar despeses
        /*
         * Metode que retorna les despeses seleccionades
         */
        private List<Int32> despesesSelecionades()
        {
            List<Int32> files = new List<Int32>();
            for (int cont = 0; cont < gridDespeses.Rows.Count; cont++)
            {
                if ((bool)gridDespeses.Rows[cont].Cells[1].FormattedValue)
                {
                    Int32 fila = Convert.ToInt32(gridDespeses.Rows[cont].Cells[0].Value);
                    files.Add(fila);
                }
            }
            return files;
        }
        #endregion

        #region Cel·la seleccionada
        /*
         * Event que detecta la seleccio de comentaris o adjunts d'una despesa 
         */
        private void clickCela(object sender, DataGridViewCellEventArgs e)
        {
            if(e.RowIndex>=0){
                //Si es selecciona la columna de comentari, s'obre el comentari de la despesa d'aquella fila
                if (e.ColumnIndex == 6 && gridDespeses.Rows[e.RowIndex].Cells["columnaComentari"].Value.Equals("Veure"))
                {
                    using (VisioComentari comentari = new VisioComentari("Comentari", Despesa.comentarisDespesa(Convert.ToInt32(gridDespeses.Rows[e.RowIndex].Cells["columnaId"].Value))))
                    {
                        comentari.ShowDialog();
                    }
                }
                //Si es selecciona la columna d'adjunts i existeix adjunt,s'obre d'adjunt d'aquella despesa
                if (e.ColumnIndex == 5 && gridDespeses.Rows[e.RowIndex].Cells["columnaAdjunt"].Value.Equals("Obrir"))
                {
                    //Obte l' adjunt de la BBDD
                    byte[] adjunt = Factura.buscarFacturaAdjunta(Convert.ToInt32(gridDespeses.Rows[e.RowIndex].Cells["columnaId"].Value));
                    if (adjunt != null)
                    {
                        string nomFactura = Factura.buscarNomFactura(Convert.ToInt32(gridDespeses.Rows[e.RowIndex].Cells["columnaId"].Value));
                        //Escriu el fitxer
                        File.WriteAllBytes(Path.GetTempPath() + "\\" + nomFactura, adjunt);
                        //Crea un nou procés per obrir el fitxer
                        Process.Start(Path.GetTempPath() + "\\" + nomFactura);
                    }
                }
            }

        }
        #endregion

        #region Tornar
        /*
         * Event per tornar a la pantalla principal per validar fulls pendents
         */
        private void clickTornar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }
        #endregion
    }
}