﻿using ClientSGDEscriptori.Accions;
using ClientSGDEscriptori.Resources;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ClientSGDEscriptori.ValidacioDespeses
{
    public partial class ValidarFullsDespeses : Form
    {
        private Int32 responsableId;

        public ValidarFullsDespeses(Int32 responsableId)
        {
            InitializeComponent();
            this.responsableId = responsableId;
        }

        #region Inici formulari
        /*
         * Event inicial del formulari
         */
        private void validarFullsCarregat(object sender, EventArgs e)
        {
            carregarFullsPendents();
        }
        #endregion

        #region Llista fulls pendents
        /*
         * Metode que carrega els fulls pendents de validar d'un responsable
         */
        private void carregarFullsPendents()
        {
            gridFullsPendents.Rows.Clear();
            List<ServeiWeb.FullDespeses> llista = FullDespeses.buscarFullsPendents(responsableId);
            //Si hi han fulls pendents es mostren al grid 
            if (llista != null && llista.Count > 0)
            {
                for (int cont = 0; cont < llista.Count; cont++)
                {
                    string[] linia = new string[] { llista[cont].Id.ToString(), llista[cont].Usuari_Id.ToString(), null, llista[cont].Obert.ToString(), llista[cont].Total.ToString(), llista[cont].Solicitat.ToString(), "Obrir" };
                    gridFullsPendents.Rows.Add(linia);
                }
                //S'habiliten els botons per rebutjar o validar fulls de despeses
                botoRebutjar.Visible = true;
                botoValidar.Visible = true;
            }
            //Si la llista no te fulls es deshabiliten els botons per fer modificacions
            else
            {
                botoRebutjar.Visible = false;
                botoValidar.Visible = false;
            }
        }
        #endregion

        #region Validar full
        /*
         * Event per acceptar el pagament d'un full de despeses
         */
        private void clickValidar(object sender, EventArgs e)
        {
            List<Int32> llista = fullsSelecionats();
            //Si no es selecciona cap full no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si es selecciona un full, es procedeix a validar-lo
            else if (llista.Count == 1)
            {
                if (FullDespeses.acceptarFullDespeses(llista[0]))
                {
                    carregarFullsPendents();
                }
            }
            //No es pot validar mes d'un full a l'hora
            else
            {
                MessageBox.Show(Missatges.UN_FULL);
            }
        }
        #endregion

        #region Llista treballadors
        /*
         * Metode que carrega el llistat de treballadors d'un responsable
         */
        private void carregarTreballadorsPropis()
        {
            gridTreballadors.Rows.Clear();
            List<ServeiWeb.Usuari> llistaUsuaris = Usuari.usuarisDeResponsable(responsableId);
            if (llistaUsuaris.Count > 0)
            {
                for (int cont = 0; cont < llistaUsuaris.Count; cont++)
                {
                    string[] linia = new string[] { llistaUsuaris[cont].Nom, llistaUsuaris[cont].Cognoms, llistaUsuaris[cont].NomCarrec };
                    gridTreballadors.Rows.Add(linia);
                }
            }
        }
        #endregion

        #region Rebutjar pagament de full
        /*
         * Event per rebutjar el pagament d' un full de despeses
         */
        private void clickRebutjar(object sender, EventArgs e)
        {
            List<Int32> llista = fullsSelecionats();
            //Si no selecciona cap full no es fa res
            if (llista.Count == 0)
            {
                //No fer res
            }
            //Si s'ha seleccionat un full, cal afegir un comentari
            else if (llista.Count == 1)
            {
                using (ComentariDenegarFull comentari = new ComentariDenegarFull())
                {
                    //Si s'omple el comentari es procedeix a denegar el pagament del full, quedant tancat
                    if (comentari.ShowDialog() == DialogResult.OK)
                    {
                        if (FullDespeses.rebutjarFullDespeses(llista[0], comentari.comentari))
                        {
                            carregarFullsPendents();
                        }
                    }
                }
            }
            //Nomes es pot rebutjar un full de despeses a l'hora
            else
            {
                MessageBox.Show(Missatges.UN_FULL);
            }
        }
        #endregion

        #region Cel·la seleccionada
        /*
         * Event que mostra l' usuari propietari del full de despeses quan es selecciona una fila.
         * Si coincideix amb la columna de comentari
         */
        private void clickCela(object sender, DataGridViewCellEventArgs e)
        {
            if(e.RowIndex>=0)
            {
                Usuari usuari = Usuari.buscarUsuariPerId(Convert.ToInt32(gridFullsPendents.Rows[e.RowIndex].Cells["usuariId"].Value));
                labelNom.Text = "Nom: " + usuari.NomUsuari;
                labelCognoms.Text = "Cognoms: " + usuari.CognomsUsuari;
                labelCarrec.Text = "Càrrec: " + Carrec.buscarCarrecPerId(usuari.CarrecId).CarrecNom;
                //Si es selecciona la cela de despeses,es procedeix a obrir el formulari per validar-les o rebutjar-les
                if (e.ColumnIndex == 6)
                {
                    this.Visible = false;
                    using (ValidarDespeses validarDespeses = new ValidarDespeses(Convert.ToInt32(gridFullsPendents.Rows[e.RowIndex].Cells["columnaId"].FormattedValue)))
                    {
                        validarDespeses.ShowDialog();
                        carregarFullsPendents();
                        panellTreballador.Visible = false;
                    }
                    this.Visible = true;
                }
                else
                {
                    panellTreballador.Visible = true;
                }
            }

        }
        #endregion

        #region Fulls seleccionats
        /*
         * Metode que retorna un llistat amb els id dels fulls de despeses seleccionats 
         */
        private List<Int32> fullsSelecionats()
        {
            List<Int32> files = new List<Int32>();
            for (int cont = 0; cont < gridFullsPendents.Rows.Count; cont++)
            {
                if ((bool)gridFullsPendents.Rows[cont].Cells["columnaSeleccionar"].FormattedValue)
                {
                    Int32 fila = Convert.ToInt32(gridFullsPendents.Rows[cont].Cells[0].Value);
                    files.Add(fila);
                }
            }
            return files;
        }
        #endregion

        #region Canviar pestanya
        /*
         * Event que actualitza els llistats, depenent de la pestanya seleccionada
         */
        private void canviPestanya(object sender, TabControlCancelEventArgs e)
        {
            if (e.TabPageIndex == 1)
            {
                panellTreballador.Visible = false;
                carregarTreballadorsPropis();
            }
            else if (e.TabPageIndex == 0)
            {
                carregarFullsPendents();
            }
        }
        #endregion

        #region Tornar
        /*
         * Event per tornar al menú principal
         */
        private void clickTornar(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }
        #endregion
    }
}
