﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class CarrecAccions
    {
        #region Buscar carrec per Id
        /*
         * Metode que retorna un carrec, donat el seu id 
         */
        public static Carrec buscarCarrecPerId(Int32 id)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarCarrecPerId", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@carrecId", id);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            Carrec carrec = new Carrec();
                            carrec.Id = (Int32)lector["id"];
                            carrec.Nom = lector["nom"].ToString();
                            if (lector["permisos_id"] != null && !Convert.IsDBNull(lector["permisos_id"]))
                            {
                                carrec.IdPermisos = lector["permisos_id"] as Int32?;
                            }
                            carrec.Habilitat = (bool)lector["habilitat"];
                            return carrec;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Verificar estat carrec
        /*
         * Metode que retorna si un carrrec esta habilitat o no, donat el seu id
         */
        public static bool verificarEstatCarrec(Int32 carrecId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("verificarCarrecHabilitat", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@id", carrecId);
                    Int32 estat = (Int32)comanda.ExecuteScalar();
                    if (estat == 1)
                    {
                        return true;
                    }
                    return false;
                }
            }
        }
        #endregion

        #region Habilitarcarrec
        /*
         * Metode que habilita un carrec,donat el seu id
         */
        public static void habilitarCarrec(Int32 carrecId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("habilitarCarrec", connexio))
                        {
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Transaction = transaccio;
                            comanda.Parameters.AddWithValue("@id", carrecId);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                        }
                        transaccio.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Deshabilitarcarrec
        /*
         * Metode que deshabilita un carrec,donat el seu id
         */
        public static void deshabilitarCarrec(Int32 carrecId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("deshabilitarCarrec", connexio))
                        {
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Transaction = transaccio;
                            comanda.Parameters.AddWithValue("@id", carrecId);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                        }
                        transaccio.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Buscar carrec per nom
        /*
         * Metode que retorna un carrec, donat el seu nom
         */
        public static Carrec buscarCarrecPerNom(string nom)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarCarrecPerNom", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@nom", nom);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            Carrec carrec = new Carrec();
                            carrec.Id = (Int32)lector["id"];
                            carrec.Nom = nom;
                            if (lector["permisos_id"] != null && !Convert.IsDBNull(lector["permisos_id"]))
                            {
                                carrec.IdPermisos = (Int32?)lector["permisos_id"];
                            }
                            carrec.Habilitat = (bool)lector["habilitat"];
                            //carrec.Actualitzacio = (DateTime)lector["actualitzacio"];
                            return carrec;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Modificar carrec
        /*
         * Metode que permet modificar un carrec existent, donat el seu id,nom i permisos nous
         */
        public static void modificarCarrec(Int32 id, string nom, int? permisos)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("modificarCarrecExistent", connexio))
                        {
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Transaction = transaccio;
                            comanda.Parameters.AddWithValue("@id", id);
                            comanda.Parameters.AddWithValue("@nom", nom);
                            if (permisos == null || permisos == 0)
                            {
                                comanda.Parameters.AddWithValue("@permisosId", DBNull.Value);
                            }
                            else if (permisos > 0)
                            {
                                comanda.Parameters.AddWithValue("@permisosId", permisos);
                            }
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                        }
                        transaccio.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Afegir carrec
        /*
         * Metode que permet afegir un nou carrec, donat el seu id,nom i permisos
         */
        public static Carrec afegirCarrecNou(string nom, int? permisos)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("afegirCarrecNou", connexio))
                        {
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Transaction = transaccio;
                            Carrec carrecNou = new Carrec();
                            carrecNou.Nom = nom;
                            comanda.Parameters.AddWithValue("@nom", carrecNou.Nom);
                            if (permisos == null || permisos == 0)
                            {
                                carrecNou.IdPermisos = 0;
                                comanda.Parameters.AddWithValue("@permisosId", DBNull.Value);
                            }
                            else if (permisos > 0)
                            {
                                carrecNou.IdPermisos = permisos;
                                comanda.Parameters.AddWithValue("@permisosId", carrecNou.IdPermisos);
                            }
                            //Per defecte els carrecs nous estan habilitats
                            carrecNou.Habilitat = true;
                            comanda.Parameters.AddWithValue("@habilitat", carrecNou.Habilitat);
                            carrecNou.Actualitzacio = DateTime.Now;
                            comanda.Parameters.AddWithValue("@actualitzacio", carrecNou.Actualitzacio);
                            carrecNou.Id = (Int32)comanda.ExecuteScalar();
                            transaccio.Commit();
                            return carrecNou;
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Llista de carrecs habilitats
        /*
         * Metode uque retorna el llistat de carrecs habilitats
         */
        public static List<Carrec> llistaCarrecsDisponibles()
        {
            List<Carrec> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistaDeCarrecsHabilitats", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Carrec>();
                        while (lector.Read())
                        {
                            Carrec carrec = new Carrec();
                            carrec.Id = (Int32)lector["id"];
                            carrec.Nom = lector["nom"].ToString();
                            carrec.IdPermisos = lector["idPermis"] as Int32?;
                            carrec.Habilitat = (bool)lector["habilitat"];
                            carrec.NomPermisos = lector["nomPermisos"].ToString();
                            llista.Add(carrec);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion

        #region Llista de carrecs existents
        /*
         * Metode que retorna el llistat de tots els carrecs existents
         */
        public static List<Carrec> llistaDeCarrecs()
        {
            List<Carrec> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistaDeCarrecs", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Carrec>();
                        while (lector.Read())
                        {
                            Carrec carrec = new Carrec();
                            carrec.Id = (Int32)lector["id"];
                            carrec.Nom = lector["nom"].ToString();
                            carrec.IdPermisos = lector["idPermis"] as Int32?;
                            carrec.Habilitat = (bool)lector["habilitat"];
                            carrec.NomPermisos = lector["nomPermisos"].ToString();
                            llista.Add(carrec);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion
    }
}