﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class DespesesAccions
    {
        #region Verificar si despesa te adjunt
        /*
         * Metode que indica si una despesa te adjunt, donat el id de la despesa
         */
        public static bool teAdjunt(Int32 idDespesa)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("despesaConteAdjunt", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@id", idDespesa);
                    Int32 teAdjunt = (Int32)comanda.ExecuteScalar();
                    if (teAdjunt == 1)
                    {
                        return true;
                    }
                }
            }
            return false;
        }
        #endregion

        #region Buscar comentari de despesa
        /*
         * Metode que retorna el comentari d'una despesa,donat el seu id
         */
        public static string comentariDeDespesa(Int32 despesaId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarComentariDeDespesa", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@despesaId", despesaId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            string comentari = lector["comentari"].ToString();
                            return comentari;
                        }
                    }
                }
            }
            return string.Empty;
        }
        #endregion

        #region Modificar despesa
        /*
         * Metode que permet modificar les dades d'una despesa inclos l'adjunt, donat el id de la despesa
         */
        public static void modificarDespesa(Int32 despesaId, DateTime data, decimal import, Int32 tipusId, string comentari, Int32 facturaId, string nomFactura, byte[] adjunt)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        DateTime actualitzacio;
                        using (SqlCommand comanda = new SqlCommand("modificarDespesa ", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Parameters.AddWithValue("@id", despesaId);
                            comanda.Parameters.AddWithValue("@data", data);
                            comanda.Parameters.AddWithValue("@import", import);
                            comanda.Parameters.AddWithValue("@tipusId", tipusId);
                            //No hi havia i no s´afegeix adjunt o hi havia adjunt i s´indica a la despesa que s´elimina
                            if (facturaId == 0 && adjunt.Length == 0 || (facturaId > 0 && adjunt.Length == 0 && nomFactura.Length == 0))
                            {
                                comanda.Parameters.AddWithValue("@adjunt", false);
                            }
                            //Si hi havia adjunt existent o hi haura un nou, s'indica a la despesa que te adjunt
                            else
                            {
                                comanda.Parameters.AddWithValue("@adjunt", true);
                            }
                            comanda.Parameters.AddWithValue("@comentari", comentari);
                            actualitzacio = DateTime.Now;
                            comanda.Parameters.AddWithValue("@actualitzacio", actualitzacio);
                            comanda.ExecuteNonQuery();
                        }

                        //Hi havia adjunt i no es modifica
                        if (facturaId > 0 && adjunt.Length == 0 && nomFactura.Length > 0)
                        {
                            //Res
                        }
                        //Hi havia adjunt i es modifica
                        else if (facturaId > 0 && adjunt.Length > 0 && nomFactura.Length > 0)
                        {
                            FacturesAccions.modificarAdjunt(transaccio, connexio, facturaId, nomFactura, adjunt, actualitzacio);
                        }
                        //Hi havia adjunt i s'elimina
                        else if (facturaId > 0 && adjunt.Length == 0 && nomFactura.Length == 0)
                        {
                            FacturesAccions.eliminarAdjunt(transaccio, connexio, facturaId);
                        }
                        //No hi havia adjunt i s'afegeix un nou
                        else if (facturaId == 0 && adjunt.Length > 0)
                        {
                            FacturesAccions.afegirAdjunt(transaccio, connexio, despesaId, nomFactura, adjunt, actualitzacio);
                        }
                        transaccio.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Buscar despesa
        /*
         * Metode que retorna una despesa donat el seu id
         */
        public static Despesa buscarDespesaPerId(Int32 despesaId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarDespesa", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@idDespesa", despesaId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            Despesa despesa = new Despesa();
                            despesa.Id = despesaId;
                            //despesa.Full_despesa = (Int32)lector["full_despesa"];
                            despesa.Data = (DateTime)lector["data"];
                            despesa.Import = (decimal)lector["import"];
                            despesa.Tipus = (Int32) lector["idTipus"];
                            despesa.NomTipus = lector["nomTipus"].ToString();
                            despesa.Adjunt = (bool)lector["adjunt"];
                            despesa.Comentari = lector["comentari"].ToString();
                            return despesa;
                        }
                        return null;
                    }
                }
            }
        }
        #endregion

        #region Denegar pagament de despesa
        /*
         * Metode que nega el pagament de despesses, donada una llista amb els id de les despeses
         */
        public static void denegarDespeses(List<Int32> despeses)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("denegarDespesa", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            for (int cont = 0; cont < despeses.Count; cont++)
                            {
                                comanda.Parameters.Clear();
                                comanda.Parameters.AddWithValue("@idDespesa", despeses[cont]);
                                comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                                comanda.ExecuteNonQuery();
                            }
                            transaccio.Commit();
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Acceptar pagament de despesa
        /*
         * Metode que autoritza el pagament de despesses, donada una llista amb els id de les despeses
         */
        public static void acceptarDespeses(List<Int32> despeses)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("acceptarDespesa", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            for (int cont = 0; cont < despeses.Count; cont++)
                            {
                                comanda.Parameters.Clear();
                                comanda.Parameters.AddWithValue("@idDespesa", despeses[cont]);
                                comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                                comanda.ExecuteNonQuery();
                            }
                        }
                        transaccio.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Eliminar despeses
        /*
         * Metode que permet eliminar despeses, donada una llista amb els id de les desepses
         */
        public static void eliminarDespeses(List<Int32> id, List<bool> despesesAdjunt)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();

                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        for (int cont = 0; cont < despesesAdjunt.Count; cont++)
                        {
                            if (despesesAdjunt[cont])
                            {
                                FacturesAccions.eliminarAdjunt(transaccio, connexio, id[cont]);
                            }
                        }

                        using (SqlCommand comanda = new SqlCommand("eliminarDespesa", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;

                            for (int cont = 0; cont < id.Count; cont++)
                            {
                                comanda.Parameters.Clear();
                                comanda.Parameters.AddWithValue("@id", id[cont]);
                                comanda.ExecuteNonQuery();
                            }
                            transaccio.Commit();
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Buscar despeses de full
        /*
         * Metode que retorna un llistat de les despeses d'un full donat el seu id
         */
        public static List<Despesa> buscarDespesesDeFull(Int32 fullId)
        {
            List<Despesa> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarDespesesDeFull", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@fullId", fullId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Despesa>();
                        while (lector.Read())
                        {
                            Despesa despesa = new Despesa();
                            despesa.Id = (Int32)lector["id"];
                            despesa.Full_despesa = fullId;
                            despesa.Data = (DateTime)lector["data"];
                            despesa.Import = (decimal)lector["import"];
                            despesa.NomTipus = lector["nomTipus"].ToString();
                            despesa.Adjunt = (bool)lector["adjunt"];
                            despesa.Comentari = lector["comentari"].ToString();
                            despesa.Acceptada = lector["acceptada"] as bool?;
                            llista.Add(despesa);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion

        #region Afegir despesa
        /*
         * Metode que permet afegir una nova despesa donades les seves dades, i afegir un adjunt si hi ha
         */
        public static Despesa afegirDespesaNova(Int32 fullId, DateTime data, decimal import, Int32 tipusId, string comentari, string nomArxiu, byte[] adjunt)
        {
            Despesa despesaNova = new Despesa();
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        //Afegir la despesa
                        using (SqlCommand comanda = new SqlCommand("crearDespesaNova", connexio))
                        {
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Transaction = transaccio;
                            despesaNova.Full_despesa = fullId;
                            comanda.Parameters.AddWithValue("@fullId", despesaNova.Full_despesa);
                            despesaNova.Data = data;
                            comanda.Parameters.AddWithValue("@data", despesaNova.Data);
                            despesaNova.Import = import;
                            comanda.Parameters.AddWithValue("@import", despesaNova.Import);
                            despesaNova.Tipus = tipusId;
                            comanda.Parameters.AddWithValue("@tipusId", despesaNova.Tipus);
                            despesaNova.Comentari = comentari;
                            comanda.Parameters.AddWithValue("@comentari", despesaNova.Comentari);
                            despesaNova.Actualitzacio = DateTime.Now;
                            comanda.Parameters.AddWithValue("@actualitzacio", despesaNova.Actualitzacio);
                            if (adjunt.Length == 0)
                            {
                                despesaNova.Adjunt = false;
                            }
                            else
                            {
                                despesaNova.Adjunt = true;
                            }
                            comanda.Parameters.AddWithValue("@adjunt", despesaNova.Adjunt);
                            despesaNova.Id = (Int32)comanda.ExecuteScalar();
                        }

                        //Si hi ha adjunt,afegir adjunt
                        if (despesaNova.Adjunt)
                        {
                            FacturesAccions.afegirAdjunt(transaccio, connexio, despesaNova.Id, nomArxiu, adjunt, despesaNova.Actualitzacio);
                        }
                        transaccio.Commit();
                        return despesaNova;
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion
    }
}