﻿using System;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class FacturesAccions
    {
        #region Afegir adjunt
        /*
         * Metode que afegeix un nou adjunt a una despesa
         */
        public static void afegirAdjunt(SqlTransaction transaccio, SqlConnection connexio, Int32 despesaId, string nomArxiu, byte[] adjunt, DateTime actualitzacio)
        {
            using (SqlCommand comanda = new SqlCommand("afegirNouAdjunt", connexio))
            {
                comanda.CommandType = CommandType.StoredProcedure;
                comanda.Transaction = transaccio;
                comanda.Parameters.AddWithValue("@despesaId", despesaId);
                comanda.Parameters.AddWithValue("@nomArxiu", nomArxiu);
                comanda.Parameters.AddWithValue("@adjunt", adjunt);
                comanda.Parameters.AddWithValue("@actualitzacio", actualitzacio);
                comanda.ExecuteNonQuery();
            }
        }
        #endregion

        #region Modificar adjunt
        /*
         * Metode que modifica l'adjunt d'una despesa
         */
        public static void modificarAdjunt(SqlTransaction transaccio, SqlConnection connexio, Int32 facturaId, string nomArxiu, byte[] adjunt, DateTime actualitzacio)
        {
            using (SqlCommand comanda = new SqlCommand("modificarAdjunt", connexio))
            {
                comanda.CommandType = CommandType.StoredProcedure;
                comanda.Transaction = transaccio;
                comanda.Parameters.AddWithValue("@idFactura", facturaId);
                comanda.Parameters.AddWithValue("@nomArxiu", nomArxiu);
                comanda.Parameters.AddWithValue("@adjunt", adjunt);
                comanda.Parameters.AddWithValue("@actualitzacio", actualitzacio);
                comanda.ExecuteNonQuery();
            }
        }
        #endregion

        #region Eliminar adjunt
        /*
         * Metode que elimina l'adjunt d'una despesa
         */
        public static void eliminarAdjunt(SqlTransaction transaccio, SqlConnection connexio, Int32 despesaId)
        {
            using (SqlCommand comanda = new SqlCommand("eliminarAdjunt", connexio))
            {
                comanda.CommandType = CommandType.StoredProcedure;
                comanda.Transaction = transaccio;
                comanda.Parameters.AddWithValue("@despesaId", despesaId);
                comanda.ExecuteNonQuery();
            }
        }
        #endregion

        #region Buscar nom d'adjunt
        /*
         * Metode que retorna el nom d'un adjunt, donat el seu id
         */
        public static string nomAdjunt(Int32 despesaId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarNomAdjunt", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@despesaId", despesaId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            return lector["nom_arxiu"].ToString();
                        }
                        return string.Empty;
                    }
                }
            }
        }
        #endregion

        #region Buscar id de factura
        /*
         * Metode que retorna el id d'una factura, donat el id de la despesa
         */
        public static Int32 idFacturaAmbIdDespesa(Int32 despesaId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarIdDeFacturaPerIdDespesa", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@despesaId", despesaId);
                    int? idFactura = comanda.ExecuteScalar() as int?;
                    if(idFactura!=null){
                        return (Int32)idFactura;
                    }
                    return 0;
                }
            }
        }
        #endregion

        #region Buscar adjunt
        /*
         * Metode que retorna l'adjunt d'una despesa 
         */
        public static byte[] obtenirAdjunt(Int32 despesaId) { 
            using(SqlConnection connexio = ConnexioSQL.Connectar()){
                connexio.Open();
                using(SqlCommand comanda = new SqlCommand("recuperarAdjunt",connexio)){
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@despesaId",despesaId);
                    using(SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if(lector.Read()){
                            byte[] adjunt =(byte[]) lector["adjunt"];
                            return adjunt;
                        }
                    }
                }
            }
            return null;
        }
        #endregion
    }
}