﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class FestiusAccions
    {
        #region Llista de festius
        /*
         * Metode que retorna el llistat de les dates festives existents
         */
        public static List<DateTime> llistarDatesFestives()
        {
            List<DateTime> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("datesFestives", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<DateTime>();
                        while (lector.Read())
                        {
                            DateTime data = (DateTime)lector["data"];
                            llista.Add(data);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion

        #region Afegir festiu
        /*
         * Metode que  afegeix un nou festiu
         */
        public static Festiu afegirFestiu(DateTime data,string descripcio)
        {
            using(SqlConnection connexio = ConnexioSQL.Connectar()){
                connexio.Open();
                using(SqlTransaction transaccio = connexio.BeginTransaction()){
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("afegirFestiuNou",connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            Festiu festiu = new Festiu();
                            festiu.Data = data;
                            comanda.Parameters.AddWithValue("@data",festiu.Data);
                            festiu.Descripcio = descripcio;
                            comanda.Parameters.AddWithValue("@descripcio", festiu.Descripcio);
                            festiu.Actualitzacio = DateTime.Now;
                            comanda.Parameters.AddWithValue("@actualitzacio",festiu.Actualitzacio);
                            comanda.ExecuteNonQuery();
                            transaccio.Commit();
                            return festiu;
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch
                        { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Eliminar festiu
        /*
         * Metode per eliminar un festiu
         */
        public static void eliminarFestiu(DateTime data)
        {
            using(SqlConnection connexio = ConnexioSQL.Connectar()){
                connexio.Open();
                using(SqlTransaction transaccio = connexio.BeginTransaction()){
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("eliminarFestiu", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Parameters.AddWithValue("@data",data);
                            comanda.ExecuteNonQuery();
                            transaccio.Commit();
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch
                        {
                            throw;
                        }
                    }
                }
            }
        }
        #endregion

        #region Llista de dates festives
        /*
         * Metode que retorna les dates dels dies festius existents
         */
        public static List<Festiu> llistaDeDiesFestius()
        {
            List<Festiu> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistaDeFestius", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Festiu>();
                        while (lector.Read())
                        {
                            Festiu dia = new Festiu();
                            dia.Data = (DateTime)lector["data"];
                            dia.Descripcio = lector["descripcio"].ToString();
                           // dia.Actualitzacio = (DateTime)lector["actualitzacio"];
                            llista.Add(dia);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion

        #region Verificar dia festiu
        /*
         * Metode per verificar si una data es festiva
         */
        public static bool verificarFestivitat(DateTime data){
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("dataFestiva", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@data",data);
                    Int32 resultat = (Int32) comanda.ExecuteScalar();
                    if(resultat==1){
                        return true;
                    }
                    return false;
                }
            }
        }
        #endregion
    }
}