﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class FullsAccions
    {
        #region Buscar fulls pendents de pagament
        /*
         * Metode que retorna els fulls de despeses pendents de pagament
         */
        public static List<FullDespeses> llistaFullsPendentsPagament()
        {
            List<FullDespeses> llistaFulls = new List<FullDespeses>();
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistarFullsPendentsPagament", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        while (lector.Read())
                        {
                            FullDespeses full = new FullDespeses();
                            full.Id = (Int32)lector["id"];
                            full.DniUsuari = lector["dni"].ToString();
                            full.NomUsuari = lector["nom"].ToString();
                            full.CognomsUsuari = lector["cognoms"].ToString();
                            full.NomCarrec = lector["nomCarrec"].ToString();
                            full.Total = (decimal)lector["total"];
                            full.Solicitat = (DateTime)lector["solicitat"];
                            llistaFulls.Add(full);
                        }
                    }
                }
            }
            return llistaFulls;
        }
        #endregion

        #region Pagar fulls
        /*
        * Metode per fulls de despeses
        */
        public static void pagarFullsDeDespeses(List<Int32> llistaFulls)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("pagarFullDespeses", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            DateTime ara = DateTime.Now;
                            for (int cont = 0; cont < llistaFulls.Count; cont++)
                            {
                                comanda.Parameters.Clear();
                                comanda.Parameters.AddWithValue("@id", llistaFulls[cont]);
                                comanda.Parameters.AddWithValue("@pagat", ara);
                                comanda.Parameters.AddWithValue("@actualitzacio", ara);
                                comanda.ExecuteNonQuery();
                            }
                            transaccio.Commit();
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch
                        {
                            throw;
                        }
                    }
                }
            }
        }
        #endregion

        #region Rebitjar pagament de full
        /*
         * Metode que rebutja el pagament d'un full de despeses
         */
        public static void rebutjarFullDeDespeses(Int32 id, string comentari)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("rebutjarFullDespeses", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            DateTime ara = DateTime.Now;
                            comanda.Parameters.AddWithValue("@idFull", id);
                            comanda.Parameters.AddWithValue("@validat", ara);
                            comanda.Parameters.AddWithValue("@comentari", comentari);
                            comanda.Parameters.AddWithValue("@actualitzacio", ara);
                            comanda.ExecuteNonQuery();
                        }
                        transaccio.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch
                        {
                            throw;
                        }
                    }
                }
            }
        }
        #endregion

        #region Buscar comentari de full
        /*
         * Metode que retorna el comentari d'un full de despeses, donat el seu id
         */
        public static string comentariDeFull(Int32 fullId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarComentariDeFull ", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@fullId", fullId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            string comentari = lector["comentari"].ToString();
                            return comentari;
                        }
                    }
                }
            }
            return string.Empty;
        }
        #endregion

        #region Acceptar pagament de full
        /*
         * Metode per acceptar el pagament de les despeses d'un full,donat el seu id
         */
        public static void acceptarFullDeDespeses(Int32 id)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("acceptarFullDespeses", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            DateTime ara = DateTime.Now;
                            comanda.Parameters.AddWithValue("@idFull", id);
                            comanda.Parameters.AddWithValue("@validat", ara);
                            comanda.Parameters.AddWithValue("@actualitzacio", ara);
                            comanda.ExecuteNonQuery();
                        }
                        transaccio.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch
                        {
                            throw;
                        }
                    }
                }
            }
        }
        #endregion

        #region Buscar full obert d'usuari
        /*
         * Metode que busca el full obert (si existeix) d'un usuari, donat el seu id
         */
        public static FullDespeses buscarFullObert(Int32 usuariId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarFullObertDeUsuari", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@usuariId", usuariId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        FullDespeses fullDespeses = new FullDespeses();
                        if (lector.Read())
                        {
                            fullDespeses.Id = (Int32)lector["id"];
                            fullDespeses.Usuari_Id = (Int32)lector["usuari_id"];
                            fullDespeses.Obert = (DateTime)lector["obert"];
                            fullDespeses.Total = (decimal)lector["total"];
                            fullDespeses.Estat = lector["estat"] as bool?;
                            fullDespeses.Solicitat = lector["solicitat"] as DateTime?;
                            fullDespeses.Validat = lector["validat"] as DateTime?;
                            fullDespeses.Pagat = lector["pagat"] as DateTime?;
                            fullDespeses.Comentari = lector["comentari"].ToString();
                            return fullDespeses;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Obrir un full nou
        /*
         * Metode per obrir un nou full de despeses, donat el id de l'usuari
         */
        public static FullDespeses crearFullNou(Int32 usuariId)
        {
            FullDespeses fullNou = new FullDespeses();
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();

                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("crearFullDespeses", connexio))
                        {

                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            fullNou.Usuari_Id = usuariId;
                            comanda.Parameters.AddWithValue("@usuariId", usuariId);
                            fullNou.Obert = DateTime.Now;
                            comanda.Parameters.AddWithValue("@obert", fullNou.Obert);
                            fullNou.Actualitzacio = DateTime.Now;
                            comanda.Parameters.AddWithValue("@actualitzacio", fullNou.Actualitzacio);
                            fullNou.Id = (Int32)comanda.ExecuteScalar();
                            transaccio.Commit();
                            return fullNou;
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Sol·licitar pagament de full
        /*
         * Metode per sol·licitar el pagament d'un full de despeses, donat el seu id
         */
        public static DateTime solicitarPagamentFullDespeses(Int32 fullId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccion = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("solicitarPagamentFullObert", connexio))
                        {
                            comanda.Transaction = transaccion;
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Parameters.AddWithValue("@fullId", fullId);
                            DateTime solicitat = DateTime.Now;
                            comanda.Parameters.AddWithValue("@solicitat", solicitat);
                            comanda.Parameters.AddWithValue("@actualitzacio", solicitat);
                            comanda.ExecuteNonQuery();
                            transaccion.Commit();
                            return solicitat;
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccion.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Buscar fulls pendents de validar d'un responsable
        /*
         * Metode que retorna el llistat de fulls pendent de validar d'un responsable,donat el seu id
         */
        public static List<FullDespeses> buscarFullsPendentsDeResponsable(Int32 responsableId)
        {
            List<FullDespeses> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarFullsPententsDeResponsable ", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@responsableId", responsableId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<FullDespeses>();
                        while (lector.Read())
                        {
                            FullDespeses fullDespeses = new FullDespeses();
                            fullDespeses.Id = (Int32)lector["id"];
                            fullDespeses.Usuari_Id = (Int32)lector["usuari_id"];
                            fullDespeses.DniUsuari = lector["dni"].ToString();
                            fullDespeses.NomUsuari = lector["nom"].ToString();
                            fullDespeses.CognomsUsuari = lector["cognoms"].ToString();
                            fullDespeses.NomCarrec = lector["nomCarrec"].ToString();
                            fullDespeses.Obert = (DateTime)lector["obert"];
                            fullDespeses.Total = (decimal)lector["total"];
                            fullDespeses.Estat = lector["estat"] as bool?;
                            fullDespeses.Solicitat = lector["solicitat"] as DateTime?;
                            fullDespeses.Validat = lector["validat"] as DateTime?;
                            fullDespeses.Pagat = lector["pagat"] as DateTime?;
                            fullDespeses.Comentari = lector["comentari"].ToString();
                            llista.Add(fullDespeses);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion

        #region Llista de fulls tancats d'un usuari
        /*
         * Metode que busca els fulls tancats d'un usuari,donat el seu id
         */
        public static List<FullDespeses> buscarFullsTancats(Int32 usuariId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("fullsTancatsDeUsuari ", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@usuariId", usuariId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        List<FullDespeses> llistaTancats = new List<FullDespeses>();
                        while (lector.Read())
                        {
                            FullDespeses fullTancat = new FullDespeses();
                            fullTancat.Id = (Int32)lector["id"];
                            fullTancat.Usuari_Id = (Int32)lector["usuari_id"];
                            fullTancat.Obert = (DateTime)lector["obert"];
                            fullTancat.Total = (decimal)lector["total"];
                            fullTancat.Estat = (bool)lector["estat"];
                            fullTancat.Solicitat = (DateTime)lector["solicitat"];
                            fullTancat.Validat = (DateTime)lector["validat"];
                            fullTancat.Pagat = lector["pagat"] as DateTime?;
                            fullTancat.Comentari = lector["comentari"].ToString();
                            llistaTancats.Add(fullTancat);
                        }
                        return llistaTancats;
                    }
                }
            }
        }
        #endregion
    }
}