﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class PermisosAccions
    {
        #region Buscar permisos per id
        /*
         * Metode que retorna els permisos aportats,donat l'id dels permis
         */
        public static Permisos buscarPermisosPerId(Int32 permisosId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarPermisosPerId", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@permisosId", permisosId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            Permisos permisos = new Permisos();
                            permisos.Id = (Int32)lector["id"];
                            permisos.Nom = lector["nom"].ToString();
                            permisos.Administrar = (bool)lector["administrar"];
                            permisos.Validar = (bool)lector["validar"];
                            permisos.VeureUsuaris = lector["veure_usuaris"] as bool?;
                            permisos.VeureDespeses = lector["veure_despeses"] as bool?;
                            return permisos;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Buscar permisos per nom
        /*
         * Metode que retorna els permisos aportats, donat el nom del permis
         */
        public static Permisos buscarPermisPerNom(string nomPermis)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarPermisPerNom", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@nom", nomPermis);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        Permisos permis = new Permisos();
                        if (lector.Read())
                        {
                            permis.Id = (Int32)lector["id"];
                            permis.Nom = nomPermis;
                            permis.Administrar = (bool)lector["administrar"];
                            permis.Validar = (bool)lector["validar"];
                            permis.VeureUsuaris = lector["veure_usuaris"] as bool?;
                            permis.VeureDespeses = lector["veure_despeses"] as bool?;
                            return permis;
                        }
                        return null;
                    }
                }
            }
        }
        #endregion

        #region Llista de permisos
        /*
         * Metode que retorna el llistat de permisos existents
         */
        public static List<Permisos> LlistaDePermisos()
        {
            List<Permisos> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistaDePermisos", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Permisos>();
                        while (lector.Read())
                        {
                            Permisos permisos = new Permisos();
                            permisos.Id = (Int32)lector["id"];
                            permisos.Nom = lector["nom"].ToString();
                            permisos.Administrar = (bool)lector["administrar"];
                            permisos.Validar = (bool)lector["validar"];
                            permisos.VeureUsuaris = lector["veure_usuaris"] as bool?;
                            permisos.VeureDespeses = lector["veure_despeses"] as bool?;
                            llista.Add(permisos);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion
    }
}