﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class TipusAccions
    {
        #region Buscar tipus de despesa per Id
        /*
         * Metode que retorna un tipus de despesa, donat el seu id
         */
        public static Tipus buscarTipusPerId(Int32 tipusId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarTipusPerId", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@tipusId", tipusId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            Tipus tipus = new Tipus();
                            tipus.Id = (Int32)lector["id"];
                            tipus.Nom = lector["nom"].ToString();
                            return tipus;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Afegir tipus de despesa
        /*
         * Metode que afegeix un nou tipus de despesa
         */
        public static Tipus afegirTipus(string nom)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("afegirTipus", connexio))
                        {
                            Tipus tipusNou = new Tipus();
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            tipusNou.Nom = nom;
                            comanda.Parameters.AddWithValue("@nom", tipusNou.Nom);
                            tipusNou.Actualitzacio = DateTime.Now;
                            comanda.Parameters.AddWithValue("@actualitzacio", tipusNou.Actualitzacio);
                            tipusNou.Habilitat = true;
                            tipusNou.Id = (Int32)comanda.ExecuteScalar();
                            transaccio.Commit();
                            return tipusNou;
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Modificar tipus de despesa
        /*
         * Metode per modificar un tipus de despesa existent
         */
        public static void modificarTipus(Int32 id, string nom)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("modificarTipus", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Parameters.AddWithValue("@id", id);
                            comanda.Parameters.AddWithValue("@nom", nom);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                            transaccio.Commit();
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Verificar si un tipus de despesa esta habilitat
        /*
         * Metode que indica si un tipus de despesa esta habilitat,donat el seu id
         */
        public static bool tipusHabilitat(Int32 id)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("tipusHabilitat", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@id", id);
                    Int32 tipusId = (Int32)comanda.ExecuteScalar();
                    if (tipusId == 1)
                    {
                        return true;
                    }
                    return false;
                }
            }
        }
        #endregion

        #region Habiliar tipus de despesa
        /*
         * Metode per habilitar un tipus de despesa, donat el seu id
         */
        public static void habilitarTipus(Int32 id)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("habilitarTipus", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Parameters.AddWithValue("@id", id);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                            transaccio.Commit();
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Deshabilitar tipus de despesa
        /*
         * Metode per deshabilitar un tipus de despesa, donat el seu id
         */
        public static void deshabilitarTipus(Int32 id)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccio = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("deshabilitarTipus", connexio))
                        {
                            comanda.Transaction = transaccio;
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Parameters.AddWithValue("@id", id);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                            transaccio.Commit();
                        }
                    }
                    catch
                    {
                        try
                        {
                            transaccio.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Buscar tipus de despesa per nom
        /*
         * Metode que retorna el nom d'un tipus de despesa,donat el seu id
         */
        public static Tipus buscarTipusPerNom(string nom)
        {
            Tipus tipus = new Tipus();
            tipus.Nom = nom;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarTipusPerNom", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@nom", tipus.Nom);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            tipus.Id = (Int32)lector["id"];
                            tipus.Habilitat = (bool)lector["habilitat"];
                            tipus.Actualitzacio = (DateTime)lector["actualitzacio"];
                            return tipus;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Llista de tipus de despeses habilitades
        /*
         * Metode que retorna un llistat del tipus de despeses habilitats
         */
        public static List<Tipus> llistaDeTipusDisponibles()
        {
            List<Tipus> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistaDeTipusHabilitats", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Tipus>();
                        while (lector.Read())
                        {
                            Tipus tipusDespesa = new Tipus();
                            tipusDespesa.Id = (Int32)lector["id"];
                            tipusDespesa.Nom = lector["nom"].ToString();
                            tipusDespesa.Actualitzacio = (DateTime)lector["actualitzacio"];
                            llista.Add(tipusDespesa);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion

        #region Llista de tipus de despeses existents
        /*
         * Metode que retorna un llistat de tots els tipus de despeses existents
         */
        public static List<Tipus> llistaDeTipus()
        {
            List<Tipus> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistaDeTipusDeDespeses", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Tipus>();
                        while (lector.Read())
                        {
                            Tipus tipusDespesa = new Tipus();
                            tipusDespesa.Id = (Int32)lector["id"];
                            tipusDespesa.Nom = lector["nom"].ToString();
                            tipusDespesa.Habilitat = (bool)lector["habilitat"];
                            llista.Add(tipusDespesa);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion
    }
}