﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;

namespace ServeiSGD.Accions
{
    public class UsuariAccions
    {
        #region Verificar usuari i contrasenya
        /*
         * Metode que indica si un usuari esta autentificat per accedir al sistema.
         */
        public static bool validacioUsuari(string usuari, string contrasenya)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("autentificarUsuari", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@usuari", usuari);
                    comanda.Parameters.AddWithValue("@contrasenya", contrasenya);

                    Int32 resultat = (Int32)comanda.ExecuteScalar();
                    if (resultat == 1)
                    {
                        return true;
                    }
                    return false;
                }
            }
        }
        #endregion

        #region Afegir usuari
        /*
         * Metode que afegeix un nou usuari
         */
        public static Usuari afegirUsuari(string dni, string nom, string cognoms, Int32 carrecId, Int32 responsableId, string contrasenya)
        {
            Usuari usuariNou = new Usuari();
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccion = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("afegirUsuariNou", connexio))
                        {
                            comanda.Transaction = transaccion;
                            comanda.CommandType = CommandType.StoredProcedure;
                            usuariNou.Dni = dni;
                            comanda.Parameters.AddWithValue("@dni", usuariNou.Dni);
                            usuariNou.Nom = nom;
                            comanda.Parameters.AddWithValue("@nom", usuariNou.Nom);
                            usuariNou.Cognoms = cognoms;
                            comanda.Parameters.AddWithValue("@cognoms", usuariNou.Cognoms);
                            usuariNou.IdCarrec = carrecId;
                            comanda.Parameters.AddWithValue("@carrecId", usuariNou.IdCarrec);
                            usuariNou.IdResponsable = responsableId;
                            comanda.Parameters.AddWithValue("@responsableId", usuariNou.IdResponsable);
                            usuariNou.Habilitat = true;
                            usuariNou.Contrasenya = contrasenya;
                            comanda.Parameters.AddWithValue("@contrasenya", contrasenya);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            usuariNou.Id = (Int32)comanda.ExecuteScalar();
                        }
                        transaccion.Commit();
                        return usuariNou;
                    }
                    catch
                    {
                        try
                        {
                            transaccion.Rollback();
                        }
                        catch { }
                        throw; 
                    }
                }
            }
        }
        #endregion

        #region Verificar si usuari esta habilitat
        /*
         * Metode que indica si un usuari esta habilitat o no
         */
        public static bool verificarAccesUsuari(Int32 usuariId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("verificarUsuariHabilitat", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@usuariId", usuariId);
                    Int32 resultat = (Int32)comanda.ExecuteScalar();
                    if (resultat == 1)
                    {
                        return true;
                    }
                    return false;
                }
            }
        }
        #endregion

        #region Habilitar usuari
        /*
         * Metode que habilita un usuari deshabilitat
         */
        public static void habilitarUsuari(Int32 usuariId)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccion = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("habilitarUsuari", connexio))
                        {
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Transaction = transaccion;
                            comanda.Parameters.AddWithValue("@usuariId", usuariId);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                        }
                        transaccion.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccion.Rollback();
                        }
                        catch {}
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Deshabilitar usuari
        /*
         * Metode que deshabilita un usuari habilitat
         */
        public static void deshabilitarUsuari(Int32 idUsuari)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccion = connexio.BeginTransaction())
                {
                    try
                    {
                        using (SqlCommand comanda = new SqlCommand("deshabilitarUsuari", connexio))
                        {
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Transaction = transaccion;
                            comanda.Parameters.AddWithValue("@usuariId", idUsuari);
                            comanda.Parameters.AddWithValue("@actualitzacio", DateTime.Now);
                            comanda.ExecuteNonQuery();
                        }
                        transaccion.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccion.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Modificar usuari
        /*
         * Metode que modifica les dades d'un usuari
         */
        public static void modificarUsuari(Int32 id, string dni, string nom, string cognoms, Int32 carrecId, Int32 responsableId, string contrasenya)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlTransaction transaccion = connexio.BeginTransaction())
                {
                    try
                    {
                        DateTime ara = DateTime.Now;
                        using (SqlCommand comanda = new SqlCommand("modificarUsuari", connexio))
                        {
                            comanda.Transaction = transaccion;
                            comanda.CommandType = CommandType.StoredProcedure;
                            comanda.Parameters.AddWithValue("@id", id);
                            comanda.Parameters.AddWithValue("@dni", dni);
                            comanda.Parameters.AddWithValue("@nom", nom);
                            comanda.Parameters.AddWithValue("@cognoms", cognoms);
                            comanda.Parameters.AddWithValue("@carrecId", carrecId);
                            comanda.Parameters.AddWithValue("@responsableId", responsableId);
                            comanda.Parameters.AddWithValue("@actualitzacio", ara);
                            comanda.ExecuteNonQuery();


                        }
                        if (contrasenya != null && contrasenya.Length > 0)
                        {
                            using (SqlCommand comanda = new SqlCommand("modificarContrasenya", connexio))
                            {
                                comanda.Transaction = transaccion;
                                comanda.CommandType = CommandType.StoredProcedure;
                                comanda.Parameters.AddWithValue("@id", id);
                                comanda.Parameters.AddWithValue("@contrasenya", contrasenya);
                                comanda.Parameters.AddWithValue("@actualitzacio", ara);
                                comanda.ExecuteNonQuery();
                            }
                        }
                        transaccion.Commit();
                    }
                    catch
                    {
                        try
                        {
                            transaccion.Rollback();
                        }
                        catch { }
                        throw;
                    }
                }
            }
        }
        #endregion

        #region Verificar si un DNI existeix
        /*
         * Metode que notifica si existeix un usuari, donat el seu DNI
         */
        public static bool existeixUsuariDNI(string dni)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("existeixUsuari", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@dni", dni);

                    Int32 resultat = (Int32)comanda.ExecuteScalar();
                    if (resultat == 1)
                    {
                        return true;
                    }
                    return false;
                }
            }
        }
        #endregion

        #region Buscar usuari per Id
        /*
         * Metode que retorna un usuari,donat el seu id
         */
        public static Usuari buscarUsuariPerId(Int32 id)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarUsuariPerID", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@id", id);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["dni"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.IdCarrec = (Int32)lector["carrec_id"];
                            usuari.IdResponsable = (Int32)lector["responsable_id"];
                            usuari.Actualitzacio = (DateTime)lector["actualitzacio"];
                            return usuari;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Buscar usuaris per nom
        /*
         * Metode que retorna un llistat amb els usuaris que tenen al nom una determinada paraula
         */
        public static List<Usuari> buscarNomsqueContinguin(string paraula)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("nomQueConte", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@paraula", paraula);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        List<Usuari> llistaUsuaris = new List<Usuari>();
                        while (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["DNI"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.NomCarrec = lector["nomCarrec"].ToString();
                            usuari.DniResponsable = lector["dniResponsable"].ToString();
                            usuari.Habilitat = (bool)lector["habilitat"];
                            llistaUsuaris.Add(usuari);
                        }
                        return llistaUsuaris;
                    }
                }
            }
        }
        #endregion

        #region Buscar usuaris per cognoms
        /*
         * Metode que retorna un llistat amb els usuaris que tenen als cognoms una determinada paraula
         */
        public static List<Usuari> buscarCognomsqueContinguin(string paraula)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("cognomsQueConten", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@paraula", paraula);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        List<Usuari> llistaUsuaris = new List<Usuari>();
                        while (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["DNI"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.NomCarrec = lector["nomCarrec"].ToString();
                            usuari.DniResponsable = lector["dniResponsable"].ToString();
                            usuari.Habilitat = (bool)lector["habilitat"];
                            llistaUsuaris.Add(usuari);
                        }
                        return llistaUsuaris;
                    }
                }
            }
        }
        #endregion

        #region Buscar usuaris per nom i/o cognoms
        /*
         * Metode que retorna un llistat amb els usuaris que tenen al nom o als cognoms unes determinades paraules
         */
        public static List<Usuari> buscarNomCognomsqueContinguin(string paraulaNom, string paraulaCognoms)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("nomsIcognomsQueContenen", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@paraulaNom", paraulaNom);
                    comanda.Parameters.AddWithValue("@paraulaCognoms", paraulaCognoms);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        List<Usuari> llistaUsuaris = new List<Usuari>();
                        while (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["DNI"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.NomCarrec = lector["nomCarrec"].ToString();
                            usuari.DniResponsable = lector["dniResponsable"].ToString();
                            usuari.Habilitat = (bool)lector["habilitat"];
                            llistaUsuaris.Add(usuari);
                        }
                        return llistaUsuaris;
                    }
                }
            }
        }
        #endregion

        #region Buscar usuari per DNI
        /*
         * Metode que retorna un usuari,donat el seu dni
         */
        public static Usuari buscarUsuariPerDNI(string dni)
        {
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarUsuariPerDNI", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@dni", dni);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        if (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["dni"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.IdCarrec = (Int32)lector["carrec_id"];
                            usuari.IdResponsable = (Int32)lector["responsable_id"];
                            usuari.Habilitat = (bool)lector["habilitat"];
                            usuari.Actualitzacio = (DateTime)lector["actualitzacio"];
                            return usuari;
                        }
                    }
                }
            }
            return null;
        }
        #endregion

        #region Buscar usuaris per DNI
        /*
         * Metode que retorna un llistat amb els usuaris que el seu dni conte un cert contingut
         */
        public static List<Usuari> buscarUsuariDNIConte(string paraula)
        {
            List<Usuari> llistaUsuaris = new List<Usuari>();
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("buscarPartDNIUsuari", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@paraula", paraula);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        while (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["dni"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.NomCarrec = lector["nomCarrec"].ToString();
                            usuari.DniResponsable = lector["dniResponsable"].ToString();
                            usuari.Habilitat = (bool)lector["habilitat"];
                            llistaUsuaris.Add(usuari);
                        }
                    }
                }
            }
            return llistaUsuaris;
        }
        #endregion

        #region Buscar treballadors de responsable
        /*
         * Metode que retorna el llistat de treballadors que te un responsable al seu carrec
         */
        public static List<Usuari> usuarisDeResponsable(Int32 responsableId)
        {
            List<Usuari> llistaUsuaris = new List<Usuari>();
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistaUsuarisDeResponsable", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    comanda.Parameters.AddWithValue("@responsableId", responsableId);
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        while (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.NomCarrec = lector["nomCarrec"].ToString();
                            llistaUsuaris.Add(usuari);
                        }
                    }
                }
            }
            return llistaUsuaris;
        }
        #endregion

        #region Llista d'usuaris
        /*
         * Metode que retorna un llistat amb tots els usuaris existents
         */
        public static List<Usuari> llistaDeUsuaris()
        {
            List<Usuari> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistarUsuaris", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Usuari>();
                        while (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["dni"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.NomCarrec = lector["nomCarrec"].ToString();
                            usuari.DniResponsable = lector["dniResponsable"].ToString();
                            usuari.Habilitat = (bool)lector["habilitat"];
                            llista.Add(usuari);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion

        #region Llista d'usuaris habilitats
        /*
         * Metode que retorna un llistat amb tots els usuaris existents
         */
        public static List<Usuari> llistaDeUsuarisDisponibles()
        {
            List<Usuari> llista;
            using (SqlConnection connexio = ConnexioSQL.Connectar())
            {
                connexio.Open();
                using (SqlCommand comanda = new SqlCommand("llistarUsuarisDisponibles", connexio))
                {
                    comanda.CommandType = CommandType.StoredProcedure;
                    using (SqlDataReader lector = comanda.ExecuteReader())
                    {
                        llista = new List<Usuari>();
                        while (lector.Read())
                        {
                            Usuari usuari = new Usuari();
                            usuari.Id = (Int32)lector["id"];
                            usuari.Dni = lector["dni"].ToString();
                            usuari.Nom = lector["nom"].ToString();
                            usuari.Cognoms = lector["cognoms"].ToString();
                            usuari.NomCarrec = lector["nomCarrec"].ToString();
                            usuari.DniResponsable = lector["dniResponsable"].ToString();
                            llista.Add(usuari);
                        }
                    }
                }
            }
            return llista;
        }
        #endregion
    }
}